<?php
/**
*	@file GameEngine.class.php
*	@brief A Class for handling game/simulation functionality
*/

// Maximum number of card drawns when defining next card
defined('ROUNDS_MAX') or define('ROUNDS_MAX', 10000);

/** A Class for handling game/simulation functionality */
class GameEngine {
 	
	/** DB GameEngine::$db Database instance. */
	private $db;
	/** array GameEngine::$game_settings Settings of the game. */
	private $game_settings;
	/** array GameEngine::$all_cards Cards of the game. */
	private $all_cards;
	/** array GameEngine::$played_cards Cards that have been played. */
	private $played_cards;
	/** array GameEngine::$available_cards Cards available for the game. */
	private $available_cards;
	/** array GameEngine::$current_card Information of the card currently being played. */
	private $current_card;
	/** array GameEngine::$previous_card Information of the card previously played. */
	private $previous_card;
	/** int GameEngine::$cancellations Number of rewinding done. */
	private $cancellations;
	/** int GameEngine::$scenario Current scenario. */
	private $scenario;
	
	/** double GameEngine::$game_id ID of the game. */
	public $game_id;
	/** double GameEngine::$profile_id ID of the player profile. */
	public $profile_id;
	/** double GameEngine::$language Language of the game. */
	public $language;
	/** bool GameEngine::$panic ID Tells if something went wrong. */
	public $panic; // is this necessarily public?
	/** array GameEngine::$player_stats Stats of a player. */
	public $player_stats;
	/** bool GameEngine::$game_ended Tells if game ended. */
	public $game_ended;
	/** bool GameEngine::$sudden_scenario Tells if sudden scenario occurded. */
//	public $sudden_scenario; // TODO: Remove because not needed
	
	/** 
	 * Constructor for GameEngine class
	 * @param double $game_id ID of the game.
	 * @param double $profile_id ID of the player profile.
	 * @param string $lang Language of the game.
	 * */
	public function __construct($game_id=0, $profile_id=0, $lang='en-US') {
		$this->game_id = $game_id;
		$this->profile_id = $profile_id;
		$this->language = $lang;
		$this->panic = false; // Flag that informs whether to return panic card
		$this->game_ended = false;
//		$this->sudden_scenario = false;
		$this->cancellations = 0;
		$this->initGame();
	}
	
	/** 
	 * Generates a positive random number.
	 * @param int $min Defines minimum number to generate.
	 * @param int $max Defines maximum number to generate.
	 * @return int Random value between $min and $max.
	 * */
	public static function drawNumber($min=0, $max) {
		
		$lower_limit = $min;
		// Negative values doesn't count
		if ($min < 0) {
			$lower_limit = 0;
		}
		$upper_limit = $max;
		if ($upper_limit < $lower_limit) {
			$upper_limit = $lower_limit;
		}
		$randval = mt_rand($lower_limit, $upper_limit);
		
		return $randval;
	}
	
	/** 
	 * Parses stat string.
	 * @param string $raw_stat Contains the raw stat string.
	 * @return array|false On success an array containing
	 * 'key', 'value', 'operator' and corresponding values.
	 * */
	public static function parseStat($raw_stat) {
		
		$parsed_stat = array();
		$trimmed = trim($raw_stat);
		if (!empty($trimmed)) {
							
			// find if theres space
			$pos = strrpos($trimmed, " ");
			if ($pos !== false) {
				$trimmed = substr($trimmed, 0, $pos);
			}
			$key = '';
			$value = '';
			$operator = '';
			$found = false;
			// Find bigger or equal
			$pos = strrpos($trimmed, ">=");
			if ($pos !== false && !$found) {
				$operator = '>=';
				$key = substr($trimmed, 0, $pos);
				$value = substr($trimmed, $pos+2);
				$found = true;
			}
			// Find smaller or equal
			$pos = strrpos($trimmed, "<=");
			if ($pos !== false && !$found) {
				$operator = '<=';
				$key = substr($trimmed, 0, $pos);
				$value = substr($trimmed, $pos+2);
				$found = true;
			}
			// Find range
			$pos = strrpos($trimmed, "[");
			$pos2 = strrpos($trimmed, "]");
			if (($pos !== false) && ($pos2 !== false) && (!$found)) {
				$operator = '[]';
				$key = substr($trimmed, 0, $pos);
				$value_temp = substr($trimmed, $pos+1);
				$pos2 = strrpos($value_temp, "]");
				$value = substr($value_temp, 0, $pos2);
				$found = true;
			}
			// Find smaller sign
			$pos = strrpos($trimmed, "<");
			if ($pos !== false && !$found) {
				$operator = '<';
				$key = substr($trimmed, 0, $pos);
				$value = substr($trimmed, $pos+1);
				$found = true;
			}
			// Find bigger sign
			$pos = strrpos($trimmed, ">");
			if ($pos !== false && !$found) {
				$operator = '>';
				$key = substr($trimmed, 0, $pos);
				$value = substr($trimmed, $pos+1);
				$found = true;
			}
			// Find equal sign
			$pos = strrpos($trimmed, "=");
			if ($pos !== false && !$found) {
				$operator = '=';
				$key = substr($trimmed, 0, $pos);
				$value = substr($trimmed, $pos+1);
				$found = true;
			}
			$pos = strrpos($trimmed, "+");
			if ($pos !== false && !$found) {
				$operator = '+';
				$key = substr($trimmed, 0, $pos);
				$value = substr($trimmed, $pos+1);
				$found = true;
			}
			$pos = strrpos($trimmed, "-");
			if ($pos !== false && !$found) {
				$operator = '-';
				$key = substr($trimmed, 0, $pos);
				$value = substr($trimmed, $pos+1);
				$found = true;
			}
			$parsed_stat['key'] = $key;
			// remove first @
			$pos = strpos($key, "@");
			if ($pos !== false) {
				$parsed_stat['key'] = substr($key, $pos+1);
			}
			$parsed_stat['value'] = $value;
			$parsed_stat['operator'] = $operator;
		}
		else {
			return false;
		}
		if (empty($parsed_stat['key']) || $parsed_stat['value'] == '' || empty($parsed_stat['operator'])) {
			return false;
		}
		return $parsed_stat;
	}
	
	/** 
	 * Tells if value to compare against stat requirements is true or not.
	 * @param string $parsed_stat Contains stat requirement.
	 * @param string $value_to_compare Contains value that is compared against $parsed_stat.
	 * @return bool Returns whether or not $value_to_compare is valid.
	 * */
	public static function validateStat($parsed_stat, $value_to_compare) {
		
		$is_valid = true;
		
		switch ($parsed_stat['operator']) {
			case '[]':
				$lower_limit = 0;
				$upper_limit = 0;
				$pos = strrpos($parsed_stat['value'], "-");
				if ($pos !== false) {
					$lower_limit = doubleval(substr($parsed_stat['value'], 0, $pos));
					$upper_limit = doubleval(substr($parsed_stat['value'], $pos+1));
				}
				if (((doubleval($value_to_compare) >= $lower_limit) && (doubleval($value_to_compare) <= $upper_limit))) {
					$is_valid = true;
				}
				else {
					$is_valid = false;
				}
				break;
			case '>=':
				if (!(doubleval($value_to_compare) >= doubleval($parsed_stat['value']))) {
					$is_valid = false;
				}
				break;
			case '<=':
				if (!(doubleval($value_to_compare) <= doubleval($parsed_stat['value']))) {
					$is_valid = false;
				}
				break;
			case '>':
				if (!(doubleval($value_to_compare) > doubleval($parsed_stat['value']))) {
					$is_valid = false;
				}
				break;
			case '<':
				if (!(doubleval($value_to_compare) < doubleval($parsed_stat['value']))) {
					$is_valid = false;
				}
				break;
			case '=':
				
				if (is_numeric($parsed_stat['value'])) {//if (is_numeric($value_to_compare)) {//
					if (!(doubleval($value_to_compare) === doubleval($parsed_stat['value']))) {
						$is_valid = false;
					}
				}
				else {
					// Remove the '=' character from value
					if ($value_to_compare[0] === '=') {
						$value_to_compare = substr($value_to_compare, 1);
					}
					if (!($value_to_compare === $parsed_stat['value'])) {
						$is_valid = false;
					}
				}
				break;
			case '+':
				if (!(doubleval($value_to_compare) >= doubleval($parsed_stat['value']))) {
					$is_valid = false;
				}
				break;
			case '-':
				if (!(doubleval($value_to_compare) >= doubleval($parsed_stat['value']))) {
					$is_valid = false;
				}
				break;
			default:
				break;
		}
		return $is_valid;
	}
	
	//TODO: logic brakes when multiple inner or statements
	/** 
	 * Validates given conditions.
	 * @param array $condition_validations Conditions to validate.
	 * @return bool Returns whether or not given conditions are valid.
	 * */
	public static function validateConditions($condition_validations) {
		
		$condition = true;
		$last_index = count($condition_validations) - 1;
		
		// If there's no conditions return false
		if ($last_index < 1) {
			
			if (empty($condition_validations[0]['logic']) && ($condition_validations[0]['valid'] == '')) {
				$condition = false;
			}
		}
		
		// Check if all the ending conditions are met
		for ($i = 0; $i < count($condition_validations); $i++) {
			
			if ($i < $last_index) {
				
				if (($condition_validations[$i]['logic'] == 'AND') && ($condition_validations[$i+1]['logic'] != 'OR')) {
					if (!($condition_validations[$i]['valid'] && $condition_validations[$i+1]['valid'])) {
						$condition = false;
					}
				}
				elseif (($condition_validations[$i]['logic'] == 'AND') && ($condition_validations[$i+1]['logic'] == 'OR')) {
					
					if (!$condition_validations[$i]['valid']) {
						$condition = false;
					}
					else {
						$or_statement_valid = false;
						// Check if atleast one OR statement is true
						for ($j = $i+1; $j < $last_index; $j++) {
							
							if ($condition_validations[$j]['logic'] == 'AND') {
								$i = $j;
								break;
							}
							elseif ($condition_validations[($j)]['valid'] || $condition_validations[($j+1)]['valid']) {
								$or_statement_valid = true;
							}
							$i = $j;
						}
						if (!$or_statement_valid ) {
							$condition = false;
						}
					}
				}
				elseif ($condition_validations[$i]['logic'] == 'OR') {
					$or_statement_valid = false;
					// Check if atleast one OR statement is true
					for ($j = $i; $j < $last_index; $j++) {
						
						if ($condition_validations[$j]['logic'] == 'AND') {
							$i = $j;
							break;
						}
						elseif ($condition_validations[($j)]['valid'] || $condition_validations[($j+1)]['valid']) {
							$or_statement_valid = true;
						}
						$i = $j;
					}
					if (!$or_statement_valid) {
						$condition = false;
					}
				}
			}
		}
		return $condition;
	}
	
	/** 
	 * A function to retrieve the card that is currently being played.
	 * @return array Returns an array of current card's or in case of panic
	 * panic card's information.
	 * */
	public function getCurrentCard() {
		
		// If panic flag is set return panic card
		if ($this->panic) {
			return $this->getPanicCard();
		}
		return $this->current_card;
	}
	
	/** 
	 * A function to retrieve the previous card that has played.
	 * @return array Returns an array of previous card's information or
	 * an empty array in case of no cards is played before.
	 * */
	public function getPrevCard() {
		
		$played_cards_count = count($this->played_cards);
		// If no cards have been played return empty array
		if ($played_cards_count < 2) {
			return array();
		}
		// Return second last card of played cards array
		return $this->previous_card;
	}
	
	/** 
	 * A function to get a result for the choice player has made and play a new card based on that.
	 * @param double $choice_id ID of a choice.
	 * @return array Returns an array containing result's information.
	 * */
	public function getCardResult($choice_id) {
		
		$this->db = DB::getInstance();
		
		$choice = 0;
		if ($choice_id > 0) {
			$choice = $choice_id;
		}
		$results = $this->db->getChoiceResults($choice, $this->language);

		// Get random number
		$randval = GameEngine::drawNumber(1,100);

		$lower_limit = 0; // percent
		$upper_limit = 0;
		$result = array();
	
		foreach ($results as $res) {
			$lower_limit = $upper_limit;
			$upper_limit = $upper_limit + intval($res['weight']);
			
			if (($lower_limit < $randval) && ($randval <= $upper_limit)) {
				$result = $res;
				break;
			}
		}
		
		$latest_card = end($this->played_cards);
		
		if (empty($result)) {
			$result['result_id'] = 0;
		}
		
		//if (!empty($result)) {
			
			$this->db->updatePlayedCard($this->profile_id, $latest_card['card_id'], $choice_id, $result['result_id'], count($this->played_cards));
			
			// update played card on private array too
			$latest_card['choice_id'] = $choice_id;
			$latest_card['result_id'] = $result['result_id'];
			$this->updatePlayerStats($result);
		//}
		 
		// If last card didn't had results then use second latest cards criteria and so on
		// until find some
		if ($result['result_id'] == 0) {
			
			$last_index = count($this->played_cards) - 1;
			$result['next_criteria'] = '';
			
			for ($i = $last_index; $i >= 0; ($i = ($i-1))) {
				if ($this->played_cards[$i]['choice_id'] > 0 && $this->played_cards[$i]['result_id'] > 0) {
					
					$old_result = $this->db->getResult($this->played_cards[$i]['result_id']);
					$result['next_criteria'] = $old_result['next_criteria'];
					break;
				}
			}
		}
		$this->playCard($result['next_criteria']);
		return $result;
	}
	
	/** 
	 * Rewinds one step back to previous situation if rewinding is allowed.
	 * @return void
	 * */
	public function rewind() {
		
		$this->db = DB::getInstance();
		
		if (!$this->countPlayedCards()) {
			return;
		}
		
		if ($this->game_settings['max_cancellations'] != -1) {
			// Check if player has cancellations left
			if ($this->game_settings['max_cancellations'] <= $this->cancellations) {
				return;
			}
		}
		// If player was in ending screen and there are no ending card and wants to go back to previous card
		if (!($this->game_ended && $this->panic)) {
			
			// Remove current card from played cards
			$this->db->removePlayedCard($this->profile_id, $this->current_card['card_id'], count($this->played_cards));
			if ($this->countPlayedCards() > 1) {
				// Reset choice and result of previous card
				$this->db->updatePlayedCard($this->profile_id, $this->previous_card['card_id'], 0, 0, count($this->played_cards) - 1);
			}
		}
		// Update cancellations
		$this->db->updatePlayerProfileCancellations($this->profile_id, $this->cancellations + 1);
		// Reset values
		if ($this->panic) {
			$this->db->updatePlayerProfilePanic($this->profile_id, 0);
		}
		$this->panic = false;
		if ($this->game_ended) {
			$this->db->updatePlayerProfileGameEnded($this->profile_id, 0);
		}
		$this->game_ended = false;
//		$this->sudden_scenario = false;
		$this->initGame();
	}
	
	/** 
	 * A function to get number of played cards.
	 * @return int Returns count of cards played.
	 * */
	public function countPlayedCards() {
		
		return count($this->played_cards);
	}
	
	/** 
	 * Tells if rewinding is allowed.
	 * @return bool Rewinding allowed.
	 * */
	public function allowCancellation() {
		
		$cancellations_left = true;
		
		if ($this->game_settings['max_cancellations'] != -1) {
			// Check if player has cancellations left
			if ($this->game_settings['max_cancellations'] <= $this->cancellations) {
				$cancellations_left = false;
			}
		}
		// If current card is the first one don't allow to cancel
		if (count($this->played_cards) < 2) {
			$cancellations_left = false;
		}
		return $cancellations_left;
	}
	
	/** 
	 * Initializes game to it's starting state.
	 * @return void
	 * */
	private function initGame() {
		
		$this->db = DB::getInstance();
		$this->game_settings = $this->db->getGame($this->game_id);
		$this->all_cards = $this->db->getAllCards($this->game_id, $this->language, false, true);
		$this->played_cards = $this->db->getPlayedCards($this->profile_id);
		$this->available_cards = $this->getAvailableCards($this->all_cards, $this->played_cards);
		
		$profile = $this->db->getProfile($this->profile_id);
   	
		if ($profile) {
			
			$this->cancellations = $profile['cancellations'];
			$this->game_ended = $profile['game_ended'];
			$this->panic = $profile['is_panic'];
			
			$scenario = 0;
			if (isset($profile['scenario'])) {
				$scenario = $profile['scenario'];
			}
			$this->scenario = $scenario;
		}
		$this->player_stats = $this->getPlayerStats($this->played_cards);
		
		$played_cards_count = count($this->played_cards);

		// First card?
		if ($played_cards_count == 0) {

			$next_card_criteria = '';
			if (isset($this->game_settings['starting_card_id']) && $this->game_settings['starting_card_id'] > 0) {
				$next_card_criteria = 'ID' . $this->game_settings['starting_card_id'];
			}
			$this->playCard($next_card_criteria);
		}
		else {
			
			$latest_card = end($this->played_cards);
			$this->current_card = $this->db->getCard($latest_card['card_id'], $this->language);

			if ($played_cards_count > 1) {
				$this->previous_card = $this->db->getCard($this->played_cards[$played_cards_count - 2]['card_id'], $this->language);
			}
			else {
				$this->previous_card = array();
			}
		}
	}
	
	/** 
	 * A function to get cards that are available for playing.
	 * @param array $all_cards An array of all cards of the game.
	 * @param array $played_cards An array of played cards.
	 * @return array Returns an array containing playable cards.
	 * */
	private function getAvailableCards(&$all_cards, &$played_cards) {
		
		$this->db = DB::getInstance();
		
		$available_cards = $all_cards;
		$index = 0;
		// First remove disabled cards
		foreach ($available_cards as $card) {
			if (!$card['enabled']) {
				array_splice($available_cards, $index, 1);
				$index--;
			}
			$index++;
		}
		unset($card);
		
		foreach ($played_cards as $p_card) {
			
			$card_info = $this->db->getCard($p_card['card_id'], $this->language);
			// If played card isn't repetitive remove it from available cards
			if (!$card_info['repetitive']) {
				for ($i = 0; $i < count($available_cards); $i++) {
				
					if ($card_info['card_id'] == $available_cards[$i]['card_id']) {
						array_splice($available_cards, $i, 1);
						break;
					}
				}
			}
		}
		unset($p_card);
		
		return $available_cards;
	}
	
	/** 
	 * A function to get panic card.
	 * @return array Returns an array containing panic card's information
	 * or an empty array if panic card is not found. If not found sets the
	 * GameEngine::$game_ended to true. 
	 * */
	private function getPanicCard() {
		
		$panic_card = array();
		if ($this->game_settings['panic_card_id'] > 0) {
			$this->db = DB::getInstance();
			$panic_card = $this->db->getCard($this->game_settings['panic_card_id'], $this->language);
		}
		if (empty($panic_card)) {
			// End simulation
			$this->game_ended = true;
		}
		return $panic_card;
	}
	
	/** 
	 * Gets the next card to be played.
	 * @param string $next_card_criteria Criteria for getting next card based on tags.
	 * @return int Returns an index of a next card. If not found index is -1
	 * */
	private function drawNextCard($next_card_criteria) {
		
		// If game has already ended then set game_ended flag to be false
		if ($this->game_ended) {
			$this->db->updatePlayerProfileGameEnded($this->profile_id, 0);
		}
		$this->game_ended = false;
//		$this->sudden_scenario = false;
		// If there was a panic situation before then set panic flag to be false
		if ($this->panic) {
			$this->db->updatePlayerProfilePanic($this->profile_id, 0);
		}
		$this->panic = false;
		
		// Tells whether or not suitable card is drawn
		$found = false;
		
		$next_card_index = -1;
		
		// First check if simulation ends
		if ($this->game_settings['use_ending_criteria']) {
			$next_card_index = $this->doesSimulationEnd($this->player_stats);
			if ($next_card_index >= 0) {
				$found = true;
			}
			else {
				$found = false;
			}
		}
		// if not, check if sudden scenario occurs
		if (!$found) {
			if ($this->game_settings['use_sudden_scenarios']) {
				// Check if sudden scenario occurs
				$next_card_index = $this->doesSuddenScenarioOccur($this->player_stats);
				if ($next_card_index >= 0) {
					$found = true;
				}
				else {
					$found = false;
				}
			}
		}
		if (!$found) {
			$next_card_index = $this->getValidCard($next_card_criteria);
		}
		return $next_card_index;
	}
	
	/** 
	 * Gets next card to be played based on given criteria and available cards
	 * @param string $criteria Criteria for getting next card based on tags.
	 * @return int Returns an index of a next card. If not found index is -1
	 * */
	private function getValidCard($criteria) {
		
		$next_card_index = -1;
		$found = false;
		$rounds = 0;
		//$upper_limit = count($this->available_cards) - 1;
				
		// Cards available for next card. Every card that doesn't match for requirements is removed from this array
		$cards_available_temp = $this->available_cards;
		// Upper limit for random number draw
		$upper_limit = count($cards_available_temp) - 1;
		
		// If no cards available return false
		if ($upper_limit < 0) {
			return -1;
		}
		$used_indexes = array();

		while (!$found) {

			// Draw numbers until correct index is found
			do {
				$next_card_index = GameEngine::drawNumber(0,$upper_limit);
			} while (in_array($next_card_index, $used_indexes) || !isset($cards_available_temp[$next_card_index]['card_id']) || count($used_indexes) == $upper_limit+1);
			$used_indexes[] = $next_card_index;

			if (doubleval($this->game_settings['panic_card_id']) != doubleval($cards_available_temp[$next_card_index]['card_id'])) {
			//if (doubleval($this->game_settings['panic_card_id']) != doubleval($this->available_cards[$next_card_index]['card_id'])) {
				
				$found = $this->validateCard($next_card_index, $criteria);

				// If card doesn't match the criteria then remove it from cards to drawn
				if (!$found) {
					unset($cards_available_temp[$next_card_index]);
					
					// Update upper limit for random number draw
					//$upper_limit = count($cards_available_temp) - 1;
					
					// If no cards left to draw return false
					if ($upper_limit < 0) {
						return -1;
					}
				}
			}
			$rounds++;
			if ($rounds > ROUNDS_MAX) {// && !found) {
				$next_card_index = -1;
				break;
			}
			if (!$found && count($used_indexes) == $upper_limit+1) {
				$next_card_index = -1;
				break;
			}
		}
		return $next_card_index;
	}
	
	/** 
	 * Plays the next card.
	 * @param string $next_card_criteria Criteria for getting next card based on tags.
	 * @return void
	 * */
	private function playCard($next_card_criteria) {
		
		$this->db = DB::getInstance();
		
		$played_cards_count = count($this->played_cards);
		
		$next_card_index = $this->drawNextCard($next_card_criteria);
		
		if ($next_card_index >= 0) {

			if ($this->db->playCard($this->profile_id, $this->available_cards[$next_card_index]['card_id'], $played_cards_count + 1)) {

				$this->played_cards = $this->db->getPlayedCards($this->profile_id);
				$played_cards_count++;
				//$this->current_card = $this->available_cards[$next_card_index];
				$this->current_card = $this->db->getCard($this->available_cards[$next_card_index]['card_id'], $this->language);
				
				// Remove played cards that can't come again
				if (!$this->current_card['repetitive']) {
					array_splice($this->available_cards, $next_card_index, 1);
				}
				if ($this->panic) {
					$this->db->updatePlayerProfilePanic($this->profile_id, 0);
				}
				$this->panic = false;
			}
			else {
				$this->panic = true;
				$this->db->updatePlayerProfilePanic($this->profile_id, 1);
				//if panic card hasn't been defined end game
				if ($this->game_settings['panic_card_id'] == 0) {
					$this->game_ended = true;
					$this->db->updatePlayerProfileGameEnded($this->profile_id, 1);
				}
				$this->current_card = $this->getPanicCard();
			}
		}
		else {
			$this->panic = true;
			$this->db->updatePlayerProfilePanic($this->profile_id, 1);
			//if panic card hasn't been defined end game
			if ($this->game_settings['panic_card_id'] == 0) {
				$this->game_ended = true;
				$this->db->updatePlayerProfileGameEnded($this->profile_id, 1);
			}
			$this->current_card = $this->getPanicCard();
		}
		if ($played_cards_count > 1) {
			$this->previous_card = $this->db->getCard($this->played_cards[$played_cards_count - 2]['card_id'], $this->language);
		}
		else {
			$this->previous_card = array();
		}
	}
	
	/** 
	 * Updates player's stats based on result.
	 * @param array $result Result which 'stat_changes' to add to player's stats
	 * @return void
	 * */
	private function updatePlayerStats($result) {
		
		if (!isset($result['stat_changes']) || empty($result['stat_changes'])) {
			return;
		}
		
		$stat_changes = explode('@', $result['stat_changes']);
				
		foreach ($stat_changes as $stat) {
						
			$parsed_stat = GameEngine::parseStat($stat);
			if ($parsed_stat != false) {
				// If stat already exists
				if (isset($this->player_stats[$parsed_stat['key']])) {
					switch ($parsed_stat['operator']) {
						case '=':
							$this->player_stats[$parsed_stat['key']] = $parsed_stat['operator'] . $parsed_stat['value'];
							break;
						case '+':
							$number = doubleval($this->player_stats[$parsed_stat['key']]);
							$number = $number + doubleval($parsed_stat['value']);
							$this->player_stats[$parsed_stat['key']] = $number;
							break;
						case '-':
							$number = doubleval($this->player_stats[$parsed_stat['key']]);
							$number = $number - doubleval($parsed_stat['value']);
							$this->player_stats[$parsed_stat['key']] = $number;
							break;
						default:
							break;
					}
				}
				else {
					if ($parsed_stat['operator'] === '=') {
						$this->player_stats[$parsed_stat['key']] = $parsed_stat['operator'] . $parsed_stat['value'];
					}
					else {
						// Check if stat operator is minus
						if ($parsed_stat['operator'] === '-') {
							$this->player_stats[$parsed_stat['key']] = -1 * $parsed_stat['value'];
						}
						else {
							$this->player_stats[$parsed_stat['key']] = $parsed_stat['value'];
						}
					}
				}
			}
		}
	}
	
	/** 
	 * Retrieves player's current stats.
	 * @return array An array containing player's stats.
	 * */
	private function getPlayerStats() {
		
		$this->db = DB::getInstance();
		$stats = array();
		
		// Set scenario stat
		if (isset($this->scenario)) {
			$stats['current_scenario'] = '=' . $this->scenario;
		}
	
		foreach($this->played_cards as $card) {
		
			if ($card['choice_id'] > 0) {
			
				$result = $this->db->getResult($card['result_id']);
				if (!empty($result)) {
				
					$data_array = explode('@', $result['stat_changes']);
				
					foreach ($data_array as $stat) {
						
						$parsed_stat = GameEngine::parseStat($stat);
						if ($parsed_stat != false) {
							// If stat already exists
							if (isset($stats[$parsed_stat['key']])) {
								switch ($parsed_stat['operator']) {
									case '=':
										$stats[$parsed_stat['key']] = $parsed_stat['operator'] . $parsed_stat['value'];
										break;
									case '+':
										$number = doubleval($stats[$parsed_stat['key']]);
										$number = $number + doubleval($parsed_stat['value']);
										$stats[$parsed_stat['key']] = $number;
										break;
									case '-':
										$number = doubleval($stats[$parsed_stat['key']]);
										$number = $number - doubleval($parsed_stat['value']);
										$stats[$parsed_stat['key']] = $number;
										break;
									default:
										break;
								}
							}
							else {
								if ($parsed_stat['operator'] === '=') {
									$stats[$parsed_stat['key']] = $parsed_stat['operator'] . $parsed_stat['value'];
								}
								else {
									// Check if stat operator is minus
									if ($parsed_stat['operator'] === '-') {
										$stats[$parsed_stat['key']] = -1 * $parsed_stat['value'];
									}
									else {
										$stats[$parsed_stat['key']] = $parsed_stat['value'];
									}
								}
							}
						}
					}
				}
			}
		}
		return $stats;
	}
	
	/** 
	 * Tells if stats fulfills cards requirements.
	 * @param array $next_card An array of card information.
	 * @param array $stats Contains an array of stats.
	 * @return bool Returns whether or not $stats are valid for card.
	 * */
	private function validateStats($next_card, $stats) {
		
		if (!isset($next_card['stat_requirements'])) {
			return false;
		}
		$card = $next_card;
		$stat_requirements = $card['stat_requirements'];

		$is_valid = true;
		
		foreach ($stat_requirements as $stat) {
						
			$parsed_stat = GameEngine::parseStat($stat);

			if ($parsed_stat !== false) {
				
				$value_to_compare;
				// If stat exists
				if (isset($stats[$parsed_stat['key']])) {
					$value_to_compare = $stats[$parsed_stat['key']];
				}
				else {
					$value_to_compare = 0;
				}
				$is_valid = GameEngine::validateStat($parsed_stat, $value_to_compare);
				if (!$is_valid) {
					break;
				}
			}
			else {
				$is_valid = false;
			}
		}
		return $is_valid;
	}
	
	/** 
	 * Check if simulation is ending based on ending criteria.
	 * @param array $player_stats Contains stats of a player.
	 * @return int Index of next card. -1 if not found.
	 * */
	private function doesSimulationEnd($player_stats) {
		
		$next_card_index = -1;
		
		$ending_criteria = array();
		
		if (isset($this->game_settings['ending_criteria'])) {
			$ending_criteria = $this->game_settings['ending_criteria'];
		}
		
		$ending = true;
		
		foreach ($ending_criteria as $criteria) {
			
			$ending_validations = array();
			
			for ($i = 0; $i < count($criteria['conditions']); $i++) {
				
				$parsed_stat = GameEngine::parseStat($criteria['conditions'][$i]['condition_text']);
				$ending_validations[$i]['logic'] = $criteria['conditions'][$i]['condition_logic'];
				$ending_validations[$i]['valid'] = false;
				
				if ($parsed_stat !== false) {
				
					$value_to_compare;
					
					if (isset($player_stats[$parsed_stat['key']])) {
						$value_to_compare = $player_stats[$parsed_stat['key']];
					}
					else {
						$value_to_compare = 0;
					}
					$ending_validations[$i]['valid'] = GameEngine::validateStat($parsed_stat, $value_to_compare);
				}
			}
			$ending = GameEngine::validateConditions($ending_validations);
			if ($ending) {
				
				$this->game_ended = true;
				$this->db->updatePlayerProfileGameEnded($this->profile_id, 1);
				
				$next_card_index = $this->getValidCard($criteria['result']);
				break;
			}
		}
		return $next_card_index;
	}
	
	/** 
	 * Check if sudden scenario occurs.
	 * @param array $player_stats Contains stats of a player.
	 * @return int Index of next card. -1 if not found.
	 * */
	private function doesSuddenScenarioOccur($player_stats) {
		
		$next_card_index = -1;
		$rand_number = GameEngine::drawNumber(0, 100);
		$sudden_scenarios = array();
		
		if (isset($this->game_settings['sudden_scenarios'])) {
			$sudden_scenarios = $this->game_settings['sudden_scenarios'];
			shuffle($sudden_scenarios);
		}
		
		foreach ($sudden_scenarios as $scenario) {
			
			$scenario_valid = true;
			$scenario_validations = array();
			
			for ($i = 0; $i < count($scenario['conditions']); $i++) {
				
				$parsed_stat = GameEngine::parseStat($scenario['conditions'][$i]['condition_text']);
				
				$scenario_validations[$i]['logic'] = $scenario['conditions'][$i]['condition_logic'];
				$scenario_validations[$i]['valid'] = false;
				
				if ($parsed_stat != false) {
				
					$value_to_compare;
					// If stat exists
					if (isset($player_stats[$parsed_stat['key']])) {
						$value_to_compare = $player_stats[$parsed_stat['key']];
					}
					else {
						$value_to_compare = 0;
					}
					$scenario_validations[$i]['valid'] = GameEngine::validateStat($parsed_stat, $value_to_compare);
				}
			}
			$scenario_valid = GameEngine::validateConditions($scenario_validations);
			
			if ($scenario_valid) {
				// Check sudden scenario happens
				if (doubleval($scenario['odds']) >= $rand_number) {
					$next_card_index = $this->getValidCard($scenario['result']);
					if ($next_card_index > -1) {
//					$this->sudden_scenario = true;
					break;
					}
				}
			}
		}
		return $next_card_index;
	}
	
	/** 
	 * Check if card at index meet the criteria.
	 * @param int $card_index Index of a card that needs to validate
	 * @param string $next_card_criteria Requirements for the card.
	 * @return bool Return if card is valid.
	 * */
	private function validateCard($card_index, $next_card_criteria) {
		
		$valid = true;
		$tags_ok = true;
		$stats_ok = true;
		if (!isset($this->available_cards[$card_index])) {
			return false;
		}
		
		$next_card = $this->available_cards[$card_index];
		
		// Check if specific id is set
		$trimmed_criteria = trim($next_card_criteria);
		$pos = strrpos($trimmed_criteria, "ID");
		if ($pos !== false) {
			$id = substr($trimmed_criteria, $pos+2);
			// If found id matches with given card, return true
			if (doubleval($next_card['card_external_id']) == doubleval($id)) {
				$valid = true;
			}
			else {
				$valid = false;
			}
		}
		else {
			
			// If card is denied to come in any other situation than when it's id is given
			// mark those cards to invalid
			if ($next_card['hidden']) {
				return false;
			}
			
			if ($this->game_settings['use_tags']) {
				// check if correct tag	
				if ($this->validateTags($next_card, $next_card_criteria)) {
					$valid = true;
				}
				else {
					$valid = false;
					$tags_ok = false;
				}
			}
			if ($this->game_settings['use_stats']) {
				
				// check if player stats meet the requirements of a card
				if ($this->validateStats($next_card, $this->player_stats)) {
					$valid = true;
				}
				else {
					$valid = false;
					$stats_ok = false;
				}
			}
			if (!$stats_ok || !$tags_ok) {
				$valid = false;
			}
		}
		return $valid;
	}
	
	/** 
	 * Check if card's tags meet the criteria.
	 * @param int $card An array of a cards information
	 * @param string $next_card_criteria Requirements for the card.
	 * @return bool Return if card is valid.
	 * */
	private function validateTags($card, $next_card_criteria) {
		
		$trimmed_criteria = trim($next_card_criteria);
		// If next card criteria is empty every card is valid
		if (empty($trimmed_criteria)) {
			return true;
		}
		// Check if specific id is set
		$pos = strrpos($trimmed_criteria, "ID");
		if ($pos !== false) {
			$id = substr($trimmed_criteria, $pos+2);
			if (is_numeric($id)) {
				// If found id matches with given card, return true
				if (doubleval($card['card_external_id']) == doubleval($id)) {
						return true;
				}
			}
		}
		$card_tags = array();
		if (isset($card['tags'])) {
			$card_tags = $card['tags'];
		}
		// Card doesn't have any tags so card can't be valid because next card criteria isn't empty
		if (empty($card_tags)) {
			return false;
		}
		// First find and validate tags that card must have
		$must_have_tags = array();
		$pos = strpos($trimmed_criteria, "("); // find first bracket
		
		if ($pos !== false) {
			// find last occurrence of closing bracket
			$temp = substr($trimmed_criteria, $pos+1);
			$pos2 = strrpos($temp, ")");
			
			if ($pos2 !== false) {
				
				$sentence = substr($temp, 0, $pos2);
				// Get all tags in array
				$data_array = explode('#', $sentence);
				// and store tags to must_have_tags array removing empty spaces
				foreach ($data_array as $tag) {
					if (!empty($tag)) {
						$must_have_tags[] = trim($tag);
					}
				}
				// Remove processed string from next_card_criteria
				$trimmed_criteria = trim(str_replace($sentence, '', $trimmed_criteria));
				$trimmed_criteria = trim(str_replace("()", '', $trimmed_criteria));
			}
		}
		if (!empty($must_have_tags)) {
			foreach ($must_have_tags as $must) {
				if(!in_array($must, $card_tags)) {
					return false;
				}
			}
		}
		// Then find and validate tags that card can't have
		$cant_have_tags = array();
		$pos = strpos($trimmed_criteria, "-#");
		
		// If tags are found
		if ($pos !== false) {
			
			$data_array = explode("-#", $trimmed_criteria);
			
			// store tags to cant_have_tags array removing empty spaces
			foreach ($data_array as $tag) {
				
				if (!empty($tag)) {
					$cleaned_tag = trim($tag);
					// Check that there are no other tags in same cell
					$pos = strpos($cleaned_tag, "#"); // find first bracket
					
					if ($pos !== false) {
						// If other tags are found, remove them from tag
						$cleaned_tag = substr($cleaned_tag, 0, $pos);
					}
					// Remove processed tags from trimmed_criteria
					$trimmed_criteria = trim(str_replace($cleaned_tag, '', $trimmed_criteria));
					$cant_have_tags[] = trim($cleaned_tag);
				}
			}
			// Remove left over negation tags
			$trimmed_criteria = trim(str_replace("-#", '', $trimmed_criteria));
		}
		
		if (!empty($cant_have_tags)) {
			foreach ($cant_have_tags as $cant) {
				if(in_array($cant, $card_tags)) {
					return false;
				}
			}
		}
		
		// Finally find and validate "optional" tags from left over criteria
		// Card needs to have atleast one of these
		if (!empty($trimmed_criteria)) {
			
			$can_have_tags = array();
			$pos = strpos($trimmed_criteria, "#");
			// If tags are found
			if ($pos !== false) {
				
				$data_array = explode("#", $trimmed_criteria);
				// store tags to can_have_tags array removing empty spaces
				foreach ($data_array as $tag) {
					if (!empty($tag)) {
						$cleaned_tag = trim($tag);
						// Check that there are no other tags in same cell
						$pos = strpos($cleaned_tag, "#"); // find first bracket
						if ($pos !== false) {
							// If other tags are found, remove them from tag
							$cleaned_tag = substr($cleaned_tag, 0, $pos);
						}
						// Remove processed tags from trimmed_criteria
						$trimmed_criteria = trim(str_replace($cleaned_tag, '', $trimmed_criteria));
						$can_have_tags[] = trim($cleaned_tag);
					}
				}
				// Remove left over tag marks
				$trimmed_criteria = trim(str_replace("#", '', $trimmed_criteria));
			}
			$can_have_tags_is_valid = false;
		
			if (!empty($can_have_tags)) {
				foreach ($can_have_tags as $can) {
					if(in_array($can, $card_tags)) {
						$can_have_tags_is_valid = true;
					}
				}
			}
			if (!$can_have_tags_is_valid) {
				return false;
			}
		}
		return true;
	}
}