<?php
/**
*	@file XMLDB.class.php
*	@brief A Class for handling XML based "database" operations
*	Concurrency related things might need some fixing.
*/

/** 
 * A Class for handling XML based "database" operations.
 * Needs to be inherited by class DB in order to password related stuff to work.
 */
class XMLDB {
	/** XMLDB $instance of XML database class. */
	private static $instance = null;
	/** string XMLDB::$xml_folder Name of the folder which contains XML files */
	private $xml_folder;
	/** array XMLDB::$games All games in GAL2. */
	private $games;
	/** array XMLDB::$users All users in GAL2. */
	private $users;
	// Below are the game specific values
	/** array XMLDB::$cards All cards of active game. */
	private $cards;
	/** array XMLDB::$profiles All player profiles of active game. */
	private $profiles;
	/** array XMLDB::$current_profile Profile currently active. */
	private $current_profile;
	/** double XMLDB::$current_game Game currently active. */
	private $current_game;
	/** array XMLDB::$latest_card Latest card used. */
	private $latest_card;
	/** array XMLDB::$latest_ids Latest used ids for auto increment to work. */
	private $latest_ids;
	/** array XMLDB::$texts Texts of the GAL2. */
	private $texts;
	
	// If true creates debug log file
	/** 
	 * bool XMLDB::$debug Tells if we are debugging this code.
	 * If true will create log file for debugging.
	 */
	private $debug;

	private function __construct() {
		
		$config = Config::getInstance();
		if (!is_dir(dirname(__FILE__) . '/../../' . $config->db_xml_folder)) {
			echo (localize('ERROR_XML_FOLDER_NOT_FOUND') . PHP_EOL);
			exit;
		}
		$this->xml_folder = dirname(__FILE__) . '/../../' . $config->db_xml_folder;
		$this->debug = false;
		$this->initDB();
	}
	
	/**
	 * Gets the only instance of the database
    *
    * @return XMLDB Database object
   */
	public static function getInstance() {
		if (is_null(self::$instance)) {
			self::$instance = new XMLDB();
		}
		return self::$instance;
	}
	
	/**
	 * Retrieves information of all games.
    * @return array Containing information of all games.
   */
	public function getAllGames() {
		
		if ($this->debug) {
			$this->writeDebug('getAllGames', '');
		}
		if (empty($this->games)) {
			$this->initDB();
		}
		
		return $this->games;
	}
	
	/**
	 * Retrieves information of games user has played.
    * @param double $user_id ID of the user.
    * @return array Containing user's games.
   */
	public function getUsersGames($user_id) {
		
		if ($this->debug) {
			$this->writeDebug('getUsersGames', '');
		}
		if (empty($this->games)) {
			$this->initDB();
		}
		$users_games = array();
		
		foreach($this->games as $game) {
			
			if (is_dir($this->xml_folder . 'games/' . $game['game_id'])) {
				
				if (is_dir($this->xml_folder . 'games/' . $game['game_id'] . '/player_profiles')) {
					
					for ($i = 1; $i <= $this->latest_ids['profile_id']; $i++) {
						
						if (is_file($this->xml_folder . 'games/' . $game['game_id'] . '/player_profiles/profile_' . $i . '.xml')) {
					
							$profile = $this->readXMLProfile($this->xml_folder . 'games/' . $game['game_id'] . '/player_profiles/profile_' . $i . '.xml');
					
							if ($profile['user_id'] == $user_id) {

								$new_game = array();
								$new_game['game_id'] = $game['game_id'];
								$new_game['name'] = $game['name'];
								$new_game['max_cancellations'] = $game['max_cancellations'];
								$new_game['profile_id'] = $profile['profile_id'];
								$new_game['profile_created'] = $profile['created'];
								$new_game['cancellations'] = $profile['cancellations'];
								$new_game['is_panic'] = $profile['is_panic'];
								$new_game['game_ended'] = $profile['game_ended'];
								$new_game['played_cards'] = count($profile['played_cards']);
								$users_games[] = $new_game;
								
							}
						}
					}
				}
			}
		}
		return $users_games;
	}
	
	/**
	 * Retrieves all user ids of administrators of the game.
    * @param double $game_id ID of the game of which administrators to retrieve.
    * @return array An array of admin ids.
   */
	public function getAdminIDs($game_id) {
		
		if ($this->debug) {
			$this->writeDebug('getAdminIDs', '');
		}
		if (empty($this->games)) {
			$this->initDB();
		}
		$admins = array();
		
		foreach ($this->games as $game) {
			if ($game['game_id'] == $game_id) {
				
				foreach ($game['admins'] as $admin) {
					$admins[] = $admin['user_id'];
				}
				break;
			}
		}
		return $admins;
	}
	
	/**
	 * Logs user into game.
    * @param string $username Name of the user.
    * @param string $password Password of the user.
    * @return array|int User array if success else error code.
    * Error codes: -1: given user isn't found, -2: wrong password
   */
	public function login($username, $password) {
		
		if ($this->debug) {
			$this->writeDebug('login', '');
		}
		$user = $this->authenticateUser($username, $password);

		if ($user <= 0) {
			// There was an error;
			return $user;
		}
		$user_array = $this->getUser($user);
		
		if (empty($user_array)) {
			// User isn't found;
			return -1;
		}
		$_SESSION['user'] = $user_array;
		$_SESSION['isLogged'] = true;
		return $user_array;
	}
	
	/**
	 * Retrieves user information (username) based on given user id.
    * @param double $user_id ID of the user.
    * @return array|false Containing 'user_id' and 'username'. False if user isn't found.
   */
	public function getUser($user_id) {
		
		if ($this->debug) {
			$this->writeDebug('getUser', '');
		}
		
		if (empty($this->users)) {
			$this->initDB();
		}
		$user = array();
		foreach ($this->users as $us) {
			
			if ($us['user_id'] == $user_id) {
				
				$user['user_id'] = doubleval($us['user_id']);
				$user['username'] = $us['username'];
				break;
			}
		}
		return $user;
	}
		
	/**
	 * Get information of single game.
    * @param double $game_id ID of the game to retrieve.
    * @return array An array containing game's information.
   */
	public function getGame($game_id) {
		
		if ($this->debug) {
			$this->writeDebug('getGame', '');
		}
		
		// Empty session variables
		unset($_SESSION['xmldb']['cards']);
		$this->cards = array();
		$this->profiles = array();
		unset($_SESSION['xmldb']['profiles']);
		$this->current_profile = array();
		unset($_SESSION['xmldb']['current_profile']);
		$this->current_game = 0;
		unset($_SESSION['xmldb']['current_game']);
		$this->latest_card = array();
		unset($_SESSION['xmldb']['latest_card']);
		$this->initDB();
		
		if (empty($this->games)) {
    		$this->initDB();
    	}
    	$game = array();
    	
    	foreach ($this->games as $gam) {
    		if ($gam['game_id'] == $game_id) {
    			$game = $gam;
    			break;
    		}
    	}
    	$this->current_game = $game_id;
    	$_SESSION['xmldb']['current_game'] = $this->current_game;
    	
    	return $game;
	}
	
	/**
	 * Opens previously created game profile.
    * @param double $profile_id ID of the profile to be opened.
    * @return array|false Game information in array or false if function failed.
   */
	public function continueGame($profile_id) {
		
		if ($this->debug) {
			$this->writeDebug('continueGame', '');
		}
		
		// Empty session variables
		unset($_SESSION['xmldb']['cards']);
		$this->cards = array();
		$this->profiles = array();
		unset($_SESSION['xmldb']['profiles']);
		$this->current_profile = array();
		unset($_SESSION['xmldb']['current_profile']);
		$this->current_game = 0;
		unset($_SESSION['xmldb']['current_game']);
		$this->latest_card = array();
		unset($_SESSION['xmldb']['latest_card']);
		$this->initDB();
		
		if ($profile_id > 0) {
			$profile = $this->getProfile($profile_id);
		
			$game = $this->getGame($profile['game_id']);
			
			if (empty($game)){
				return false;
			}
			
			$game['profile'] = $profile_id;
			$this->current_game = $game_id;
			$_SESSION['xmldb']['current_game'] = $this->current_game;
			return $game;
		}
		return false;
	}
	
	/**
	 * Get all registered users that are tied to given game_id.
    * @param double $game_id ID of the game.
    * @return array Containing 'username' and 'user_id' subarrays.
   */
	public function getAllUsers($game_id) {
		
		if ($this->debug) {
			$this->writeDebug('getAllUsers', '');
		}
		if (empty($this->users)) {
    		$this->initDB();
    	}
    	
		$users = array();
		
		if (is_dir($this->xml_folder . 'games/' . $game_id)) {
				
			if (is_dir($this->xml_folder . 'games/' . $game_id . '/player_profiles')) {
					
				for ($i = 1; $i <= $this->latest_ids['profile_id']; $i++) {
						
					if (is_file($this->xml_folder . 'games/' . $game_id . '/player_profiles/profile_' . $i . '.xml')) {
					
						$profile = $this->readXMLProfile($this->xml_folder . 'games/' . $game_id . '/player_profiles/profile_' . $i . '.xml');
						
						if (isset($profile['user_id']) && !empty($profile['user_id'])) {
							$user = array();
							$user['user_id'] = $profile['user_id'];
							foreach ($this->users as $us) {
								if ($us['user_id'] == $user['user_id']) {
									$user['username'] = $us['username'];
									break;
								}
							}
							$users[] = $user;
						}
					}
				}
			}
		}
		
		return $users;
	}
	
	/**
	 * Retrieves all player profiles related to game.
    * @param double $game_id ID of the game of which profiles to retrieve.
    * @return array An array of player profiles.
   */
	public function getAllProfiles($game_id) {
		
		if ($this->debug) {
			$this->writeDebug('getAllProfiles', 'game_id:' . $game_id);
		}
		$profiles = array();
		
		if ($this->current_game > 0 && $this->current_game['game_id'] == $game_id) {
			
			if (!empty($this->profiles)) {
				$profiles = $this->profiles;
				// Get played cards count too
				foreach ($profiles as &$profile) {
					$profile['played_cards'] = count($profile['played_cards']);
				}
				return $profiles;
			}
		}
		
		elseif (is_dir($this->xml_folder . 'games/' . $game_id)) {
				
			if (is_dir($this->xml_folder . 'games/' . $game_id . '/player_profiles')) {
					
				for ($i = 1; $i <= $this->latest_ids['profile_id']; $i++) {
						
					if (is_file($this->xml_folder . 'games/' . $game_id . '/player_profiles/profile_' . $i . '.xml')) {
					
						$profile = $this->readXMLProfile($this->xml_folder . 'games/' . $game_id . '/player_profiles/profile_' . $i . '.xml');
						$profiles[] = $profile;
					}
				}
				$this->current_game = $game_id;
				$this->profiles = $profiles;
				$_SESSION['xmldb']['profiles'] = $this->profiles;
			}
		}
		// Get played cards count too
		foreach ($profiles as &$profile) {
			$profile['played_cards'] = count($profile['played_cards']);
		}
		
		return $profiles;
	}
	
	/**
	 * Retrieves a single player profile.
    * @param double $profile_id ID of the player profile to be retrieved.
    * @return array An array of information about player profile.
   */
	public function getProfile($profile_id) {
		
		if ($this->debug) {
			$this->writeDebug('getProfile', 'profile_id:' . $profile_id);
		}
		if (empty($this->games)) {
    		$this->initDB();
    	}
		$profile = array();
		
		$found = false;
		
		if ($this->current_game > 0) {
			if (!empty($this->current_profile)) {
				if ($this->current_profile['profile_id'] == $profile_id) {
					$profile = $this->current_profile;
					$found = true;
				}
			}
			if (!$found && !empty($this->profiles)) {
				foreach ($this->profiles as $prof) {
					if ($prof['profile_id'] == $profile_id) {
						$profile = $prof;
						$this->current_profile = $profile;
						$_SESSION['xmldb']['current_profile'] = $this->current_profile;
						$found = true;
						break;
					}
				}
				unset($prof);
			}
			
		}
		if (!$found) {
			foreach ($this->games as $game) {
				$found = false;
				if (is_dir($this->xml_folder . 'games/' . $game['game_id'])) {
				
					if (is_dir($this->xml_folder . 'games/' . $game['game_id'] . '/player_profiles')) {
					
						for ($i = 1; $i <= $this->latest_ids['profile_id']; $i++) {
						
							if (is_file($this->xml_folder . 'games/' . $game['game_id'] . '/player_profiles/profile_' . $i . '.xml')) {
					
								$profile = $this->readXMLProfile($this->xml_folder . 'games/' . $game['game_id'] . '/player_profiles/profile_' . $i . '.xml');
								if ($profile['profile_id'] == $profile_id) {
									$this->current_profile = $profile;
									$_SESSION['xmldb']['current_profile'] = $this->current_profile;
									$found = true;
									break;
								}
							}
							
						}
					}
				}
				if ($found) {
					break;
				}
			}
		}
		// Get played cards count too
		$profile['played_cards'] = count($this->getPlayedCards($profile_id));
		
		return $profile;
	}
	
	/**
	 * Retrieves information about cards user has played.
    * @param double $profile_id ID of the player profile.
    * @return array An array of played cards.
   */
	public function getPlayedCards($profile_id) {
		
		if ($this->debug) {
			$this->writeDebug('getPlayedCards', 'profile: ' . $profile_id);
		}
		if (empty($this->games)) {
    		$this->initDB();
    	}
		$played_cards = array();
		
		$found = false;
		
		if ($this->current_game > 0) {
			
			if (!empty($this->current_profile)) {
				
				if ($this->current_profile['profile_id'] == $profile_id) {
					$played_cards = $this->current_profile['played_cards'];
					$found = true;
				}
			}
			
			if (!$found && !empty($this->profiles)) {
				foreach ($this->profiles as $prof) {
					if ($prof['profile_id'] == $profile_id) {
						$played_cards = $prof['played_cards'];
						$this->current_profile = $prof;
						
						$found = true;
						break;
					}
				}
				unset($prof);
			}
		}
		
		if (!$found) {
			
			foreach ($this->games as $game) {
				
				$found = false;
				if (is_dir($this->xml_folder . 'games/' . $game['game_id'])) {
					
					if (is_dir($this->xml_folder . 'games/' . $game['game_id'] . '/player_profiles')) {
					
						for ($i = 1; $i <= $this->latest_ids['profile_id']; $i++) {
						
							if (is_file($this->xml_folder . 'games/' . $game['game_id'] . '/player_profiles/profile_' . $i . '.xml')) {
					
								$profile = $this->readXMLProfile($this->xml_folder . 'games/' . $game['game_id'] . '/player_profiles/profile_' . $i . '.xml');
								
								if ($profile['profile_id'] == $profile_id) {
									$this->current_profile = $profile;
									$_SESSION['xmldb']['current_profile'] = $this->current_profile;
									$played_cards = $profile['played_cards'];
									$found = true;
									break;
								}
							}
							
						}
					}
				}
				if ($found) {
					break;
				}
			}
		}
		
		return $played_cards;
	}
	
	/**
	 * Retrieves the number of all cards related to a game.
	 * @param double $game_id ID of the game of which cards to retrieve.
    * @return int Number of cards.
   */
	public function getNbrOfCardsInGame($game_id) {
		
		if ($this->debug) {
			$this->writeDebug('getNbrOfCardsInGame', 'game_id:' . $game_id);
		}
		if (empty($this->games) || empty($this->users)) {
			$this->initDB();
		}
		$cards = $this->cards;
		
		if (empty($this->cards) || $this->current_game != $game_id) {
			
			$cards = $this->getAllCards($game_id);
		}

		return count($cards);
	}
	
	/**
	 * Retrieves all cards related to a game.
	 * @param double $game_id ID of the game of which cards to retrieve.
	 * @param string $lang Language code of the language cards are wanted.
    * @return array An array of cards.
   */
	public function getAllCards($game_id, $lang='en-US', $include_choices=true) {
		
		// Choices are included everytime in XML-version
		
		if ($this->debug) {
			$this->writeDebug('getAllCards', 'game_id:' . $game_id);
		}
		if (empty($this->games) || empty($this->users)) {
			$this->initDB();
		}
		
		if (empty($this->cards) || $this->current_game != $game_id) {
			
			if (is_dir($this->xml_folder . 'games/' . $game_id)) {
				
				if (is_file($this->xml_folder . 'games/' . $game_id . '/cards_' . $game_id . '.xml')) {
					
					$cards = $this->readXMLCards($this->xml_folder . 'games/' . $game_id . '/cards_' . $game_id . '.xml');
					
					// Add creator and editor names
					foreach ($cards as &$card) {
						
						$card['creator_name'] = '';
						$card['editor_name'] = '';
						if ($card['creator_id'] > 0) {
							$user = $this->getUser($card['creator_id']);
							$card['creator_name'] = $user['username'];
						}
						if ($card['editor_id'] > 0) {
							$user = $this->getUser($card['editor_id']);
							$card['editor_name'] = $user['username'];
						}
					}
					unset($card);
					$this->cards = $cards;
					$_SESSION['xmldb']['cards'] = $this->cards;
					$this->current_game = $game_id;
					$_SESSION['xmldb']['current_game'] = $this->current_game;
				}
			}
		}
		$cards_to_return = array();
		
		foreach ($this->cards as $card) {
			$card_to_add = array();
			$card_to_add['card_id'] = $card['card_id'];
			$card_to_add['card_external_id'] = $card['card_external_id'];
			$card_to_add['game_id'] = $card['game_id'];
			$card_to_add['enabled'] = $card['enabled'];
			$card_to_add['repetitive'] = $card['repetitive'];
			$card_to_add['created'] = $card['created'];
			$card_to_add['modified'] = $card['modified'];
			$card_to_add['creator_id'] = $card['creator_id'];
			$card_to_add['editor_id'] = $card['editor_id'];
			
			$tags_array = explode(' ', $card['tags']);
			$clean_tags = array();
				
			foreach ($tags_array as $tag) {
				$trimmed = trim($tag);
				if (!empty($trimmed)) {
					$clean_tags[] = $trimmed;
				}
			}
			$card_to_add['tags'] = $clean_tags;
			
			$stats_array = explode(' ', $card['stat_requirements']);
			$clean_stats = array();
				
			foreach ($stats_array as $stat) {
				$trimmed = trim($stat);
				if (!empty($trimmed)) {
					$clean_stats[] = $trimmed;
				}
			}
			$card_to_add['stat_requirements'] = $clean_stats;
			$card_to_add['comment'] = $card['comment'];
			$card_to_add['attachment'] = $card['attachment'];
			$card_to_add['layout'] = $card['layout'];
			$card_to_add['hidden'] = $card['hidden'];
			
			$tags_array = explode(' ', $card['related_tags']);
			$clean_tags = array();
				
			foreach ($tags_array as $tag) {
				$trimmed = trim($tag);
				if (!empty($trimmed)) {
					$clean_tags[] = $trimmed;
				}
			}
			$card_to_add['related_tags'] = $clean_tags;
			
			$stats_array = explode(' ', $card['related_tags']);
			$clean_stats = array();
				
			foreach ($stats_array as $stat) {
				$trimmed = trim($stat);
				if (!empty($trimmed)) {
					$clean_stats[] = $trimmed;
				}
			}
			$card_to_add['related_stats'] = $clean_stats;
			
			$card_to_add['creator_name'] = $card['creator_name'];
			$card_to_add['editor_name'] = $card['editor_name'];
			$card_to_add['default_name'] = '';
			$card_to_add['default_description'] = '';
			$card_to_add['default_expert_advice'] = '';
			$card_to_add['default_lang'] = '';
			$card_to_add['card_name'] = '';
			$card_to_add['description'] = '';
			$card_to_add['expert_advice'] = '';
			$card_to_add['lang'] = '';
					
			foreach ($card['translations'] as $trans) {
				if ($trans['is_default'] == 1) {
					$card_to_add['default_name'] = $trans['card_name'];
					$card_to_add['default_description'] = $trans['description'];
					$card_to_add['default_expert_advice'] = $trans['expert_advice'];
					$card_to_add['default_lang'] = $trans['language_code'];
				}
				if ($trans['language_code'] == $lang) {
					$card_to_add['card_name'] = $trans['card_name'];
					$card_to_add['description'] = $trans['description'];
					$card_to_add['expert_advice'] = $trans['expert_advice'];
					$card_to_add['lang'] = $trans['language_code'];
				}
			}
			if (empty($card_to_add['lang'])) {
				$card_to_add['lang'] = $card_to_add['default_lang'];
			}
			if (empty($card_to_add['tags'])) {
				$card_to_add['tags'] = array();
				foreach ($card['tags'] as $tag) {
					$card_to_add['tags'][] = $tag['tag_name'];
				}
			}
			$card_to_add['choices'] = array();
			
			foreach ($card['choices'] as $choi) {
				$choice = array();
				$choice['choice_id'] = $choi['choice_id'];
				$choice['card_id'] = $card['card_id'];
				$choice['ordinal'] = $choi['ordinal'];
				$choice['default_text'] = '';
				$choice['default_lang'] = '';
				$choice['choice_text'] = '';
				$choice['lang'] = '';
				foreach ($choi['translations'] as $trans) {
					if ($trans['is_default'] == 1) {
						$choice['default_text'] = $trans['text'];
						$choice['default_lang'] = $trans['language_code'];
					}
					if ($trans['language_code'] == $lang) {
						$choice['choice_text'] = $trans['text'];
						$choice['lang'] = $trans['language_code'];
					}
				}
				unset($trans);
				$choice['results'] = array();
				
				foreach ($choi['results'] as $res) {
					$result = array();
					$result['result_id'] = $res['result_id'];
					$result['choice_id'] = $choi['choice_id'];
					$result['weight'] = $res['weight'];
					$result['stat_changes'] = $res['stat_changes'];
					$result['next_criteria'] = $res['next_criteria'];
					$result['ordinal'] = $res['ordinal'];
					$result['choice_ordinal'] = $choi['ordinal'];
					$result['default_text'] = '';
					$result['default_lang'] = '';
					$result['result_text'] = '';
					$result['lang'] = '';
					
					foreach ($res['translations'] as $trans) {
						if ($trans['is_default'] == 1) {
							$result['default_text'] = $trans['text'];
							$result['default_lang'] = $trans['language_code'];
						}
						if ($trans['language_code'] == $lang) {
							$result['result_text'] = $trans['text'];
							$result['lang'] = $trans['language_code'];
						}
					}
					unset($trans);
					$choice['results'][] = $result;
				}
				$card_to_add['choices'][] = $choice;
			}
			$cards_to_return[] = $card_to_add;
		}
		
		return $cards_to_return;
	}
	
	/**
	 * Retrieves a single card.
	 * @param double $card_id ID of the card to retrieve.
	 * @param string $lang Language code of the language card is wanted.
    * @return array Containing information about the card.
   */
	public function getCard($card_id, $lang='en-US') {
		
		if ($this->debug) {
			$this->writeDebug('getCard', 'card_id:' . $card_id);
		}
		
		if (empty($this->games) || empty($this->users)) {
			$this->initDB();
		}
		
		if (empty($this->cards)) {
			$game_id = 0;
			if ($this->current_game > 0) {
				$game_id = $this->current_game;
			}
			elseif (isset($_SESSION['admin']['game']['game_id'])) {
				$game_id = $_SESSION['admin']['game']['game_id'];
			}
			$this->getAllCards($game_id, $lang);
		}
		$cards = $this->cards;

		$card_to_return = array();
		
		if (!empty($this->latest_card) && $this->latest_card['card_id'] == $card_id && $this->latest_card['lang'] == $lang) {
			$card_to_return = $this->latest_card;
		}
		else {
			
			foreach ($cards as $card) {
				
				if (doubleval($card['card_id']) == doubleval($card_id)) {
					
					$card_to_return['card_id'] = $card['card_id'];
					$card_to_return['card_external_id'] = $card['card_external_id'];
					$card_to_return['game_id'] = $card['game_id'];
					$card_to_return['enabled'] = $card['enabled'];
					$card_to_return['repetitive'] = $card['repetitive'];
					$card_to_return['created'] = $card['created'];
					$card_to_return['modified'] = $card['modified'];
					$card_to_return['creator_id'] = $card['creator_id'];
					$card_to_return['editor_id'] = $card['editor_id'];
					
					$tags_array = explode(' ', $card['tags']);
					$clean_tags = array();
				
					foreach ($tags_array as $tag) {
						$trimmed = trim($tag);
						if (!empty($trimmed)) {
							$clean_tags[] = $trimmed;
						}
					}
					$card_to_return['tags'] = $clean_tags;
					
					$stats_array = explode(' ', $card['stat_requirements']);
					$clean_stats = array();
				
					foreach ($stats_array as $stat) {
						$trimmed = trim($stat);
						if (!empty($trimmed)) {
							$clean_stats[] = $trimmed;
						}
					}
					$card_to_return['stat_requirements'] = $clean_stats;
					$card_to_return['comment'] = $card['comment'];
					$card_to_return['attachment'] = $card['attachment'];
					$card_to_return['layout'] = $card['layout'];
					$card_to_return['hidden'] = $card['hidden'];
					
					$tags_array = explode(' ', $card['related_tags']);
					$clean_tags = array();
				
					foreach ($tags_array as $tag) {
						$trimmed = trim($tag);
						if (!empty($trimmed)) {
						$clean_tags[] = $trimmed;
						}
					}
					$card_to_return['related_tags'] = $clean_tags;
						
					$stats_array = explode(' ', $card['related_tags']);
					$clean_stats = array();
				
					foreach ($stats_array as $stat) {
						$trimmed = trim($stat);
						if (!empty($trimmed)) {
							$clean_stats[] = $trimmed;
						}
					}
					$card_to_return['related_stats'] = $clean_stats;
					
					$card_to_return['creator_name'] = $card['creator_name'];
					$card_to_return['editor_name'] = $card['editor_name'];
					$card_to_return['default_name'] = '';
					$card_to_return['default_description'] = '';
					$card_to_return['default_expert_advice'] = '';
					$card_to_return['default_lang'] = '';
					$card_to_return['card_name'] = '';
					$card_to_return['description'] = '';
					$card_to_return['expert_advice'] = '';
					$card_to_return['lang'] = '';
					
					foreach ($card['translations'] as $trans) {
						if ($trans['is_default'] == 1) {
							$card_to_return['default_name'] = $trans['card_name'];
							$card_to_return['default_description'] = $trans['description'];
							$card_to_return['default_expert_advice'] = $trans['expert_advice'];
							$card_to_return['default_lang'] = $trans['language_code'];
						}
						if ($trans['language_code'] == $lang) {
							$card_to_return['card_name'] = $trans['card_name'];
							$card_to_return['description'] = $trans['description'];
							$card_to_return['expert_advice'] = $trans['expert_advice'];
							$card_to_return['lang'] = $trans['language_code'];
						}
					}
					if (empty($card_to_return['lang'])) {
						$card_to_return['lang'] = $card_to_return['default_lang'];
					}
					if (empty($card_to_return['tags'])) {
						$card_to_return['tags'] = array();
						foreach ($card['tags'] as $tag) {
							$card_to_return['tags'][] = $tag['tag_name'];
						}
					}
					$card_to_return['choices'] = array();
			
					foreach ($card['choices'] as $choi) {
						$choice = array();
						$choice['choice_id'] = $choi['choice_id'];
						$choice['card_id'] = $card['card_id'];
						$choice['ordinal'] = $choi['ordinal'];
						$choice['default_text'] = '';
						$choice['default_lang'] = '';
						$choice['choice_text'] = '';
						$choice['lang'] = '';
						foreach ($choi['translations'] as $trans) {
							if ($trans['is_default'] == 1) {
								$choice['default_text'] = $trans['text'];
								$choice['default_lang'] = $trans['language_code'];
							}
							if ($trans['language_code'] == $lang) {
								$choice['choice_text'] = $trans['text'];
								$choice['lang'] = $trans['language_code'];
							}
						}
						unset($trans);
						$choice['results'] = array();
				
						foreach ($choi['results'] as $res) {
							$result = array();
							$result['result_id'] = $res['result_id'];
							$result['choice_id'] = $choi['choice_id'];
							$result['weight'] = $res['weight'];
							$result['stat_changes'] = $res['stat_changes'];
							$result['next_criteria'] = $res['next_criteria'];
							$result['ordinal'] = $res['ordinal'];
							$result['choice_ordinal'] = $choi['ordinal'];
							$result['default_text'] = '';
							$result['default_lang'] = '';
							$result['result_text'] = '';
							$result['lang'] = '';
					
							foreach ($res['translations'] as $trans) {
								if ($trans['is_default'] == 1) {
									$result['default_text'] = $trans['text'];
									$result['default_lang'] = $trans['language_code'];
								}
								if ($trans['language_code'] == $lang) {
									$result['result_text'] = $trans['text'];
									$result['lang'] = $trans['language_code'];
								}
							}
							unset($trans);
							$choice['results'][] = $result;
						}
						$card_to_return['choices'][] = $choice;
					}
					break;
				}
			}
			$this->latest_card = $card_to_return;
			$_SESSION['xmldb']['latest_card'] = $this->latest_card;
		}
		
		return $card_to_return;
	}
	
	/**
	 * Retrieves information about single result.
	 * @param double $result_id ID of the result to retrieve.
    * @param string $lang Language code of the result to retrieve.
    * @return array An array containing the information about retrieved result.
   */
	public function getResult($result_id, $lang='en-US') {
		
		if ($this->debug) {
			$this->writeDebug('getResult', 'result_id:' . $result_id);
		}
		if (empty($this->games) || empty($this->users)) {
			$this->initDB();
		}

		$result = array();
		
		if (!empty($this->latest_card)) {
			
			foreach ($this->latest_card['choices'] as $choice) {
				
				foreach ($choice['results'] as $res) {
					
					if ($res['result_id'] == $result_id && $res['lang'] == $lang) {
						
						$choice_ordinal = $choice['ordinal'];
						break;
					}
				}
				unset($res);
				if(!empty($result)) {
					break;
				}
			} 
		}
		
		if (empty($result)) {
			
			$game_id = 0;
			if ($this->current_game > 0) {
				$game_id = $this->current_game;
			}
			elseif (isset($_SESSION['admin']['game']['game_id'])) {
				$game_id = $_SESSION['admin']['game']['game_id'];
			}
			$cards = $this->getAllCards($game_id, $lang);
			
			foreach ($cards as $card) {
				foreach ($card['choices'] as $choice) {
					foreach ($choice['results'] as $res) {
						if (doubleval($res['result_id']) == doubleval($result_id)) {
							$result = $res;
							break;
						}
					}
					if (!empty($result)) {
						break;
					}
				}
				if (!empty($result)) {
					break;
				}
			}
		}
		return $result;
	}
	
	/**
	 * Retrieves results linked to a single choice.
	 * @param double $choice_id ID of the choice of which results to retrieve.
    * @param string $lang Language code of the results to retrieve.
    * @return array An array containing the results.
   */
	public function getChoiceResults($choice_id, $lang='en-US') {
		
		if ($this->debug) {
			$this->writeDebug('getChoiceResults', 'choice: ' . $choice_id);
		}
		if (empty($this->games) || empty($this->users)) {
			$this->initDB();
		}
		$results = array();
		
		if (!empty($this->latest_card)) {
			
			foreach ($this->latest_card['choices'] as $choice) {
				
				if ($choice['choice_id'] == $choice_id && $choice['lang'] == $lang) {
					$results = $choice['results'];
					break;
				}
			}
			unset($choice);
		}
		
		if (empty($results)) {
			
			$game_id = 0;
			if ($this->current_game > 0) {
				$game_id = $this->current_game;
			}
			elseif (isset($_SESSION['admin']['game']['game_id'])) {
				$game_id = $_SESSION['admin']['game']['game_id'];
			}
			$cards = $this->getAllCards($game_id, $lang);
			
			foreach ($cards as $card) {
				
				foreach ($card['choices'] as $choice) {
					
					if ($choice['choice_id'] == $choice_id) {
						$results = $choice['results'];
						break;
					}
				}
				if (!empty($result)) {
					break;
				}
			}
		}
		return $results;
	}
	
	/**
	 * Retrieves all translations of a card.
	 * @param double $card_id ID of the card in which translations to retrieve.
    * @return array An array of translations.
   */
	public function getCardTranslations($card_id) {
		
		if ($this->debug) {
			$this->writeDebug('getCardTranslations', 'card: ' . $card_id);
		}
		if (empty($this->games) || empty($this->users)) {
			$this->initDB();
		}
		$translations = array();
		
		if (empty($this->cards)) {
			
			$game_id = 0;
			if ($this->current_game > 0) {
				$game_id = $this->current_game;
			}
			elseif (isset($_SESSION['admin']['game']['game_id'])) {
				$game_id = $_SESSION['admin']['game']['game_id'];
			}
			$this->getAllCards($game_id, $lang);
		}

		foreach ($this->cards as $card) {
			if ($card['card_id'] == $card_id) {
				$translations = $card['translations'];
			}
		}
		return $translations;
	}
	
	/**
	 * Retrieves all tags and stats related to the cards of the game.
    * @param double $game_id ID of the game of which tags to retrieve.
    * @return array An array containing 'tags' and 'stats'.
   */
	public function getAllTagsAndStats($game_id) {
		
		if ($this->debug) {
			$this->writeDebug('getAllTagsAndStats', 'game: ' . $game_id);
		}
		if (empty($this->games) || empty($this->users)) {
			$this->initDB();
		}
		
		$current_lang = '';
		if (isset($_SESSION['lang'])) {
			$current_lang = $_SESSION['lang'];
		}
		else {
			global $language;
			$current_lang = $language;
		}
		
		$cards = $this->getAllCards($game_id, $current_lang);
		$tags_and_stats = array();
		$tags_and_stats['tags'] = array();
		$tags_and_stats['stats'] = array();
		
		foreach ($cards as $card) {
			
			if (!isset($card['related_tags']) || empty($card['related_tags']) || !isset($card['related_stats']) || empty($card['related_stats'])) {
				$this->updateCardRelatedTagsAndStats($card['card_id']);
				$card = $this->getCard($card['card_id']);
			}
			foreach ($card['related_tags'] as $tag) {
				if (!in_array($tag, $tags_and_stats['tags'])) {
					$tags_and_stats['tags'][] = $tag;
				}
			}
			foreach ($card['related_stats'] as $stat) {
				if (!in_array($stat, $tags_and_stats['stats'])) {
					$tags_and_stats['stats'][] = $stat;
				}
			}
		}
		
		if (!empty($tags_and_stats['tags'])) {
			sort($tags_and_stats['tags'], SORT_STRING);
		}
		if (!empty($tags_and_stats['stats'])) {
			sort($tags_and_stats['stats'], SORT_STRING);
		}
		return $tags_and_stats;
	}
	
	/**
	 * Retrieves all tags related to the cards of the game.
    * @param double $game_id ID of the game of which tags to retrieve.
    * @return array An array of tags.
   */
	public function getAllTags($game_id) {
		
		if ($this->debug) {
			$this->writeDebug('getAllTags', 'game: ' . $game_id);
		}
		if (empty($this->games) || empty($this->users)) {
			$this->initDB();
		}
		
		$current_lang = '';
		if (isset($_SESSION['lang'])) {
			$current_lang = $_SESSION['lang'];
		}
		else {
			global $language;
			$current_lang = $language;
		}
		
		$cards = $this->getAllCards($game_id, $current_lang);
		$tags = array();
		
		foreach ($cards as $card) {
			
			if (isset($card['tags'])) {
				
				foreach ($card['tags'] as $tag) {
					
					if (!in_array($tag, $tags)) {
						$tags[] = $tag;
					}
				}
			}
			// Get tags from results
			if (isset($card['choices'])) {
				
				foreach ($card['choices'] as $choice) {
					
					foreach ($choice['results'] as $result) {
						
						$next_criteria = explode("#", $result['next_criteria']);
							
						if (!empty($next_criteria)) {
							
							foreach ($next_criteria as &$crit) {
								
								$crit = trim($crit);
								
								if (!empty($crit)) {
									// Check if tag is id
									$pos = strpos($crit, "ID");
									if ($pos !== false) {
										$id = substr($crit, $pos+2);
										if (is_numeric($id)) {
											// and continue if it is
											continue;
										}
									}
									// If current tag isn't in array of tags, add it there
									if (!in_array($crit, $tags)) {
										$tags[] = $crit;
									}
								}
							}
						}
					}
				}
			}
		}
		if (!empty($tags)) {
			sort($tags, SORT_STRING);
		}

		return $tags;
	}
	
	/**
	 * Retrieves all stats related to the cards of the game.
    * @param double $game_id ID of the game of which stats to retrieve.
    * @return array An array of stats.
   */
	public function getAllStats($game_id) {
		
		if ($this->debug) {
			$this->writeDebug('getAllStats', 'game: ' . $game_id);
		}
		
		$current_lang = '';
		if (isset($_SESSION['lang'])) {
			$current_lang = $_SESSION['lang'];
		}
		else {
			global $language;
			$current_lang = $language;
		}
		$cards = $this->getAllCards($game_id, $current_lang);
		
		$stats = array();
		
		foreach ($cards as $card) {
			
			// Get stats from card stat requirements
			if (isset($card['stat_requirements'])) {
				
				foreach ($card['stat_requirements'] as $stat) {
						
					$parsed_stat = GameEngine::parseStat($stat);
					
					if ($parsed_stat != false) {
						$parsed_stat['key'] = '@' . $parsed_stat['key'];
						// If current stat isn't in array, add it there
						if (!in_array($parsed_stat['key'], $stats)) {
							
							$stats[] = $parsed_stat['key'];
						}
					}
				}
			}
			// Get stats from results
			if (isset($card['choices'])) {
				
				foreach ($card['choices'] as $choice) {
					
					foreach ($choice['results'] as $result) {

						$stat_changes = explode("@", $result['stat_changes']);

						if (!empty($stat_changes)) {

							foreach ($stat_changes as $stat) {
						
								$parsed_stat = GameEngine::parseStat($stat);
					
								if ($parsed_stat != false) {
									$parsed_stat['key'] = '@' . $parsed_stat['key'];
									// If current stat isn't in array of stats, add it there
									if (!in_array($parsed_stat['key'], $stats)) {
										$stats[] = $parsed_stat['key'];
									}
								}
							}
						}
					}
				}
			}
		}
		
		if (!empty($stats)) {
			sort($stats, SORT_STRING);
		}
		return $stats;
	}
	
	/**
	 * Changes user's password.
    * @param string $username Name of the user.
    * @param string $old_password Current password of the user.
    * @param string $new_password New password of the user.
    * @return bool Was operation successful?
   */
	public function changePassword($username, $old_password, $new_password) {
		
		if ($this->debug) {
			$this->writeDebug('changePassword', 'username: ' . $username);
		}
		
		if (empty($this->users)) {
			$this->initDB();
		}
		// First authenticate user
		$user_id = $this->authenticateUser($username, $old_password);

		if ($user_id <= 0) {
			// There was an error;
			return false;
		}
		$hash = '';
		$config = Config::getInstance();
		if (version_compare($config->php_version, '3.5.7') >= 0) {
			$hash = password_hash($new_password, PASSWORD_BCRYPT);
		}
		else {
			$hasher = new PasswordHash($hash_cost_log2, $hash_portable);
			$hash = $hasher->HashPassword($new_password);
			if (strlen($hash) < 20) {
				echo('Failed to hash new password' . PHP_EOL);
				exit();
			}
			unset($hasher);
		}
		// Save password to users array
		foreach ($this->users as &$user) {
			if ($user['user_id'] == $user_id) {
				$user['password'] = $hash;
				break;
			}
		}
		$_SESSION['xmldb']['users'] = $this->users;
		// Update xml file
		$success = $this->writeXMLUsers($this->xml_folder . 'users.xml', $this->users);
		
		return $success;
	}
	
	/**
	 * Creates new user.
    * @param string $username Name of the new user.
    * @param string $password Password of the new user.
    * @return double User id or in case of error -1
   */
	public function createUser($username, $password) {
		
		if ($this->debug) {
			$this->writeDebug('createUser', 'username: ' . $username);
		}
		
		// Empty old arrays so we can get the latest information
		$_SESSION['xmldb']['users'] = array();
		$this->users = array();
		
		if (empty($this->users)) {
			$this->initDB();
		}
		// First check if username is already taken
		foreach ($this->users as $user) {
			if ($user['username'] == $username) {
				return -1;
				break;
			}
		}
		$hash = '';
		// Create new user
		$config = Config::getInstance();
		if (version_compare($config->php_version, '5.3.7') >= 0) {
			$hash = password_hash($password, PASSWORD_BCRYPT);
		}
		else {
			$hasher = new PasswordHash($hash_cost_log2, $hash_portable);
			$hash = $hasher->HashPassword($password);
			if (strlen($hash) < 20) {
				echo('Failed to hash new password' . PHP_EOL);
				exit();
			}
			unset($hasher);
		}
		
		$new_user = array();
		
		$new_user['user_id'] = $this->latest_ids['user_id'] + 1;
		$new_user['username'] = $username;
		$new_user['password'] = $hash;
		
		$this->users[] = $new_user;
		
		$user_id = 0;
		
		// Update xml file
		if ($this->writeXMLUsers($this->xml_folder . 'users.xml', $this->users)) {
			$_SESSION['xmldb']['users'] = $this->users;
			$this->latest_ids['user_id'] = $new_user['user_id'];
			$this->writeXMLLatestIds($this->xml_folder . 'latest_ids.xml', $this->latest_ids);
			$user_id = $new_user['user_id'];
		}
		return $user_id;
	}
	
	/**
	 * Deletes user.
    * @param double $user_id ID of the user to be deleted.
    * @return int Affected rows. 0 if no user deleted.
   */
	public function deleteUser($user_id) {
		
		if ($this->debug) {
			$this->writeDebug('deleteUser', 'user: ' . $user_id);
		}
		if (empty($this->users)) {
			$this->initDB();
		}
		$deleted = false;
		$index = 0;
		
		foreach ($this->users as $user) {
			
			if ($user['user_id'] == $user_id) {
				
				array_splice($this->users, $index, 1);
				$deleted = true;
				$this->writeXMLUsers($this->xml_folder . 'users.xml', $this->users);
				$_SESSION['xmldb']['users'] = $this->users;
				break;
			}
			$index++;
		}
		
		return $deleted;
	}
	
	/**
	 * Creates a new game.
    * @param string $name Name of the game. Must be unique.
    * @param string $admin_id Main administrator of the game.
    * @return int ID of the created game or in case of error -1.
   */
	public function createGame($name, $admin_id) {
		
		if ($this->debug) {
			$this->writeDebug('createGame', 'name: ' . $name);
		}
		
		if (empty($this->games)) {
			$this->initDB();
		}
		// First check if game name is already taken
		foreach ($this->games as $game) {
			if ($game['name'] == $name) {
				return -1;
				break;
			}
		}
		
		$new_game = array();
		$new_game['game_id'] = $this->latest_ids['game_id'] + 1;
		$new_game['name'] = $name;
		$new_game['main_admin_id'] = $admin_id;
		$new_game['starting_card_id'] = 0;
		$new_game['panic_card_id'] = 0;
		$new_game['image_folder'] = '';
		$new_game['use_tags'] = 1;
		$new_game['use_stats'] = 1;
		$new_game['use_expert_advice'] = 1;
		$new_game['use_sudden_scenarios'] = 1;
		$new_game['use_ending_criteria'] = 1;
		$new_game['use_login'] = 1;
		$new_game['use_name'] = 1;
		$new_game['multiple_languages'] = 1;
		$new_game['show_cards_ending'] = 1;
		$new_game['max_cancellations'] = -1;
		$new_game['background_color'] = '';
		$new_game['text_color'] = '';
		$new_game['link_color'] = '';
		$new_game['link_hover_color'] = '';
		$new_game['button_color'] = '';
		$new_game['button_hover_color'] = '';
		$new_game['background_tile'] = '';
		$new_game['logo'] = '';
		$new_game['visible_stats'] = array();
		
		// Create unique slug
		$unique = false;
		$slug = friendly_url($name);
		$counter = 1;
		// make slug unique
		while (!$unique) {
			
			$new_slug = $slug;
			if ($counter > 1) {
				$new_slug .= $counter; 
			}
			$game_id = $this->getGameIdBySlug($new_slug);
			
			// If slug is found increase counter
			if ($game_id > 0) {
				$counter++;
			}
			else {
				$unique = true;
				$slug = $new_slug;
			}
		}
		$new_game['slug'] = $slug;
		
		$new_game['latest_card_id'] = 0;
		$new_game['admins'] = array();
		
		$new_admin = array();
		$new_admin['user_id'] = $admin_id;
		$new_admin['relationship_id'] = $this->latest_ids['admin_relationship_id'] + 1;
		$new_game['admins'][] = $new_admin;
		
		$new_game['sudden_scenarios'] = array();				
		$new_game['ending_criteria'] = array();
		
		$mysqldate = date( 'Y-m-d H:i:s' );
		$new_game['created'] = $mysqldate;
							
		$this->games[] = $new_game;
		
		$game_id = 0;
		
		// Update xml file
		if ($this->writeXMLGames($this->xml_folder . 'games.xml', $this->games)) {
			$_SESSION['xmldb']['games'] = $this->games;
			$this->latest_ids['game_id'] = $new_game['game_id'];
			$this->latest_ids['admin_relationship_id'] = $new_admin['relationship_id'];
			$this->writeXMLLatestIds($this->xml_folder . 'latest_ids.xml', $this->latest_ids);
			$game_id = $new_game['game_id'];
		}
		
		// Create folder and files
		if (!is_dir($this->xml_folder . 'games/' . $game_id . '/')) {
			
			if (!mkdir($this->xml_folder . 'games/' . $game_id . '/')) {
				$this->deleteGame($game_id);
				die(localize('ERROR_CANT_CREATE_DIRECTORY') . PHP_EOL);
			}
		}
		
		if (!is_dir($this->xml_folder . 'games/' . $game_id . '/player_profiles')) {
			
			if (!mkdir($this->xml_folder . 'games/' . $game_id . '/player_profiles/')) {
				$this->deleteGame($game_id);
				die(localize('ERROR_CANT_CREATE_DIRECTORY') . PHP_EOL);
			}
		}
		
		$file_name = $this->xml_folder . 'games/' . $game_id . '/cards_' . $game_id . '.xml';
				
		if ($handle = fopen($file_name, 'wb')) {
			
			fclose($handle);
					
			$xml_base = <<<XML
<?xml version="1.0" encoding="UTF-8"?>
<cards>
</cards>
XML;
			$xml_cards = simplexml_load_string($xml_base);
		
			$xml_cards->addAttribute('game_id', $game_id);
			if (!$xml_cards->saveXML($this->xml_folder . 'games/' . $game_id . '/cards_' . $game_id . '.xml')) {
				$this->deleteGame($game_id);
				die(localize('ERROR_CANT_WRITE_FILE') . PHP_EOL);
			}
		}
		else {
			$this->deleteGame($game_id);
			die(localize('ERROR_CANT_WRITE_FILE') . PHP_EOL);
		}	
		
		return $game_id;
	}
	
	/**
	 * Deletes game and all that is related to it.
    * @param double $game_id ID of the game to be deleted.
    * @return int Number of games deleted. 0 if none.
   */
	public function deleteGame($game_id) {
		
		if ($this->debug) {
			$this->writeDebug('deleteGame', 'game: ' . $game_id);
		}
		
		if (empty($this->games)) {
			$this->initDB();
		}
		
		$deleted = false;
		$index = 0;
		
		foreach ($this->games as $game) {
			if ($game['game_id'] == $game_id) {
				array_splice($this->games, $index, 1);
				$deleted = true;
				$this->writeXMLGames($this->xml_folder . 'games.xml', $this->games);
				$_SESSION['xmldb']['games'] = $this->games;
				break;
			}
			$index++;
		}
		// Delete files and folders too
		$game_folder = $this->xml_folder . 'games/' . $game_id . '/';
		
		if (is_dir($game_folder)) {
			unlink($game_folder . 'cards_' . $game_id . '.xml');
			unlink($game_folder . 'profiles_' . $game_id . '.xml');
			unlink($game_folder . 'texts_' . $game_id . '.xml');
		}
		
		$profiles_folder = $game_folder . '/player_profiles/';
		if (is_dir($profiles_folder)) {
			for ($i = 1; $i <= $this->latest_ids['profile_id']; $i++) {
						
				if (is_file($profiles_folder . 'profile_' . $i . '.xml')) {	
					unlink($profiles_folder . 'profile_' . $i . '.xml');
				}
			}
			rmdir($profiles_folder);
		}
		rmdir($game_folder);
		
		return $deleted;
	}
	
	/**
	 * Updates game settings.
    * @param array $game Array containing new game settings.
    * @return void
   */
	public function updateGame($game) {
		
		if ($this->debug) {
			$this->writeDebug('updateGame', '');
		}
		
		if (empty($this->games)) {
			$this->initDB();
		}
		
		// First make sure that slug is unique
		$unique = false;
		$slug = friendly_url($game['slug']);
		if (empty($slug)) {
			$slug = friendly_url($game['name']);
		}
		$counter = 1;
		// make slug unique
		while (!$unique) {
			
			$new_slug = $slug;
			if ($counter > 1) {
				$new_slug .= $counter; 
			}
			$game_id = $this->getGameIdBySlug($new_slug);

			// If slug is found on another game increase counter
			if ($game_id != $game['game_id'] && $game_id > 0) {
				$counter++;
			}
			else {
				$unique = true;
				$slug = $new_slug;
			}
		}
		
		foreach ($this->games as &$current_game) {
			
			if ($current_game['game_id'] == $game['game_id']) {
				
				$current_game['name'] = $game['name'];
				$current_game['main_admin_id'] =  $game['main_admin_id'];
				$current_game['starting_card_id'] = $game['starting_card_id'];
				$current_game['panic_card_id'] = $game['panic_card_id'];
				$current_game['image_folder'] = $game['image_folder'];
				$current_game['slug'] = $slug;
				$current_game['latest_card_id'] = $game['latest_card_id'];
				$current_game['use_tags'] = $game['use_tags'];
				$current_game['use_stats'] = $game['use_stats'];
				$current_game['use_expert_advice'] = $game['use_expert_advice'];
				$current_game['use_sudden_scenarios'] = $game['use_sudden_scenarios'];
				$current_game['use_ending_criteria'] = $game['use_ending_criteria'];
				$current_game['use_login'] = $game['use_login'];
				$current_game['use_name'] = $game['use_name'];
				$current_game['multiple_languages'] = $game['multiple_languages'];
				$current_game['show_cards_ending'] = $game['show_cards_ending'];
				$current_game['max_cancellations'] = $game['max_cancellations'];
				$current_game['background_color'] = $game['background_color'];
				$current_game['text_color'] = $game['text_color'];
				$current_game['link_color'] = $game['link_color'];
				$current_game['link_hover_color'] = $game['link_hover_color'];
				$current_game['button_color'] = $game['button_color'];
				$current_game['button_hover_color'] = $game['button_hover_color'];
				$current_game['background_tile'] = $game['background_tile'];
				$current_game['logo'] = $game['logo'];
				$current_game['visible_stats'] = $game['visible_stats'];
		
				$current_game['sudden_scenarios'] = array();				
				$current_game['ending_criteria'] = array();
		
				// Add new scenarios
				if (isset($game['sudden_scenarios'])) {
			
					foreach ($game['sudden_scenarios'] as $scenario) {
				
						if (isset($scenario['odds']) && isset($scenario['result']) && isset($scenario['ordinal']) && isset($scenario['conditions'])) {
					
							$scenario_id = $this->latest_ids['scenario_id'] + 1;
							$this->latest_ids['scenario_id'] = $scenario_id;
							
							$new_scenario = array();
							$new_scenario['scenario_id'] = $scenario_id;
							$new_scenario['odds'] = $scenario['odds'];
							$new_scenario['result'] = $scenario['result'];
							$new_scenario['ordinal'] = $scenario['ordinal'];
							$new_scenario['conditions'] = array();
							
							if ($scenario_id > 0) {
								// Create new conditions
								foreach ($scenario['conditions'] as $condition) {
									
									$new_condition = array();
									$condition_id = $this->latest_ids['scenario_condition_id'] + 1;
									$this->latest_ids['scenario_condition_id'] = $condition_id;
									
									$new_condition['condition_id'] = $condition_id;
									$new_condition['condition_text'] = $condition['condition_text'];
									$new_condition['condition_logic'] = $condition['condition_logic'];
									$new_condition['ordinal'] = $condition['ordinal'];
									$new_scenario['conditions'][] = $new_condition;
								}
								$current_game['sudden_scenarios'][] = $new_scenario;
							}
						}
					} 
				}
				// Add new criteria
				if (isset($game['ending_criteria'])) {
			
					foreach ($game['ending_criteria'] as $criteria) {
				
						if (isset($criteria['result']) && isset($criteria['ordinal']) && isset($criteria['conditions'])) {
					
							$criteria_id = $this->latest_ids['criteria_id'] + 1;
							$this->latest_ids['criteria_id'] = $criteria_id;
							
							$new_ending = array();
							$new_ending['criteria_id'] = $criteria_id;
							$new_ending['result'] = $criteria['result'];
							$new_ending['ordinal'] = $criteria['ordinal'];
							$new_ending['conditions'] = array();
							
							if ($criteria_id > 0) {
								// Create new conditions
								foreach ($criteria['conditions'] as $condition) {
									
									$new_condition = array();
									$condition_id = $this->latest_ids['criteria_condition_id'] + 1;
									$this->latest_ids['scenario_condition_id'] = $condition_id;
									
									$new_condition['condition_id'] = $condition_id;
									$new_condition['condition_text'] = $condition['condition_text'];
									$new_condition['condition_logic'] = $condition['condition_logic'];
									$new_condition['ordinal'] = $condition['ordinal'];
									$new_ending['conditions'][] = $new_condition;
								}
								$current_game['ending_criteria'][] = $new_ending;
							}
						}
					}
				}
				// Update xml file
				if ($this->writeXMLGames($this->xml_folder . 'games.xml', $this->games)) {
					$_SESSION['xmldb']['games'] = $this->games;
					$this->writeXMLLatestIds($this->xml_folder . 'latest_ids.xml', $this->latest_ids);
				}
				break;
			}
		}
	}
	
	/**
	 * Adds new administrator for the game.
    * @param double $user_id ID of the admin to be added.
    * @param double $game_id ID of the game of which to add admin.
    * @return double|void|int ID of the admin relationship, void if relationship already exists
    * and -1 in case of error.
   */
	public function addAdmin($user_id, $game_id) {
		
		if ($this->debug) {
			$this->writeDebug('addAdmin', 'user: ' . $user_id . ' game: ' . $game_id);
		}
		
		if (empty($this->games)) {
			$this->initDB();
		}
		$index = 0;
		// First check if connection already exists
		foreach ($this->games as $game) {
			if ($game['game_id'] == $game_id) {
				foreach ($game['admins'] as $admin) {
					// If relationship is already found return
					if ($admin['user_id'] == $user_id) {
						return;
						break;
					}
				}
				break;
			}
			$index++;
		}
		// If there is not such game return false
		if (!isset($this->games[$index])) {
			return -1;
		}
		$relationship_id = 0;
		$new_admin = array();
		$new_admin['user_id'] = $user_id;
		$new_admin['relationship_id'] = $this->latest_ids['admin_relationship_id'] + 1;
		
		$this->games[$index]['admins'][] = $new_admin;
		
		// Update xml file
		if ($this->writeXMLGames($this->xml_folder . 'games.xml', $this->games)) {
			$_SESSION['xmldb']['games'] = $this->games;
			$this->latest_ids['admin_relationship_id'] = $new_admin['relationship_id'];
			$this->writeXMLLatestIds($this->xml_folder . 'latest_ids.xml', $this->latest_ids);
			$relationship_id  = $new_admin['relationship_id'];
		}
		return $relationship_id;
	}
	
	/**
	 * Removes an administrator of the game.
    * @param double $user_id ID of the admin to be removed.
    * @param double $game_id ID of the game of which to remove admin.
    * @return bool Was the operation successful?
   */
	public function removeAdmin($user_id, $game_id) {
		
		if ($this->debug) {
			$this->writeDebug('removeAdmin', 'user: ' . $user_id . ' game: ' . $game_id);
		}
		if (empty($this->games)) {
			$this->initDB();
		}
		$deleted = false;
		// First check if connection exists
		foreach ($this->games as &$game) {
			
			if ($game['game_id'] == $game_id) {
				
				$index = 0;
				foreach ($game['admins'] as $admin) {
					
					// If relationship is found return
					if ($admin['user_id'] == $user_id) {
						
						array_splice($game['admins'], $index, 1);
						$deleted = true;
						$this->writeXMLGames($this->xml_folder . 'games.xml', $this->games);
						$_SESSION['xmldb']['games'] = $this->games;
						
						break;
					}
					$index++;
				}
				break;
			}
		}
		
		return $deleted;
	}
	
	/**
	 * Ties a new card to a player profile as being played.
    * @param double $profile_id ID of the player profile of which to add played card.
    * @param double $card_id ID of the card to be played.
    * @param int $ordinal_nbr Ordinal number of the played card.
    * @return bool Was the operation successful?
   */
	public function playCard($profile_id, $card_id, $ordinal_nbr) {
		
		if ($this->debug) {
			$this->writeDebug('playCard', 'profile: ' . $profile_id . ' card: ' . $card_id . ' ordinal:' . $ordinal_nbr);
		}
		if (empty($this->games) || empty($this->users)) {
			$this->initDB();
		}
		$game_id = $this->current_game;
		
		if (empty($this->current_profile)) {
			$this->getProfile($profile_id);
		}
		if (empty($this->profiles)) {
			if ($this->current_game > 0) {
				$game_id = $this->current_game;
			}
			elseif (isset($_SESSION['admin']['game']['game_id'])) {
				$game_id = $_SESSION['admin']['game']['game_id'];
			}
			$this->getAllProfiles($game_id);
		}
		$card_relationship = 0;
		
		if (!empty($this->current_profile)) {

			if ($this->current_profile['profile_id'] == $profile_id) {

				$new_card = array();
				$new_card['relationship_id'] = $this->latest_ids['played_card_id'] + 1;
				$new_card['card_id'] = $card_id;
				$new_card['choice_id'] = 0;
				$new_card['result_id'] = 0;
				$new_card['ordinal'] = $ordinal_nbr;
				$this->current_profile['played_cards'][] = $new_card;
				$card_relationship = $new_card['relationship_id'];
				$_SESSION['xmldb']['current_profile'] = $this->current_profile;
			}
		}
		if ($card_relationship < 1) {

			foreach ($this->profiles as &$profile) {
			
				if ($profile['profile_id'] == $profile_id) {
					
					$new_card = array();
					$new_card['relationship_id'] = $this->latest_ids['played_card_id'] + 1;
					$new_card['card_id'] = $card_id;
					$new_card['choice_id'] = 0;
					$new_card['result_id'] = 0;
					$new_card['ordinal'] = $ordinal_nbr;
					$profile['played_cards'][] = $new_card;
					$card_relationship = $new_card['relationship_id'];
					$this->current_profile = $profile;
					$_SESSION['xmldb']['current_profile'] = $this->current_profile;
					break;
				}
			}
		}
		// Update xml file
		if ($this->writeXMLProfile($this->xml_folder . 'games/' . $game_id . '/player_profiles/profile_' . $profile_id . '.xml', $this->current_profile)) {
			$this->latest_ids['played_card_id'] = $card_relationship;
			$this->writeXMLLatestIds($this->xml_folder . 'latest_ids.xml', $this->latest_ids);
		}
		else {
			return false;
		}
		return true;
	}
	
	/**
	 * Removes card user has played.
    * @param double $profile_id ID of the player profile of which card is to be removed.
    * @param double $card_id ID of the card to be removed.
    * @param int $ordinal_nbr Ordinal number of the played card to be removed.
    * @return bool Was the operation successful?
   */
	public function removePlayedCard($profile_id, $card_id, $ordinal_nbr) {
		
		if ($this->debug) {
			$this->writeDebug('removePlayedCard', 'profile: ' . $profile_id . ' card: ' . $card_id . ' ordinal:' . $ordinal_nbr);
		}
		if (empty($this->games) || empty($this->users)) {
			$this->initDB();
		}
		$game_id = $this->current_game;
		
		if (empty($this->current_profile)) {
			$this->getProfile($profile_id);
		}
		
		if (empty($this->profiles)) {
			
			if ($this->current_game > 0) {
				$game_id = $this->current_game;
			}
			elseif (isset($_SESSION['admin']['game']['game_id'])) {
				$game_id = $_SESSION['admin']['game']['game_id'];
			}
			$this->getAllProfiles($game_id);
		}
		
		if (empty($this->current_profile)) {
			foreach ($this->profiles as $profile) {
				if ($profile['profile_id'] == $profile_id) {
				
					$this->current_profile = $profile;
					$_SESSION['xmldb']['current_profile'] = $this->current_profile;
				}
			}
		}	
		
		$deleted = false;
		$index = 0;
		
		if ($this->current_profile['profile_id'] == $profile_id) {
			foreach ($this->current_profile['played_cards'] as &$played_card) {
				if ($played_card['card_id'] == $card_id && $played_card['ordinal'] == $ordinal_nbr) {
					array_splice($this->current_profile['played_cards'], $index, 1);
					$deleted = true;
					$this->writeXMLProfile($this->xml_folder . 'games/' . $game_id . '/player_profiles/profile_' . $profile_id . '.xml', $this->current_profile);
					$_SESSION['xmldb']['current_profile'] = $this->current_profile;
					break;
				}
				$index++;
			}
		}
		return $deleted;
	}
	
	/**
	 * Updates information about card user has played.
    * @param double $profile_id ID of the player profile of which card is to be updated.
    * @param double $card_id ID of the card.
    * @param double $choice_id ID of the choice user has made. 0 means none.
    * @param double $result_id ID of the result related to choice. 0 means none.
    * @param int $ordinal_nbr Ordinal number of the played card.
    * @return void
   */
	public function updatePlayedCard($profile_id, $card_id, $choice_id, $result_id, $ordinal_nbr) {
		
		if ($this->debug) {
			$this->writeDebug('updatePlaydedCard', 'profile: ' . $profile_id . ' card: ' . $card_id . ' choice: ' . $choice_id . ' result: ' . $result_id . ' ordinal:' . $ordinal_nbr);
		}
		if (empty($this->games) || empty($this->users)) {
			$this->initDB();
		}
		$game_id = $this->current_game;
		
		if (empty($this->current_profile)) {
			$this->getProfile($profile_id);
		}
		
		if (empty($this->profiles)) {

			if ($this->current_game > 0) {
				$game_id = $this->current_game;
			}
			elseif (isset($_SESSION['admin']['game']['game_id'])) {
				$game_id = $_SESSION['admin']['game']['game_id'];
			}
			$this->getAllProfiles($game_id);
		}
		
		if (empty($this->current_profile)) {
			
			foreach ($this->profiles as $profile) {
				if ($profile['profile_id'] == $profile_id) {
				
					$this->current_profile = $profile;
					$_SESSION['xmldb']['current_profile'] = $this->current_profile;
				}
			}
		}
		
		if ($this->current_profile['profile_id'] == $profile_id) {
			foreach ($this->current_profile['played_cards'] as &$played_card) {
				if ($played_card['card_id'] == $card_id && $played_card['ordinal'] == $ordinal_nbr) {
					
					$played_card['choice_id'] = $choice_id;
					$played_card['result_id'] = $result_id;
					break;
				}
			}
			$this->writeXMLProfile($this->xml_folder . 'games/' . $game_id . '/player_profiles/profile_' . $profile_id . '.xml', $this->current_profile);
			$_SESSION['xmldb']['current_profile'] = $this->current_profile;
		}
	}
	
	/**
	 * Starts new simulation.
    * @param double $game_id ID of the game.
    * @param string $username User name of the player.
    * @return array|false Game information in array or false if function failed.
   */
	public function startNewGame($game_id, $username) {
		
		if ($this->debug) {
			$this->writeDebug('startNewGame', 'game: ' . $game_id . ' user: ' . $username);
		}
		
		if ($game_id > 0) {
			// Get game array
			$game = $this->getGame($game_id);
		
			if (empty($game) || empty($username)) {
				return false;
			}
			// Draw scenario (1-5)
			$scenario = mt_rand(1, 5);
			$profile_id = $this->createNewProfile($game_id, $username, $scenario);
			//$profile_id = $this->createNewProfile($game_id, $username);
			if ($profile_id > 0) {
				$game['profile'] = $profile_id;
				return $game;
			}
		}
		return false;
	}
	
	/**
	 * Creates a new player profile.
    * @param double $game_id ID of the game which to create a new profile.
    * @param string $username The user name of a new profile.
    * @param int $scenario ID of the scenario.
    * @return int|false ID of the created profile or false in case of error.
   */
	public function createNewProfile($game_id, $username, $scenario) {
		
		if ($this->debug) {
			$this->writeDebug('createNewProfile', 'game: ' . $game_id . ' user: ' . $username . ' scenario: ' . $scenario);
		}
		
		if ($game_id > 0 && !empty($username)) {
			
			$user_id = 0;
			if (isset($_SESSION['user']['user_id']) && isset($_SESSION['isLogged']) && $_SESSION['isLogged']) {
				$user_id = $_SESSION['user']['user_id'];
			}
			
			$new_profile = array();
			$new_profile['game_id'] = $game_id;
			$new_profile['profile_id'] = $this->latest_ids['profile_id'] + 1;
			$new_profile['user_id'] = $user_id;
			$new_profile['player_name'] = $username;
			
			$mysqldate = date( 'Y-m-d H:i:s' );
			$new_profile['created'] = $mysqldate;
			$new_profile['cancellations'] = 0;
			$new_profile['is_panic'] = 0;
			$new_profile['game_ended'] = 0;
			$new_profile['scenario'] = $scenario;
			$new_profile['played_cards'] = array();
			
			if (empty($this->profiles)) {
				$this->getAllProfiles($game_id);
			}
			$this->profiles[] = $new_profile;
			$this->current_profile = $new_profile;
			$profile_id = 0;
		
			// Update xml file
			if ($this->writeXMLProfile($this->xml_folder . 'games/' . $game_id . '/player_profiles/profile_' . $new_profile['profile_id'] . '.xml', $this->current_profile)) {
				$_SESSION['xmldb']['profiles'] = $this->profiles;
				$_SESSION['xmldb']['current_profile'] = $this->current_profile;
				$this->latest_ids['profile_id'] = $new_profile['profile_id'];
				
				$this->writeXMLLatestIds($this->xml_folder . 'latest_ids.xml', $this->latest_ids);
				$profile_id = $new_profile['profile_id'];
			}
			return $profile_id;
		}
		return false;
	}
	
	/**
	 * Deletes a player profile.
    * @param double $profile_id ID of the player profile to be deleted.
    * @return bool Was the operation successful?
   */
	public function deleteProfile($profile_id) {
		
		if ($this->debug) {
			$this->writeDebug('deleteProfile', 'profile: ' . $profile_id);
		}
		
		if (empty($this->games)) {
    		$this->initDB();
    	}
		$game_id = 0;
		
		if (!empty($this->profiles)) {
			$index = 0;
			foreach ($this->profiles as &$prof) {
				if ($prof['profile_id'] == $profile_id) {
					$game_id = $prof['game_id'];
					array_splice($this->profiles, $index, 1);
					$this->current_profile = array();
					unset($_SESSION['xmldb']['current_profile']);
					$_SESSION['xmldb']['profiles'] = $this->profiles;
					break;
				}
				$index++;
			}
			unset($prof);
		}
		
		if ($game_id > 0) {
			
			if (is_dir($this->xml_folder . 'games/' . $game_id)) {
				
				if (is_dir($this->xml_folder . 'games/' . $game_id . '/player_profiles')) {
					
					if (is_file($this->xml_folder . 'games/' . $game_id . '/player_profiles/profile_' . $profile_id . '.xml')) {
						
						unlink($this->xml_folder . 'games/' . $game_id . '/player_profiles/profile_' . $profile_id . '.xml');
						return true;
					}
				}
			}
		}
		else {
			foreach ($this->games as $game) {
				
				if (is_dir($this->xml_folder . 'games/' . $game['game_id'])) {
				
					if (is_dir($this->xml_folder . 'games/' . $game['game_id'] . '/player_profiles')) {
					
						if (is_file($this->xml_folder . 'games/' . $game['game_id'] . '/player_profiles/profile_' . $profile_id . '.xml')) {
					
							unlink($this->xml_folder . 'games/' . $game['game_id'] . '/player_profiles/profile_' . $profile_id . '.xml');
							return true;
						}
					}
				}
			}
		}
		return false;
	}
	
	/**
	 * Updates number of rewindings done to a player profile.
    * @param double $profile_id ID of the player profile to be updated.
    * @param int $cancellations Number of rewindings done.
    * @return void
   */
	public function updatePlayerProfileCancellations($profile_id, $cancellations) {
		
		if ($this->debug) {
			$this->writeDebug('updatePlayerProfileCancellations', 'profile: ' . $profile_id);
		}
		
		if (empty($this->games) || empty($this->users)) {
			$this->initDB();
		}
		$game_id = $this->current_game;
		
		if (empty($this->current_profile)) {
			$this->getProfile($profile_id);
		}
		
		if (empty($this->profiles)) {
			
			if ($this->current_game > 0) {
				$game_id = $this->current_game;
			}
			elseif (isset($_SESSION['admin']['game']['game_id'])) {
				$game_id = $_SESSION['admin']['game']['game_id'];
			}
			$this->getAllProfiles($game_id);
		}
		
		if (empty($this->current_profile)) {
			foreach ($this->profiles as $profile) {
				if ($profile['profile_id'] == $profile_id) {
				
					$this->current_profile = $profile;
					$_SESSION['xmldb']['current_profile'] = $this->current_profile;
				}
			}
		}
		
		if ($this->current_profile['profile_id'] == $profile_id) {
			
			$this->current_profile['cancellations'] = $cancellations;
			$this->writeXMLProfile($this->xml_folder . 'games/' . $game_id . '/player_profiles/profile_' . $profile_id . '.xml', $this->current_profile);
			$_SESSION['xmldb']['current_profile'] = $this->current_profile;
		}
	}
	
	/**
	 * Links user id to player profile.
    * @param double $profile_id ID of the player profile to be updated.
    * @param double $user_id ID of the user to be linked with profile.
    * @return void
   */
	public function updatePlayerProfileUser($profile_id, $user_id) {
		
		if ($this->debug) {
			$this->writeDebug('updatePlayerProfileUser', 'profile: ' . $profile_id . ' user: ' . $user_id);
		}
		
		if (empty($this->games) || empty($this->users)) {
			$this->initDB();
		}
		$game_id = $this->current_game;
		
		if (empty($this->current_profile)) {
			$this->getProfile($profile_id);
		}
		
		if (empty($this->profiles)) {
			
			if ($this->current_game > 0) {
				$game_id = $this->current_game;
			}
			elseif (isset($_SESSION['admin']['game']['game_id'])) {
				$game_id = $_SESSION['admin']['game']['game_id'];
			}
			$this->getAllProfiles($game_id);
		}
		
		if (empty($this->current_profile)) {
			foreach ($this->profiles as $profile) {
				if ($profile['profile_id'] == $profile_id) {
				
					$this->current_profile = $profile;
					$_SESSION['xmldb']['current_profile'] = $this->current_profile;
					break;
				}
			}
		}
		
		if ($this->current_profile['profile_id'] == $profile_id) {
			
			$this->current_profile['user_id'] = $user_id;
			$this->writeXMLProfile($this->xml_folder . 'games/' . $game_id . '/player_profiles/profile_' . $profile_id . '.xml', $this->current_profile);
			$_SESSION['xmldb']['current_profile'] = $this->current_profile;
		}
	}
	
	/**
	 * Sets value of panic flag to a player profile.
    * @param double $profile_id ID of the player profile to be updated.
    * @param int $is_panic 0: No panic, 1: panic.
    * @return void
   */
	public function updatePlayerProfilePanic($profile_id, $is_panic) {
		
		if ($this->debug) {
			$this->writeDebug('updatePlayerProfilePanic', 'profile: ' . $profile_id . ' panic: ' . $is_panic);
		}
		if (empty($this->games) || empty($this->users)) {
			$this->initDB();
		}
		$game_id = $this->current_game;
		
		if (empty($this->current_profile)) {
			$this->getProfile($profile_id);
		}
		
		if (empty($this->profiles)) {
			
			if ($this->current_game > 0) {
				$game_id = $this->current_game;
			}
			elseif (isset($_SESSION['admin']['game']['game_id'])) {
				$game_id = $_SESSION['admin']['game']['game_id'];
			}
			$this->getAllProfiles($game_id);
		}
		
		if (empty($this->current_profile)) {
			foreach ($this->profiles as $profile) {
				if ($profile['profile_id'] == $profile_id) {
				
					$this->current_profile = $profile;
					$_SESSION['xmldb']['current_profile'] = $this->current_profile;
					break;
				}
			}
		}
		
		if ($this->current_profile['profile_id'] == $profile_id) {
			
			$this->current_profile['is_panic'] = $is_panic;
			$this->writeXMLProfile($this->xml_folder . 'games/' . $game_id . '/player_profiles/profile_' . $profile_id . '.xml', $this->current_profile);
			$_SESSION['xmldb']['current_profile'] = $this->current_profile;
		}
	}
	
	/**
	 * Sets value of scenario to a player profile.
    * @param double $profile_id ID of the player profile to be updated.
    * @param int $scenario (1-5)
    * @return void
   */
	public function updatePlayerProfileScenario($profile_id, $scenario) {
		
		if ($this->debug) {
			$this->writeDebug('updatePlayerProfilePanic', 'profile: ' . $profile_id . ' scenario: ' . $scenario);
		}
		if (empty($this->games) || empty($this->users)) {
			$this->initDB();
		}
		$game_id = $this->current_game;
		
		if (empty($this->current_profile)) {
			$this->getProfile($profile_id);
		}
		
		if (empty($this->profiles)) {
			
			if ($this->current_game > 0) {
				$game_id = $this->current_game;
			}
			elseif (isset($_SESSION['admin']['game']['game_id'])) {
				$game_id = $_SESSION['admin']['game']['game_id'];
			}
			$this->getAllProfiles($game_id);
		}
		
		if (empty($this->current_profile)) {
			foreach ($this->profiles as $profile) {
				if ($profile['profile_id'] == $profile_id) {
				
					$this->current_profile = $profile;
					$_SESSION['xmldb']['current_profile'] = $this->current_profile;
					break;
				}
			}
		}
		
		if ($this->current_profile['profile_id'] == $profile_id) {
			
			$this->current_profile['scenario'] = $scenario;
			$this->writeXMLProfile($this->xml_folder . 'games/' . $game_id . '/player_profiles/profile_' . $profile_id . '.xml', $this->current_profile);
			$_SESSION['xmldb']['current_profile'] = $this->current_profile;
		}
	}
	
	/**
	 * Marks game ended or not to a player profile.
    * @param double $profile_id ID of the player profile to be updated.
    * @param int $game_ended 0: Game hasn't ended, 1: game is ended.
    * @return void
   */
	public function updatePlayerProfileGameEnded($profile_id, $game_ended) {
		
		if ($this->debug) {
			$this->writeDebug('updatePlayerProfileGameEnded', 'profile: ' . $profile_id . ' ended: ' . $game_ended);
		}
		if (empty($this->games) || empty($this->users)) {
			$this->initDB();
		}
		$game_id = $this->current_game;
		
		if (empty($this->current_profile)) {
			$this->getProfile($profile_id);
		}
		
		if (empty($this->profiles)) {
			
			if ($this->current_game > 0) {
				$game_id = $this->current_game;
			}
			elseif (isset($_SESSION['admin']['game']['game_id'])) {
				$game_id = $_SESSION['admin']['game']['game_id'];
			}
			$this->getAllProfiles($game_id);
		}
		
		if (empty($this->current_profile)) {
			foreach ($this->profiles as $profile) {
				if ($profile['profile_id'] == $profile_id) {
				
					$this->current_profile = $profile;
					$_SESSION['xmldb']['current_profile'] = $this->current_profile;
					break;
				}
			}
		}
		
		if ($this->current_profile['profile_id'] == $profile_id) {
			
			$this->current_profile['game_ended'] = $game_ended;
			$this->writeXMLProfile($this->xml_folder . 'games/' . $game_id . '/player_profiles/profile_' . $profile_id . '.xml', $this->current_profile);
			$_SESSION['xmldb']['current_profile'] = $this->current_profile;
		}
	}
	
	/**
	 * Creates a new card with base information.
    * @param double $card_external_id ID of the card to be shown in editor (different than actual ID of the card).
    * @param double $game_id ID of the game in which card is created.
    * @param int $repetitive Flag to tell whether or not card can be played multiple times.
    * @param double $creator_id ID of the user who created the card.
    * @param string $tags Tags of the card.
    * @param string $stat_requirements Stat requirements of a card.
    * @param string $comment Comment creator has added to card.
    * @param string $attachment Name of the image linked to card.
    * @return double ID of the created card.
   */
	public function createCard($card_external_id, $game_id, $repetitive, $creator_id, $tags, $stat_requirements='', $comment='', $attachment='', $layout='normal') {
		
		if ($this->debug) {
			$this->writeDebug('createCard', '');
		}
		
		if (empty($this->cards)) {
			$this->initDB();
		}
		
		$this->getAllCards($game_id);
		$new_card = array();
		$new_card['card_id'] = $this->latest_ids['card_id'] + 1;
		$new_card['card_external_id'] = $card_external_id;
		$new_card['game_id'] = $game_id;
		$new_card['enabled'] = 1;
		$new_card['repetitive'] = $repetitive;
		$mysqldate = date( 'Y-m-d H:i:s' );
		$new_card['created'] = $mysqldate;
		$new_card['modified'] = '';
		$new_card['creator_id'] = $creator_id;
		$new_card['editor_id'] = 0;
		$new_card['tags'] = $tags;
		$new_card['stat_requirements'] = $stat_requirements;
		$new_card['comment'] = $comment;
		$new_card['attachment'] = $attachment;
		$new_card['layout'] = $layout;
		
		$new_card['related_tags'] = array();
		$new_card['related_stats'] = array();
		
		$new_card['translations'] = array();
		
		$new_card['choices'] = array();
		
		$new_card['creator_name'] = '';
		$new_card['editor_name'] = '';
		if ($new_card['creator_id'] > 0) {
			$user = $this->getUser($creator_id);
			$new_card['creator_name'] = $user['username'];
		}						
		$this->cards[] = $new_card;
		
		$card_id = 0;
		
		// Update xml file
		if ($this->writeXMLCards($this->xml_folder . 'games/' . $game_id . '/cards_' . $game_id . '.xml', $this->cards)) {
			$_SESSION['xmldb']['cards'] = $this->cards;
			$this->latest_ids['card_id'] = $new_card['card_id'];
			$this->writeXMLLatestIds($this->xml_folder . 'latest_ids.xml', $this->latest_ids);
			$card_id = $new_card['card_id'];
			// Update latest_card_id to game
			foreach ($this->games as &$game) {
				if ($game['latest_card_id'] < $card_external_id) {
					$game['latest_card_id'] = $card_external_id;
					$this->writeXMLGames($this->xml_folder . 'games.xml', $this->games);
					break;
				}
			}
		}
		$this->updateCardRelatedTagsAndStats($card_id);
		
		return $card_id;
	}
	
	/**
	 * Updates card's base information.
	 * @param double $card_id The actual ID of the card to be updated.
    * @param double $card_external_id ID of the card to be shown in editor (different than actual ID of the card).
    * @param int $repetitive Flag to tell whether or not card can be played multiple times.
    * @param double $editor_id ID of the user who edited the card.
    * @param string $tags Tags of the card.
    * @param string $stat_requirements Stat requirements of a card.
    * @param string $comment Comment creator has added to card.
    * @param string $attachment A name of the image linked to card.
    * @param double $game_id ID of the game in which card is related.
    * @return void
   */
	public function editCard($card_id, $card_external_id, $repetitive, $editor_id, $tags, $stat_requirements, $comment, $attachment, $layout='normal', $game_id) {
		
		if ($this->debug) {
			$this->writeDebug('editCard', '');
		}
		if (empty($this->games) || empty($this->users)) {
			$this->initDB();
		}
		if ($this->current_game != $game_id || empty($this->cards)) {
			$this->getAllCards($game_id);
		}
		
		foreach ($this->cards as &$card) {
			
			if ($card['card_id'] == $card_id) {
				$card['card_external_id'] = $card_external_id;
				$card['repetitive'] = $repetitive;
				$mysqldate = date( 'Y-m-d H:i:s' );
				$card['modified'] = $mysqldate;
				$card['editor_id'] = $editor_id;
				$card['tags'] = $tags;
				$card['stat_requirements'] = $stat_requirements;
				$card['comment'] = $comment;
				$card['attachment'] = $attachment;
				$card['layout'] = $layout;
				$card['editor_name'] = '';
				if ($card['editor_id'] > 0) {
					$user = $this->getUser($editor_id);
					$card['editor_name'] = $user['username'];
				}
				break;
			}
		}
		// Update xml file
		if ($this->writeXMLCards($this->xml_folder . 'games/' . $game_id . '/cards_' . $game_id . '.xml', $this->cards)) {
			$_SESSION['xmldb']['cards'] = $this->cards;
			// empty latest card
			$this->latest_card = array();
			$_SESSION['xmldb']['latest_card'] = array();
			// Update latest_card_id to game
			foreach ($this->games as &$game) {
				if ($game['latest_card_id'] < $card_external_id) {
					$game['latest_card_id'] = $card_external_id;
					$this->writeXMLGames($this->xml_folder . 'games.xml', $this->games);
					break;
				}
			}
		}
		$this->updateCardRelatedTagsAndStats($card_id);
	}
	
	/**
	 * Sets or unsets card to be drawn only when it's ID is defined in criteria.
	 * @param double $card_id The actual ID of the card to be updated.
	 * @param int $hidden Flag 0: false, 1: true
	 * @return void
	 */
	function toggleCardVisibility($card_id, $hidden) {
		
		if ($this->debug) {
			$this->writeDebug('toggleCardVisibility', '');
		}
		if (empty($this->games) || empty($this->users)) {
			$this->initDB();
		}
		$game_id = $this->current_game;
		
		if (empty($this->cards)) {
			$game_id = 0;
			if ($this->current_game > 0) {
				$game_id = $this->current_game;
			}
			elseif (isset($_SESSION['admin']['game']['game_id'])) {
				$game_id = $_SESSION['admin']['game']['game_id'];
			}
			$this->getAllCards($game_id);
		}
		
		foreach ($this->cards as &$card) {
			
			if ($card['card_id'] == $card_id) {
				
				$card['hidden'] = $hidden;
				break;
			}
		}
		// Update xml file
		if ($this->writeXMLCards($this->xml_folder . 'games/' . $game_id . '/cards_' . $game_id . '.xml', $this->cards)) {
			$_SESSION['xmldb']['cards'] = $this->cards;
			// empty latest card
			$this->latest_card = array();
			$_SESSION['xmldb']['latest_card'] = array();
		}
	}
	
	/**
	 * Adds new translation of a card.
	 * @param double $card_id ID of the card in which to add new translation.
	 * @param string $card_name Name of the card in translation language.
	 * @param string $description Description of the card in translation language.
	 * @param string $expert_advice Expert advice of the card in translation language.
	 * @param string $lang Language code of the new translation.
    * @param int $is_default Flag that determines whether or not new translation is in default language. 0: false, 1: true
    * @return double ID of the new translation or the old one if it already existed
    * and -1 in case of error.
   */
	public function addCardTranslation($card_id, $card_name, $description, $expert_advice, $lang, $is_default) {
		
		if ($this->debug) {
			$this->writeDebug('addCardTranslation', '');
		}
		if (empty($this->games) || empty($this->users)) {
			$this->initDB();
		}
		$game_id = $this->current_game;
		
		if (empty($this->cards)) {
			$game_id = 0;
			if ($this->current_game > 0) {
				$game_id = $this->current_game;
			}
			elseif (isset($_SESSION['admin']['game']['game_id'])) {
				$game_id = $_SESSION['admin']['game']['game_id'];
			}
			$this->getAllCards($game_id, $lang);
		}
		$cards = $this->cards;
		$card_index = 0;
		$trans_id = 0;
		
		foreach ($cards as &$card) {
			
			if ($card['card_id'] == $card_id) {
				// First check if card already has translation in current language
				foreach ($card['translations'] as $trans) {
				
					if ($trans['language_code'] == $lang) {
						$trans_id = $trans['translation_id'];
						break;
					}
				}
				break;
			}
			$card_index++;
		}
		// If translation is found return translation_id
		if ($trans_id > 0) {
			return $trans_id;
		}
		
		if ($card_index >= count($this->cards)) {
			return -1;
		}
		
		$new_trans = array();
		$new_trans['translation_id'] = $this->latest_ids['card_translation_id'] + 1;
		$this->latest_ids['card_translation_id'] = $trans_id;
		$new_trans['card_name'] = $card_name;
		$new_trans['description'] = $description;
		$new_trans['expert_advice'] = $expert_advice;
		$new_trans['language_code'] = $lang;
		$new_trans['is_default'] = $is_default;
		
		$this->cards[$card_index]['translations'][] = $new_trans;
		
		// Update xml file
		if ($this->writeXMLCards($this->xml_folder . 'games/' . $game_id . '/cards_' . $game_id . '.xml', $this->cards)) {
			$_SESSION['xmldb']['cards'] = $this->cards;
			$this->latest_ids['card_translation_id'] = $new_trans['translation_id'];
			$this->writeXMLLatestIds($this->xml_folder . 'latest_ids.xml', $this->latest_ids);
			$trans_id = $new_trans['translation_id'];
		}
		
		return $trans_id;
	}
	
	/**
	 * Updates translation of a card.
	 * @param double $card_id ID of the card in which translation to update.
	 * @param string $card_name Name of the card in translation language.
	 * @param string $description Description of the card in translation language.
	 * @param string $expert_advice Expert advice of the card in translation language.
	 * @param string $lang Language code of the new translation.
    * @param int $is_default Flag that determines whether or not new translation is in default language. 0: false, 1: true
    * @param string $default_lang Language code of the default language.
    * @return void
   */
	public function updateCardTranslation($card_id, $card_name, $description, $expert_advice, $lang, $is_default, $default_lang) {
		
		if ($this->debug) {
			$this->writeDebug('updateCardTranslation', '');
		}
		if (empty($this->games) || empty($this->users)) {
			$this->initDB();
		}
		$game_id = $this->current_game;
		if (empty($this->cards)) {
			$game_id = 0;
			if ($this->current_game > 0) {
				$game_id = $this->current_game;
			}
			elseif (isset($_SESSION['admin']['game']['game_id'])) {
				$game_id = $_SESSION['admin']['game']['game_id'];
			}
			$this->getAllCards($game_id, $lang);
		}		
		
		foreach ($this->cards as &$card) {
			
			if ($card['card_id'] == $card_id) {
				
				foreach ($card['translations'] as &$trans) {
					// If default language is same as translation language then mark all other translations to false
					if ($default_lang == $lang) {
						$trans['is_default'] = 0;
					}
					// Update the right translation
					if ($trans['language_code'] == $lang) {
						$trans['card_name'] = $card_name;
						$trans['description'] = $description;
						$trans['expert_advice'] = $expert_advice;
						$trans['is_default'] = $is_default;
					}
				}
				break;
			}
		}
		// Update xml file
		if ($this->writeXMLCards($this->xml_folder . 'games/' . $game_id . '/cards_' . $game_id . '.xml', $this->cards)) {
			$_SESSION['xmldb']['cards'] = $this->cards;
		}
	}
	
	/**
	 * Adds choices and results to card.
	 * @param double $card_id The actual ID of the card to be updated.
    * @param array $choices Choices to be added.
    * @param array $results Results to be added.
    * @param string $lang Language code of the choices and results.
    * @param int $is_default Flag that determines whether or not choices are in default language. 0: false, 1: true
    * @return void|false Returns false in case of errors.
   */
	public function addCardChoicesAndResults($card_id, $choices, $results, $lang, $is_default) {
		
		if ($this->debug) {
			$this->writeDebug('addCardChoicesAndResults', '');
		}
		if ($card_id <= 0) {
			return false;
		}
		
		if (empty($this->games) || empty($this->users)) {
			$this->initDB();
		}
		$game_id = $this->current_game;
		if (empty($this->cards)) {
			$game_id = 0;
			if ($this->current_game > 0) {
				$game_id = $this->current_game;
			}
			elseif (isset($_SESSION['admin']['game']['game_id'])) {
				$game_id = $_SESSION['admin']['game']['game_id'];
			}
			$this->getAllCards($game_id, $lang);
		}
		$choice_id = $this->latest_ids['choice_id'];
		$choice_trans_id = $this->latest_ids['choice_translation_id'];
		$result_id = $this->latest_ids['result_id'];
		$result_trans_id = $this->latest_ids['result_translation_id'];
		
		foreach ($this->cards as &$card) {
			
			if ($card['card_id'] == $card_id) {
		
				$choice_counter = 0;
				foreach ($choices as $choice) {
					
					$new_choice = array();
					$new_choice['choice_id'] = $choice_id + 1;
					$choice_id = $new_choice['choice_id'];
					$new_choice['ordinal'] = $choice_counter+1;
					$new_choice['translations'] = array();
					
					$new_translation = array();
					$new_translation['card_id'] = $card_id;
					$new_translation['translation_id'] = $choice_trans_id + 1;
					$choice_trans_id = $new_translation['translation_id'];
					$new_translation['is_default'] = $is_default;
					$new_translation['language_code'] = $lang;
					$new_translation['text'] = $choice['text'];
					$new_choice['translations'][] = $new_translation;
					
					$new_choice['results'] = array();
					
					$result_counter = 0;
					foreach ($results[$choice_counter] as $result) {
						
						$new_result = array();
						$new_result['result_id'] = $result_id + 1;
						$result_id = $new_result['result_id'];
						$new_result['ordinal'] = $result_counter+1;
						$new_result['weight'] = $result['weight'];
						$new_result['stat_changes'] = $result['stat_changes'];
						$new_result['next_criteria'] = $result['next_criteria'];
						$result['translations'] = array();
						
						$new_translation = array();
						$new_translation['card_id'] = $card_id;
						$new_translation['translation_id'] = $result_trans_id + 1;
						$result_trans_id = $new_translation['translation_id'];
						$new_translation['is_default'] = $is_default;
						$new_translation['language_code'] = $lang;
						$new_translation['text'] = $result['text'];
						$new_result['translations'][] = $new_translation;
						
						$result_counter++;
						$new_choice['results'][] = $new_result;
					}
					$choice_counter++;
					
					$card['choices'][] = $new_choice;
				}
				break;
			}
		}
		// Update xml file
		if ($this->writeXMLCards($this->xml_folder . 'games/' . $game_id . '/cards_' . $game_id . '.xml', $this->cards)) {
			$_SESSION['xmldb']['cards'] = $this->cards;
			$this->latest_ids['choice_id'] = $choice_id;
			$this->latest_ids['choice_translation_id'] = $choice_trans_id;
			$this->latest_ids['result_id'] = $result_id;
			$this->latest_ids['result_translation_id'] = $result_trans_id;
			$this->writeXMLLatestIds($this->xml_folder . 'latest_ids.xml', $this->latest_ids);
		}
		$this->updateCardRelatedTagsAndStats($card_id);
	}
	
	/**
	 * Updates choices and results of the card.
	 * @param double $card_id The actual ID of the card to be updated.
    * @param array $choices Choices to be updated.
    * @param array $results Results to be updated.
    * @param string $lang Language code of the choices and results.
    * @param int $is_default Flag that determines whether or not choices are in default language. 0: false, 1: true
    * @param string $default_lang The default language of the choices and results.
    * @return void|false Returns false in case of errors.
   */
	public function updateCardChoicesAndResults($card_id, $choices, $results, $lang, $is_default, $default_lang) {
		
		if ($this->debug) {
			$this->writeDebug('updateCardChoicesAndResults', '');
		}
		if ($card_id <= 0) {
			return false;
		}
		if (empty($this->games) || empty($this->users)) {
			$this->initDB();
		}
		$game_id = $this->current_game;
		if (empty($this->cards)) {
			$game_id = 0;
			if ($this->current_game > 0) {
				$game_id = $this->current_game;
			}
			elseif (isset($_SESSION['admin']['game']['game_id'])) {
				$game_id = $_SESSION['admin']['game']['game_id'];
			}
			$this->getAllCards($game_id, $lang);
		}
		
		$choice_id = $this->latest_ids['choice_id'];
		$choice_trans_id = $this->latest_ids['choice_translation_id'];
		$result_id = $this->latest_ids['result_id'];
		$result_trans_id = $this->latest_ids['result_translation_id'];
		
		foreach ($this->cards as &$card) {
			
			if ($card['card_id'] == $card_id) {
				
				$new_choices_count = count($choices);
				$old_choices_count = count($card['choices']);
				$limit = 0;
				
				// Remove choices and results that aren't needed anymore
				if ($new_choices_count < $old_choices_count) {
	
					$limit = $new_choices_count;
					
					for ($i = $new_choices_count; $i < $old_choices_count; $i++) {
						// Delete choices & related results
						array_splice($card['choices'], $i, 1);
						$i--;
						$old_choices_count--;
					}
				}
				// Create new choices and results
				else if ($new_choices_count > $old_choices_count) {

					$limit = $old_choices_count;
					for ($i = $old_choices_count; $i < $new_choices_count; $i++) {

						$new_choice = array();
						$new_choice['choice_id'] = $choice_id + 1;
						$choice_id = $new_choice['choice_id'];
						$new_choice['ordinal'] = $i+1;
						$new_choice['translations'] = array();
					
						$new_translation = array();
						$new_translation['card_id'] = $card_id;
						$new_translation['translation_id'] = $choice_trans_id + 1;
						$choice_trans_id = $new_translation['translation_id'];
						$new_translation['is_default'] = $is_default;
						$new_translation['language_code'] = $lang;
						$new_translation['text'] = $choices[$i]['text'];
						$new_choice['translations'][] = $new_translation;
						
						$new_choice['results'] = array();
						$result_counter = 0;
						foreach ($results[$i] as $result) {
							
							$new_result = array();
							$new_result['result_id'] = $result_id + 1;
							$result_id = $new_result['result_id'];
							$new_result['ordinal'] = $result_counter+1;
							$new_result['weight'] = $result['weight'];
							$new_result['stat_changes'] = $result['stat_changes'];
							$new_result['next_criteria'] = $result['next_criteria'];
							$new_result['translations'] = array();
							
							$new_translation = array();
							$new_translation['card_id'] = $card_id;
							$new_translation['translation_id'] = $result_trans_id + 1;
							$result_trans_id = $new_translation['translation_id'];
							$new_translation['is_default'] = $is_default;
							$new_translation['language_code'] = $lang;
							$new_translation['text'] = $result['text'];
							$new_result['translations'][] = $new_translation;
						
							$result_counter++;
							$new_choice['results'][] = $new_result;
						}
						unset($result);
						$card['choices'][] = $new_choice;
					}
				}
				else {
					$limit = $new_choices_count;
				}
				// update old choices
				for ($i = 0; $i < $limit; $i++) {
				
					$card['choices'][$i]['ordinal'] = $i+1;
					// Update choice translation
					foreach ($card['choices'][$i]['translations'] as &$translation) {
						// If default language is same as translation language then mark all other translations to false
						if ($default_lang == $lang) {
							$translation['is_default'] = 0;
						}
						// Update the right translation
						if ($translation['language_code'] == $lang) {
							$translation['text'] = $choices[$i]['text'];
							$translation['is_default'] = $is_default;
						}
					}
					$result_limit = 0;
					$new_results_count = count($results[$i]);
					$old_results_count = count($card['choices'][$i]['results']);
					
					// Delete unused results
					if ($new_results_count < $old_results_count) {
						
						$result_limit = $new_results_count;
						
						for ($j = $new_results_count; $j < $old_results_count; $j++) {
							// Delete unneeded results
							array_splice($card['choices'][$i]['results'], $j, 1);
							$j--;
							$old_results_count--;
						}
					}
					// Add new results
					elseif ($new_results_count > $old_results_count) {
						
						$result_limit = $old_results_count;
						for ($j = $old_results_count; $j < $new_results_count; $j++) {
							
							$new_result = array();
							$new_result['result_id'] = $result_id + 1;
							$result_id = $new_result['result_id'];
							$new_result['ordinal'] = $j+1;
							$new_result['weight'] = $results[$i][$j]['weight'];
							$new_result['stat_changes'] = $results[$i][$j]['stat_changes'];
							$new_result['next_criteria'] = $results[$i][$j]['next_criteria'];
							$new_result['translations'] = array();
								
							$new_translation = array();
							$new_translation['card_id'] = $card_id;
							$new_translation['translation_id'] = $result_trans_id + 1;
							$result_trans_id = $new_translation['translation_id'];
							$new_translation['is_default'] = $is_default;
							$new_translation['language_code'] = $lang;
							$new_translation['text'] = $results[$i][$j]['text'];
							$new_result['translations'][] = $new_translation;
							$card['choices'][$i]['results'][] = $new_result;
						}
					}
					else {
						$result_limit = $new_results_count;
					}		
					// Update old results
					for ($j = 0; $j < $result_limit; $j++) {
						
						$card['choices'][$i]['results'][$j]['ordinal'] = $j+1;
						$card['choices'][$i]['results'][$j]['weight'] = $results[$i][$j]['weight'];
						$card['choices'][$i]['results'][$j]['stat_changes'] = $results[$i][$j]['stat_changes'];
						$card['choices'][$i]['results'][$j]['next_criteria'] = $results[$i][$j]['next_criteria'];
						
						// Update result translation
						foreach ($card['choices'][$i]['results'][$j]['translations'] as &$trans) {
							// If default language is same as translation language then mark all other translations to false
							if ($default_lang == $lang) {
								$trans['is_default'] = 0;
							}
							// Update the right translation
							if ($trans['language_code'] == $lang) {
								$trans['text'] = $results[$i][$j]['text'];
								$trans['is_default'] = $is_default;
							}
						}
					}
				}
				break;
			}
		}
		// Update xml file
		if ($this->writeXMLCards($this->xml_folder . 'games/' . $game_id . '/cards_' . $game_id . '.xml', $this->cards)) {
			$_SESSION['xmldb']['cards'] = $this->cards;
			$this->latest_card = array();
			$_SESSION['xmldb']['latest_card'] = array();
			$this->latest_ids['choice_id'] = $choice_id;
			$this->latest_ids['choice_translation_id'] = $choice_trans_id;
			$this->latest_ids['result_id'] = $result_id;
			$this->latest_ids['result_translation_id'] = $result_trans_id;
			$this->writeXMLLatestIds($this->xml_folder . 'latest_ids.xml', $this->latest_ids);
		}
		$this->updateCardRelatedTagsAndStats($card_id);
	}
	
	/**
	 * Adds new translation of a choice.
	 * @param double $choice_id ID of the choice in which to add new translation.
	 * @param string $choice_text Text of the choice in translation language.
	 * @param string $lang Language code of the new translation.
    * @param int $is_default Flag that determines whether or not new translation is in default language. 0: false, 1: true
    * @param double $card_id ID of the card which choice is related.
    * @return double ID of the new translation or the old one if it already existed
    * and -1 in case of error.
   */
	public function addChoiceTranslation($choice_id, $choice_text, $lang, $is_default, $card_id=0) {
		
		if ($this->debug) {
			$this->writeDebug('addChoiceTranslation', '');
		}
		
		if (empty($this->games) || empty($this->users)) {
			$this->initDB();
		}
		$game_id = $this->current_game;
		
		if (empty($this->cards)) {
			$game_id = 0;
			if ($this->current_game > 0) {
				$game_id = $this->current_game;
			}
			elseif (isset($_SESSION['admin']['game']['game_id'])) {
				$game_id = $_SESSION['admin']['game']['game_id'];
			}
			$this->getAllCards($game_id, $lang);
		}
		$cards = $this->cards;
		$card_index = 0;
		$trans_id = 0;
		$choice_index = 0;
		
		foreach ($cards as $card) {
			
			if ($card['card_id'] == $card_id) {
				$counter = 0;
				// First check if choice already has translation in current language
				foreach ($card['choices'] as $choice) {
					
					if ($choice['choice_id'] == $choice_id) {
						
						$choice_index = $counter;
						
						foreach ($choice['translations'] as $trans) {
				
							if ($trans['language_code'] == $lang) {
								$trans_id = $trans['translation_id'];
								break;
							}
						}
					}
					if ($trans_id > 0) {
						break;
					}
					$counter++;
				}
				unset($choice);
				break;
			}
			$card_index++;
		}
		// If translation is found return translation_id
		if ($trans_id > 0) {
			return $trans_id;
		}
		
		if ($card_index >= count($this->cards)) {
			return -1;
		}
		
		$new_translation = array();
		$new_translation['card_id'] = $card_id;
		$new_translation['translation_id'] = $this->latest_ids['choice_translation_id'] + 1;
		$new_translation['is_default'] = $is_default;
		$new_translation['language_code'] = $lang;
		$new_translation['text'] = $choice_text;
		$this->cards[$card_index]['choices'][$choice_index]['translations'][] = $new_translation;
		
		// Update xml file
		if ($this->writeXMLCards($this->xml_folder . 'games/' . $game_id . '/cards_' . $game_id . '.xml', $this->cards)) {
			$_SESSION['xmldb']['cards'] = $this->cards;
			$this->latest_ids['choice_translation_id'] = $new_translation['translation_id'];
			$this->writeXMLLatestIds($this->xml_folder . 'latest_ids.xml', $this->latest_ids);
			$trans_id = $new_translation['translation_id'];
		}
		
		return $trans_id;
	}
	
	/**
	 * Adds new translation of a result.
	 * @param double $result_id ID of the result in which to add new translation.
	 * @param string $result_text Text of the result in translation language.
	 * @param string $lang Language code of the new translation.
    * @param int $is_default Flag that determines whether or not new translation is in default language. 0: false, 1: true
    * @param double $card_id ID of the card which result is related.
    * @return double ID of the new translation or the old one if it already existed
    * and -1 in case of error.
   */
	public function addResultTranslation($result_id, $result_text, $lang, $is_default, $card_id=0) {
		
		if ($this->debug) {
			$this->writeDebug('addResultTranslation', '');
		}
		
		if (empty($this->games) || empty($this->users)) {
			$this->initDB();
		}
		$game_id = $this->current_game;
		
		if (empty($this->cards)) {
			$game_id = 0;
			if ($this->current_game > 0) {
				$game_id = $this->current_game;
			}
			elseif (isset($_SESSION['admin']['game']['game_id'])) {
				$game_id = $_SESSION['admin']['game']['game_id'];
			}
			$this->getAllCards($game_id, $lang);
		}
		$cards = $this->cards;
		$card_index = 0;
		$choice_index = 0;
		$result_index = 0;
		$trans_id = 0;
		
		foreach ($cards as $card) {
			
			if ($card['card_id'] == $card_id) {
				$choice_counter = 0;
				// First check if choice already has translation in current language
				foreach ($card['choices'] as $choice) {
					$result_counter = 0;
					foreach ($choice['results'] as $result) {
					
						if ($result['result_id'] == $result_id) {
							
							$choice_index = $choice_counter;
							$result_index = $result_counter;
							
							foreach ($result['translations'] as $trans) {
				
								if ($trans['language_code'] == $lang) {
									$trans_id = $trans['translation_id'];
									break;
								}
							}
						}
						if ($trans_id > 0) {
							break;
						}
						$result_counter++;
					}
					if ($trans_id > 0) {
						break;
					}
					$choice_counter++;
				}
				break;
			}
			$card_index++;
		}
		// If translation is found return translation_id
		if ($trans_id > 0) {
			return $trans_id;
		}
		
		if ($card_index >= count($this->cards)) {
			return -1;
		}
		
		$new_translation = array();
		$new_translation['card_id'] = $card_id;
		$new_translation['translation_id'] = $this->latest_ids['result_translation_id'] + 1;
		$new_translation['is_default'] = $is_default;
		$new_translation['language_code'] = $lang;
		$new_translation['text'] = $result_text;
		$this->cards[$card_index]['choices'][$choice_index]['results'][$result_index]['translations'][] = $new_translation;
		
		// Update xml file
		if ($this->writeXMLCards($this->xml_folder . 'games/' . $game_id . '/cards_' . $game_id . '.xml', $this->cards)) {
			$_SESSION['xmldb']['cards'] = $this->cards;
			$this->latest_ids['result_translation_id'] = $new_translation['translation_id'];
			$this->writeXMLLatestIds($this->xml_folder . 'latest_ids.xml', $this->latest_ids);
			$trans_id = $new_translation['translation_id'];
		}
		
		return $trans_id;
	}
	
	/**
	 * Enables or disables a card
	 * @param double $card_id ID of the card to update.
	 * @param int $enabled Flag to enable or disable the card. 0: disabled, 1: enabled.
    * @return void
   */
	public function enable_disable_card($card_id, $enabled) {
		
		if ($this->debug) {
			$this->writeDebug('enable_disable_card', '');
		}
		if (empty($this->games) || empty($this->users)) {
			$this->initDB();
		}
		$game_id = $this->current_game;
		
		if (empty($this->cards)) {
			$game_id = 0;
			if ($this->current_game > 0) {
				$game_id = $this->current_game;
			}
			elseif (isset($_SESSION['admin']['game']['game_id'])) {
				$game_id = $_SESSION['admin']['game']['game_id'];
			}
			$this->getAllCards($game_id);
		}
		
		foreach ($this->cards as &$card) {
			
			if ($card['card_id'] == $card_id) {
				
				$card['enabled'] = $enabled;
				$mysqldate = date( 'Y-m-d H:i:s' );
				$card['modified'] = $mysqldate;
				$card['editor_id'] = $editor_id;
				$card['editor_name'] = '';
				if ($card['editor_id'] > 0) {
					$user = $this->getUser($editor_id);
					$card['editor_name'] = $user['username'];
				}
				break;
			}
		}
		// Update xml file
		if ($this->writeXMLCards($this->xml_folder . 'games/' . $game_id . '/cards_' . $game_id . '.xml', $this->cards)) {
			$_SESSION['xmldb']['cards'] = $this->cards;
			// empty latest card
			$this->latest_card = array();
			$_SESSION['xmldb']['latest_card'] = array();
		}
	}
	
	/**
	 * Deletes a card and all related information.
    * @param double $card_id ID of the card to be deleted.
    * @return bool Was the operation successful
   */
	public function deleteCard($card_id) {
		
		if ($this->debug) {
			$this->writeDebug('deleteCard', '');
		}
		if ($card_id <= 0) {
			return false;
		}
		if (empty($this->games) || empty($this->users)) {
			$this->initDB();
		}
		$game_id = $this->current_game;
		if (empty($this->cards)) {
			$game_id = 0;
			if ($this->current_game > 0) {
				$game_id = $this->current_game;
			}
			elseif (isset($_SESSION['admin']['game']['game_id'])) {
				$game_id = $_SESSION['admin']['game']['game_id'];
			}
			$this->getAllCards($game_id, $lang);
		}
		
		$card_index = 0;
		$found = false;
		$deleted = false;
		
		foreach ($this->cards as $card) {
			
			if ($card['card_id'] == $card_id) {
				
				// if card is found stop the loop
				$found = true;
				break;
			}
			$card_index++;
		}
		if ($found) {
			// Remove card from cards		
			array_splice($this->cards, $card_index, 1);
		
			// Delete card from every profiles played_cards
			$this->getAllProfiles($game_id);
		
			foreach ($this->profiles as &$profile) {
			
				$played_cards_count = count($profile['played_cards']);
				for ($i = 0; $i < $played_cards_count; $i++) {
				
					if ($profile['played_cards'][$i]['card_id'] == $card_id) {
					
						array_splice($profile['played_cards'], $i, 1);
						$i--;
						$played_cards_count--;
					}
				}
				$this->writeXMLProfile($this->xml_folder . 'games/' . $game_id . '/player_profiles/profile_' . $profile['profile_id'] . '.xml', $profile);
			}
			// Update xml file
			if ($this->writeXMLCards($this->xml_folder . 'games/' . $game_id . '/cards_' . $game_id . '.xml', $this->cards)) {
				$_SESSION['xmldb']['cards'] = $this->cards;
				$_SESSION['xmldb']['profiles'] = $this->profiles;
				$deleted = true;
			}
		}
		return $deleted;
	}
	
	/**
	 * Imports card into game. If given external id exist replaces that card.
    * @param double $card_external_id ID of the card to be shown in editor (different than actual ID of the card).
    * @param int $repetitive Flag to tell whether or not card can be played multiple times.
    * @param string $tags Tags of the card.
    * @param string $stat_requirements Stat requirements of a card.
    * @param string $comment Comment creator has added to card.
    * @param string $attachment A name of the image linked to card.
    * @param string $card_name A name of the card in default language.
    * @param string $description A description of the card in default language.
    * @param string $expert_advice An expert advice of the card in default language.
    * @param array $choices Choices of the card in default language.
    * @param array $results Results of the card in default language.
    * @param double $game_id ID of the game in which to import card.
    * @param string $lang Language code of default language of the card.
    * @param double $creator_id ID of the user who created the card.
    * @return double ID of the created card.
   */
	public function importCard($card_external_id, $repetitive, $tags, $stat_requirements, $comment, $attachment, $layout='normal', $card_name, $description, $expert_advice, $choices, $results, $game_id, $lang, $creator_id) {
		
		if ($this->debug) {
			$this->writeDebug('importCard', '');
		}
		if (empty($this->games) || empty($this->users)) {
			$this->initDB();
		}
		
		if (empty($this->cards) || $this->current_game != $game_id) {
			$this->getAllCards($game_id, $lang);
		}
		
		$card_exist = false;
		$card_index = 0;
		$card_id = 0;
		$card_translations = array();
		// If card id is given then first check if card already exists
		if (doubleval($card_external_id) > 0) {
			
			foreach ($this->cards as $card) {
			
				if ($card['card_external_id'] == $card_external_id && $card['game_id'] == $game_id) {
					// If card is found mark that
					$card_exist = true;
					$card_id = $card['card_id'];
					$card_translations = $card['translations'];
					break;
				}
				$card_index++;
			}
		}
		// If card exist update that
		if ($card_exist) {
			
			$this->editCard(doubleval($card_id), doubleval($card_external_id), $repetitive, doubleval($editor_id), $tags, $stat_requirements, $comment, $attachment, $layout, $game_id);
			
			$translation_id = 0;
			foreach ($card['translations'] as $trans) {
				
				if ($trans['language_code'] == $lang) {
					$translation_id = $trans['translation_id'];
				}
				break;
			}
			// If translation exists update that
			if ($translation_id > 0) {
				$is_default = true;
				$card = $this->getCard($card_id);
				if ($card['default_lang'] != $lang) {
					$is_default = false;
				}
				$this->updateCardTranslation(doubleval($card_id), $card_name, $description, $expert_advice, $lang, $is_default, $card['default_lang']);
				$this->updateCardChoicesAndResults(doubleval($card_id), $choices, $results, $lang, $is_default, $card['default_lang']);
				//$this->updateCardTranslation(doubleval($card_id), $card_name, $description, $expert_advice, $lang, true, $lang);
				//$this->updateCardChoicesAndResults(doubleval($card_id), $choices, $results, $lang, true, $lang);
			}
			// Else create new translation
			else {
				$this->addCardTranslation(doubleval($card_id), $card_name, $description, $expert_advice, $lang, false);
				
				$card = $this->getCard($card_id);
				$choice_counter = 0;
				foreach ($choices as $choice) {
					if (isset($card['choices'][$choice_counter])) {
						$trans_id = $this->addChoiceTranslation($card['choices'][$choice_counter]['choice_id'], $choice['text'], $lang, 0, doubleval($card_id));
						$card_results = $this->getChoiceResults($card['choices'][$choice_counter]['choice_id']);
						$result_counter = 0;
						foreach ($results as $result) {
							if (isset($card_results[$result_counter])) {
								$trans_id = $this->addResultTranslation($card_results[$result_counter]['result_id'], $result[$choice_counter]['text'], $lang, 0, doubleval($card_id));
							}
							$result_counter++;
						}
					}
					$choice_counter++;
				}
				//$this->addCardChoicesAndResults(doubleval($card_id), $choices, $results, $lang, false);
			}
		}
		// else create new card
		else {
			
			$new_card_id = 0;
			$new_card_id = $this->createCard(doubleval($card_external_id), doubleval($game_id), $repetitive, doubleval($creator_id), $tags, $stat_requirements, $comment, $attachment, $layout);
			
			if ($new_card_id > 0) {
			
				$this->addCardTranslation($new_card_id, $card_name, $description, $expert_advice, $lang, true);
				$this->addCardChoicesAndResults($new_card_id, $choices, $results, $lang, true);
			}
		}
	}
	
	/**
	 * Creates or updates game texts translations.
    * @param double $game_id ID of the game.
    * @param array $texts An array containing texts of the game in given language.
    * @param string $lang Language code of these texts.
    * @param int $is_default Defines whether or not these are the default texts true=1/false=0.
    * @param string $default_lang Language code of default texts.
    * @return void
   */
	public function addOrUpdateGameTexts($game_id, $texts, $lang, $is_default, $default_lang) {
		
		if ($this->debug) {
			$this->writeDebug('addOrUpdateGameTexts', 'language: ' . $lang . ' is_default: ' . $is_default . ' default_lang: ' . $default_lang);
		}
		if (empty($this->texts)) {
			$this->initDB();
		}
		if (empty($this->texts)) {
			
			foreach($this->games as $game) {
			
				if (is_dir($this->xml_folder . 'games/' . $game['game_id'])) {
				
					if (is_file($this->xml_folder . 'games/' . $game['game_id'] . '/texts_' . $game['game_id'] . '.xml')) {
						$game_texts = $this->readXMLTexts($this->xml_folder . 'games/' . $game['game_id'] . '/texts_' . $game['game_id'] . '.xml');
						$this->texts = array_merge($this->texts, $game_texts);
					}
				}
			}
		}
		$found = false;		
		$translation_id = 0;
		$texts_to_write = array();
		
		// First check if translation exists and update it if it does
		foreach ($this->texts as &$text) {
			
			if ($text['game_id'] == $game_id) {
				// If default language is same as translation language then mark all other translations to false
				if ($default_lang == $lang) {
					$text['is_default'] = 0;
				}
				// Update the right translation
				if ($text['language_code'] == $lang) {
					$found = true;
					$translation_id = $text['translation_id'];
					$text['game_id'] = $game_id;
					$text['welcome'] = html_entity_decode($texts['welcome'], ENT_COMPAT,'UTF-8');
					$text['front_up'] = html_entity_decode($texts['front_up'], ENT_COMPAT,'UTF-8');
					$text['front_down'] = html_entity_decode($texts['front_down'], ENT_COMPAT,'UTF-8');
					$text['registration_up'] = html_entity_decode($texts['registration_up'], ENT_COMPAT,'UTF-8');
					$text['registration_down'] = html_entity_decode($texts['registration_down'], ENT_COMPAT,'UTF-8');
					$text['login_up'] = html_entity_decode($texts['login_up'], ENT_COMPAT,'UTF-8');
					$text['login_down'] = html_entity_decode($texts['login_down'], ENT_COMPAT,'UTF-8');
					$text['new_up'] = html_entity_decode($texts['new_up'], ENT_COMPAT,'UTF-8');
					$text['new_down'] = html_entity_decode($texts['new_down'], ENT_COMPAT,'UTF-8');
					$text['open_up'] = html_entity_decode($texts['open_up'], ENT_COMPAT,'UTF-8');
					$text['open_down'] = html_entity_decode($texts['open_down'], ENT_COMPAT,'UTF-8');
					$text['continue_up'] = html_entity_decode($texts['continue_up'], ENT_COMPAT,'UTF-8');
					$text['continue_down'] = html_entity_decode($texts['continue_down'], ENT_COMPAT,'UTF-8');
					$text['loading'] = html_entity_decode($texts['loading'], ENT_COMPAT,'UTF-8');
					$text['your_life'] = html_entity_decode($texts['your_life'], ENT_COMPAT,'UTF-8');
					$text['menu'] = html_entity_decode($texts['menu'], ENT_COMPAT,'UTF-8');
					$text['selection'] = html_entity_decode($texts['selection'], ENT_COMPAT,'UTF-8');
					$text['is_default'] = $is_default;
				}
				$texts_to_write[] = $text;
			}
		}
		
		// If it isn't found create a new one
		if (!$found) {
			
			$new_translation = array();
			$translation_id = $this->latest_ids['game_texts_translation_id'] + 1;
			$new_translation['translation_id'] = $translation_id;
			$new_translation['game_id'] = $game_id;
			$new_translation['welcome'] = html_entity_decode($texts['welcome'], ENT_COMPAT,'UTF-8');
			$new_translation['front_up'] = html_entity_decode($texts['front_up'], ENT_COMPAT,'UTF-8');
			$new_translation['front_down'] =  html_entity_decode($texts['front_down'], ENT_COMPAT,'UTF-8');
			$new_translation['registration_up'] = html_entity_decode($texts['registration_up'], ENT_COMPAT,'UTF-8');
			$new_translation['registration_down'] = html_entity_decode($texts['registration_down'], ENT_COMPAT,'UTF-8');
			$new_translation['login_up'] = html_entity_decode($texts['login_up'], ENT_COMPAT,'UTF-8');
			$new_translation['login_down'] = html_entity_decode($texts['login_down'], ENT_COMPAT,'UTF-8');
			$new_translation['new_up'] = html_entity_decode($texts['new_up'], ENT_COMPAT,'UTF-8');
			$new_translation['new_down'] = html_entity_decode($texts['new_down'], ENT_COMPAT,'UTF-8');
			$new_translation['open_up'] = html_entity_decode($texts['open_up'], ENT_COMPAT,'UTF-8');
			$new_translation['open_down'] = html_entity_decode($texts['open_down'], ENT_COMPAT,'UTF-8');
			$new_translation['continue_up'] = html_entity_decode($texts['continue_up'], ENT_COMPAT,'UTF-8');
			$new_translation['continue_down'] = html_entity_decode($texts['continue_down'], ENT_COMPAT,'UTF-8');
			$new_translation['loading'] = html_entity_decode($texts['loading'], ENT_COMPAT,'UTF-8');
			$new_translation['your_life'] = html_entity_decode($texts['your_life'], ENT_COMPAT,'UTF-8');
			$new_translation['menu'] = html_entity_decode($texts['menu'], ENT_COMPAT,'UTF-8');
			$new_translation['is_default'] = $is_default;
			$new_translation['language_code'] = $lang;
			$this->texts[] = $new_translation;
			$texts_to_write[] = $new_translation;
		}
		
		// Update xml file
		if ($this->writeXMLTexts($this->xml_folder . 'games/' . $game_id . '/texts_' . $game_id . '.xml', $texts_to_write, $game_id)) {
			$_SESSION['xmldb']['texts'] = $this->texts;
			if (!$found) {
				$this->latest_ids['game_texts_translation_id'] = $translation_id;
				$this->writeXMLLatestIds($this->xml_folder . 'latest_ids.xml', $this->latest_ids);
			}
		}
	}
	
	/**
	 * Retrieves texts to application pages.
    * @param double $game_id ID of the game.
    * @return array Containing application texts.
   */
	public function getGameTexts($game_id) {
		
		if ($this->debug) {
			$this->writeDebug('getGameTexts', '');
		}
		if (empty($this->texts)) {
			$this->initDB();
		}
		$texts = array();
		
		if (empty($this->texts)) {
			
			foreach($this->games as $game) {
			
				if (is_dir($this->xml_folder . 'games/' . $game['game_id'])) {
				
					if (is_file($this->xml_folder . 'games/' . $game['game_id'] . '/texts_' . $game['game_id'] . '.xml')) {
						$game_texts = $this->readXMLTexts($this->xml_folder . 'games/' . $game['game_id'] . '/texts_' . $game['game_id'] . '.xml');
						$this->texts = array_merge($this->texts, $game_texts);
						if ($game['game_id'] == $game_id) {
							$texts = $game_texts;
						}
					}
				}
			}
		}
		else {
		
			foreach ($this->texts as $txt) {
			
				if ($txt['game_id'] == $game_id) {
					$texts[] = $txt;
				}
			}
		}
		
		$_SESSION['xmldb']['texts'] = $this->texts;
		return $texts;
	}
	
	/**
	 * Retrieves corresponding game id based on given slug.
    * @param string $slug Slug of the game.
    * @return double ID of the game linked to given slug.
   */
	public function getGameIdBySlug($slug) {
		
		if ($this->debug) {
			$this->writeDebug('getGameIdBySlug', '');
		}
		if (empty($this->games)) {
			$this->initDB();
		}
		$game_id = 0;
		
		foreach ($this->games as $game) {
			
			if ($game['slug'] == $slug) {
				$game_id = $game['game_id'];
				break;
			}
		}
    	return $game_id;
	}
	
	/**
	 * Prints the debug text entry to a log file.
    * @param string $functionname Name of the function which wants to write debug entry.
    * @param string $message Debug entry text.
    * @return void
   */
	private function writeDebug($functionname, $message) {
		
		$file = $this->xml_folder . 'log.txt';
		$logfile = '';
		// Open the file to get existing content
		if (is_file($file)) {
			$logfile = file_get_contents($file);
		}
		
		$date = date( 'Y-m-d H:i:s' );
		$entry = $date . ' ';
		$entry .= $functionname . ': ';
		$entry .= $message . PHP_EOL;
		// Append a new entry to file
		$logfile .= $entry;
		// Write the contents back to the file
		file_put_contents($file, $logfile);
	}
	
	/**
	 * Initializes the XML database.
    * @return void
   */
	private function initDB() {
		
		if ($this->debug) {
			$this->writeDebug('initDB', '');
		}
		
		if (!is_file($this->xml_folder . 'games.xml')) {
			echo("ERROR: games.xml is missing!" . PHP_EOL);
		}
		if (!is_file($this->xml_folder . 'users.xml')) {
			echo("ERROR: users.xml is missing!" . PHP_EOL);
		}
		if (!isset($_SESSION['xmldb']['users']) || empty($_SESSION['xmldb']['users'])) {
			// Parse users to array
			$this->users = $this->readXMLUsers($this->xml_folder . 'users.xml');
			$_SESSION['xmldb']['users'] = $this->users;
		}
		else {
			$this->users = $_SESSION['xmldb']['users'];
		}
		if (!isset($_SESSION['xmldb']['games']) || empty($_SESSION['xmldb']['games'])) {
			// Parse games to array
			$this->games = $this->readXMLGames($this->xml_folder . 'games.xml');
			$_SESSION['xmldb']['games'] = $this->games;
		}
		else {
			$this->games = $_SESSION['xmldb']['games'];
		}
		$this->cards = array();
		if (isset($_SESSION['xmldb']['cards']) && !empty($_SESSION['xmldb']['cards'])) {
			$this->cards = $_SESSION['xmldb']['cards'];
		}
		$this->profiles = array();
		if (isset($_SESSION['xmldb']['profiles']) && !empty($_SESSION['xmldb']['profiles'])) {
			$this->profiles = $_SESSION['xmldb']['profiles'];
		}
		if (isset($_SESSION['xmldb']['current_profile']) && !empty($_SESSION['xmldb']['current_profile'])) {
			$this->current_profile = $_SESSION['xmldb']['current_profile'];
		}
		$this->current_game = 0;
		if (isset($_SESSION['xmldb']['current_game']) && $_SESSION['xmldb']['current_game'] > 0) {
			$this->current_game = $_SESSION['xmldb']['current_game'];
		}
		$this->latest_card = array();
		if (isset($_SESSION['xmldb']['latest_card']) && !empty($_SESSION['xmldb']['latest_card'])) {
			$this->latest_card = $_SESSION['xmldb']['latest_card'];
		}
		$this->latest_ids = $this->readXMLLatestIds($this->xml_folder . 'latest_ids.xml');
		$this->texts = array();
		if (isset($_SESSION['xmldb']['texts']) && !empty($_SESSION['xmldb']['texts'])) {
			$this->texts = $_SESSION['xmldb']['texts'];
		}
	}
	
	/**
	 * Parses the XML file containing information about users of GAL2.
    * @param string $xml_file File path of the user XML file.
    * @return array Contains users' ids, usernames and passwords.
   */
	private function readXMLUsers($xml_file) {
		
		if ($this->debug) {
			$this->writeDebug('readXMLUsers', '');
		}
		
		$users = array();
		
		$xml_users = simplexml_load_file($xml_file);

		if ($xml_users) {
			// Get number of users
			if (count($xml_users->user) >= 1) {
				
				foreach ($xml_users->user as $user) {

					if ((string)$user['id']) {
						$new_user = array();
						$new_user['user_id'] = doubleval(strval($user['id']));
						$new_user['username'] = strval($user->username);
						$new_user['password'] = strval($user->password);
						$users[] = $new_user;
					}
				}
			}
		}
		
		return $users;
	}
	
	/**
	 * Writes the XML file containing information about users of GAL2.
    * @param string $xml_file File path of the user XML file.
    * @param array $users An array containing user data.
    * @return bool Was the operation successful.
   */
	private function writeXMLUsers($xml_file, $users) {
		
		if ($this->debug) {
			$this->writeDebug('writeXMLUsers', '');
		}
		
		$xml_base = <<<XML
<?xml version="1.0" encoding="UTF-8" ?>
<users>
</users>
XML;
		
		$xml_users = simplexml_load_string($xml_base);
		
		foreach ($users as $user) {
			$xml_user = $xml_users->addChild('user');
			$xml_user->addAttribute('id', $user['user_id']);
			$xml_user->addChild('username', $user['username']);
			$xml_user->addChild('password', $user['password']);
		}
		
		if ($xml_users->saveXML($xml_file)) {
			return true;
		}
		return false;
	}
	
	/**
	 * Parses the XML file containing information about games of GAL2.
    * @param string $xml_file File path of the games XML file.
    * @return array Contains settings of every game.
   */
	private function readXMLGames($xml_file) {
		
		if ($this->debug) {
			$this->writeDebug('readXMLGames', '');
		}
		
		$xml_games = simplexml_load_file($xml_file);
		
		$games = array();
		
		if ($xml_games) {
			
			// Get number of games
			if (count($xml_games->game) >= 1) {
				
				foreach ($xml_games->game as $game) {

					if ((string)$game['id']) {
						$new_game = array();
						$new_game['game_id'] = doubleval(strval($game['id']));
						$new_game['name'] = htmlspecialchars_decode(strval($game->name));
						$new_game['created'] = (string)$game->created;
						$new_game['main_admin_id'] = doubleval(strval($game->main_admin_id));
						$new_game['starting_card_id'] = 0;
						if (doubleval(strval($game->starting_card_id)) > 0) {
							$new_game['starting_card_id'] = doubleval(strval($game->starting_card_id));
						}
						$new_game['panic_card_id'] = 0;
						if (doubleval(strval($game->panic_card_id)) > 0) {
							$new_game['panic_card_id'] = doubleval(strval($game->panic_card_id));
						}
						$new_game['image_folder'] = strval($game->image_folder);
						$new_game['slug'] = '';
						if (isset($game->slug)) {
							$new_game['slug'] = strval($game->slug);
						}
						$new_game['latest_card_id'] = 0;
						if (isset($game->latest_card_id) && doubleval(strval($game->latest_card_id)) > 0) {
							$new_game['latest_card_id'] = doubleval($game->latest_card_id);
						}
						$new_game['use_tags'] = 1;
						if (intval(strval($game->use_tags)) < 1) {
							$new_game['use_tags'] = 0;
						}
						$new_game['use_stats'] = 1;
						if (intval(strval($game->use_stats)) < 1) {
							$new_game['use_stats'] = 0;
						}
						$new_game['use_expert_advice'] = 1;
						if (intval(strval($game->use_expert_advice)) < 1) {
							$new_game['use_expert_advice'] = 0;
						}
						$new_game['use_sudden_scenarios'] = 1;
						if (intval(strval($game->use_sudden_scenarios)) < 1) {
							$new_game['use_sudden_scenarios'] = 0;
						}
						$new_game['use_ending_criteria'] = 1;
						if (intval(strval($game->use_ending_criteria)) < 1) {
							$new_game['use_ending_criteria'] = 0;
						}
						$new_game['use_login'] = 1;
						if (intval(strval($game->use_login)) < 1) {
							$new_game['use_login'] = 0;
						}
						$new_game['use_name'] = 1;
						if (intval(strval($game->use_name)) < 1) {
							$new_game['use_name'] = 0;
						}
						$new_game['multiple_languages'] = 1;
						if (intval(strval($game->multiple_languages)) < 1) {
							$new_game['multiple_languages'] = 0;
						}
						$new_game['show_cards_ending'] = 1;
						if (intval(strval($game->show_cards_ending)) < 1) {
							$new_game['show_cards_ending'] = 0;
						}					
						$new_game['max_cancellations'] = -1;
						if (intval(strval($game->max_cancellations)) >= 0) {
							$new_game['max_cancellations'] = intval(strval($game->max_cancellations));
						}
						$new_game['background_color'] = '';
						if (isset($game->background_color)) {
							$new_game['background_color'] = strval($game->background_color);
						}
						$new_game['text_color'] = '';
						if (isset($game->text_color)) {
							$new_game['text_color'] = strval($game->text_color);
						}
						$new_game['link_color'] = '';
						if (isset($game->link_color)) {
							$new_game['link_color'] = strval($game->link_color);
						}
						$new_game['link_hover_color'] = '';
						if (isset($game->link_hover_color)) {
							$new_game['link_hover_color'] = strval($game->link_hover_color);
						}
						$new_game['button_color'] = '';
						if (isset($game->button_color)) {
							$new_game['button_color'] = strval($game->button_color);
						}
						$new_game['button_hover_color'] = '';
						if (isset($game->button_hover_color)) {
							$new_game['button_hover_color'] = strval($game->button_hover_color);
						}
						$new_game['background_tile'] = '';
						if (isset($game->background_tile)) {
							$new_game['background_tile'] = strval($game->background_tile);
						}
						$new_game['logo'] = '';
						if (isset($game->logo)) {
							$new_game['logo'] = strval($game->logo);
						}
						$new_game['visible_stats'] = array();
						if (isset($game->visible_stats)) {
							$stat_string = htmlspecialchars_decode(strval($game->visible_stats));
							$visible_stats = explode('@', $stat_string);
							foreach ($visible_stats as $stat) {
								$new_stat = trim($stat);
								if (!empty($new_stat)) {
									$new_game['visible_stats'][] = $new_stat;
								}
							}
							unset($stat);
						}
						
						$new_game['admins'] = array();
						foreach ($game->admins->user_id as $admin) {
							$new_admin = array();
							$new_admin['user_id'] = 0;
							$new_admin['relationship_id'] = 0;
							if (isset($admin['relationship_id'])) {
								$new_admin['relationship_id'] = doubleval(strval($admin['relationship_id']));
							}
							if (doubleval(strval($admin)) > 0) {
								$new_admin['user_id'] = doubleval(strval($admin));
								$new_game['admins'][] = $new_admin;
							}
						}
						
						$new_game['sudden_scenarios'] = array();
						foreach ($game->sudden_scenarios->scenario as $scenario) {
							$new_scenario = array();
							$new_scenario['scenario_id'] = doubleval(strval($scenario['id']));
							$new_scenario['odds'] = intval(strval($scenario->odds));
							$new_scenario['result'] = strval($scenario->result);
							$new_scenario['ordinal'] = intval(strval($scenario->ordinal));
							$new_scenario['conditions'] = array();
							foreach ($scenario->conditions->condition as $condition) {
								$new_condition = array();
								$new_condition['condition_id'] = doubleval(strval($condition['id']));
								$new_condition['condition_text'] = htmlspecialchars_decode(strval($condition->text));
								$new_condition['condition_logic'] = strval($condition->logic);
								$new_condition['ordinal'] = intval(strval($condition->ordinal));
								$new_scenario['conditions'][] = $new_condition;
							}
							$new_game['sudden_scenarios'][] = $new_scenario;
						}
						
						$new_game['ending_criteria'] = array();
						foreach ($game->ending_criteria->criteria as $criteria) {
							$new_ending = array();
							$new_ending['criteria_id'] = doubleval(strval($criteria['id']));
							$new_ending['result'] = strval($criteria->result);
							$new_ending['ordinal'] = intval(strval($criteria->ordinal));
							$new_ending['conditions'] = array();
							foreach ($criteria->conditions->condition as $condition) {
								$new_condition = array();
								$new_condition['condition_id'] = doubleval(strval($condition['id']));
								$new_condition['condition_text'] = htmlspecialchars_decode(strval($condition->text));
								$new_condition['condition_logic'] = strval($condition->logic);
								$new_condition['ordinal'] = intval(strval($condition->ordinal));
								$new_ending['conditions'][] = $new_condition;
							}
							$new_game['ending_criteria'][] = $new_ending;
						}
						
						$games[] = $new_game;
					}
				}
			}
		}
		
		return $games;
	}
	
	/**
	 * Writes the XML file containing information about games of GAL2.
    * @param string $xml_file File path of the games XML file.
    * @param array $games An array containing settings of every game.
    * @return bool Was the operation successful.
   */
	private function writeXMLGames($xml_file, $games) {
		
		if ($this->debug) {
			$this->writeDebug('writeXMLGames', '');
		}
		
		$xml_base = <<<XML
<?xml version="1.0" encoding="UTF-8" ?>
<games>
</games>
XML;
		$xml_games = simplexml_load_string($xml_base);
		
		foreach ($games as $game) {
			$xml_game = $xml_games->addChild('game');
			$xml_game->addAttribute('id', $game['game_id']);
			$xml_game->addChild('name', htmlspecialchars($game['name'], ENT_COMPAT, "UTF-8"));
			$xml_game->addChild('created', $game['created']);
			$xml_game->addChild('main_admin_id', $game['main_admin_id']);
			$xml_game->addChild('starting_card_id', $game['starting_card_id']);
			$xml_game->addChild('panic_card_id', $game['panic_card_id']);
			$xml_game->addChild('image_folder', $game['image_folder']);
			$xml_game->addChild('slug', $game['slug']);
			$xml_game->addChild('latest_card_id', $game['latest_card_id']);
			$xml_game->addChild('use_tags', $game['use_tags']);
			$xml_game->addChild('use_stats', $game['use_stats']);
			$xml_game->addChild('use_expert_advice', $game['use_expert_advice']);
			$xml_game->addChild('use_sudden_scenarios', $game['use_sudden_scenarios']);
			$xml_game->addChild('use_ending_criteria', $game['use_ending_criteria']);
			$xml_game->addChild('use_login', $game['use_login']);
			$xml_game->addChild('use_name', $game['use_name']);
			$xml_game->addChild('multiple_languages', $game['multiple_languages']);
			$xml_game->addChild('show_cards_ending', $game['show_cards_ending']);
			$xml_game->addChild('max_cancellations', $game['max_cancellations']);
			$xml_game->addChild('background_color', $game['background_color']);
			$xml_game->addChild('text_color', $game['text_color']);
			$xml_game->addChild('link_color', $game['link_color']);
			$xml_game->addChild('link_hover_color', $game['link_hover_color']);
			$xml_game->addChild('button_color', $game['button_color']);
			$xml_game->addChild('button_hover_color', $game['button_hover_color']);
			$xml_game->addChild('background_tile', $game['background_tile']);
			$xml_game->addChild('logo', $game['logo']);
			
			$visible_stats = '';
			$stat_string = implode(' @', $game['visible_stats']);
			$stat_string = trim($stat_string);
			if (!empty($stat_string)) {
				$visible_stats = htmlspecialchars($stat_string, ENT_COMPAT, "UTF-8");
			}
			$xml_game->addChild('visible_stats', $visible_stats);
			
			$admins = $xml_game->addChild('admins');
			foreach ($game['admins'] as $admin) {
				$user = $admins->addChild('user_id', $admin['user_id']);
				$user->addAttribute('relationship_id', $admin['relationship_id']);
			}
			
			$sudden_scenarios = $xml_game->addChild('sudden_scenarios');
			foreach ($game['sudden_scenarios'] as $sudden) {
				
				$scenario = $sudden_scenarios->addChild('scenario');
				$scenario->addAttribute('id', $sudden['scenario_id']);
				$scenario->addChild('odds', $sudden['odds']);
				$scenario->addChild('result', $sudden['result']);
				$scenario->addChild('ordinal', $sudden['ordinal']);
				
				$conditions = $scenario->addChild('conditions');
				foreach ($sudden['conditions'] as $cond) {
					$condition = $conditions->addChild('condition');
					$condition->addAttribute('id', $cond['condition_id']);
					$condition->addChild('text', $cond['condition_text']);
					$condition->addChild('logic', $cond['condition_logic']);
					$condition->addChild('ordinal', $cond['ordinal']);
				}
				unset($cond);
			}
			$ending_criteria = $xml_game->addChild('ending_criteria');
			foreach ($game['ending_criteria'] as $ending) {
				
				$criteria = $ending_criteria->addChild('criteria');
				$criteria->addAttribute('id', $ending['criteria_id']);
				$criteria->addChild('result', $ending['result']);
				$criteria->addChild('ordinal', $ending['ordinal']);
				
				$conditions = $criteria->addChild('conditions');
				foreach ($ending['conditions'] as $cond) {
					$condition = $conditions->addChild('condition');
					$condition->addAttribute('id', $cond['condition_id']);
					$condition->addChild('text', $cond['condition_text']);
					$condition->addChild('logic', $cond['condition_logic']);
					$condition->addChild('ordinal', $cond['ordinal']);
				}
				unset($cond);
			}
		}
		
		if ($xml_games->saveXML($xml_file)) {
			return true;
		}
		return false;
	}
	
	/**
	 * Parses the XML file containing information about single player profile.
    * @param string $xml_file File path of the profile XML file.
    * @return array Contains information about profile.
   */
	private function readXMLProfile($xml_file) {
		
		if ($this->debug) {
			$this->writeDebug('readXMLProfile', '');
		}
		$profile = array();
		
		$xml_profile = simplexml_load_file($xml_file);
		
		if ($xml_profile) {
			
			$profile_id = 0;
			
			if (strval($xml_profile['id'])) {
						
				$profile['game_id'] = doubleval(strval($xml_profile->game_id));
				$profile['profile_id'] = doubleval(strval($xml_profile['id']));
				$profile['user_id'] = doubleval(strval($xml_profile->user_id));
				$profile['player_name'] = htmlspecialchars_decode(strval($xml_profile->player_name));
				$profile['created'] = strval($xml_profile->created);
				$profile['cancellations'] = intval(strval($xml_profile->cancellations));
				$profile['is_panic'] = intval(strval($xml_profile->is_panic));
				$profile['game_ended'] = intval(strval($xml_profile->game_ended));
				
				$scenario = 0;
				if (isset($xml_profile->scenario)) {
					$scenario = intval(strval($xml_profile->scenario));
				}
				$profile['scenario'] = $scenario;
				// Set scenario if not set
				if ($scenario < 1) {
					// Draw scenario (1-5)
					$profile['scenario'] = mt_rand(1, 5);
					$this->updatePlayerProfileScenario($id, $profile['scenario']);
				}
			
				$profile['played_cards'] = array();
				
				foreach ($xml_profile->played_cards->played_card as $played_card) {
					$card = array();
					$card['relationship_id'] = doubleval(strval($played_card['id']));
					$card['card_id'] = doubleval(strval($played_card->card_id));
					$card['choice_id'] = doubleval(strval($played_card->choice_id));
					$card['result_id'] = doubleval(strval($played_card->result_id));
					$card['ordinal'] = intval(strval($played_card->ordinal));
					$profile['played_cards'][] = $card;
				}
			}
		}
		return $profile;
	}
	
	/**
	 * Writes the XML file containing information about a single player profile.
    * @param string $xml_file File path of the profile XML file.
    * @param array $player_profile An array containing information about player profile.
    * @return bool Was the operation successful.
   */
	private function writeXMLProfile($xml_file, $player_profile) {
		
		if ($this->debug) {
			$this->writeDebug('writeXMLProfile', '');
		}
		
		$xml_base = <<<XML
<?xml version="1.0" encoding="UTF-8" ?>
<player_profile>
</player_profile>
XML;
		$xml_profile = simplexml_load_string($xml_base);
		$xml_profile->addAttribute('id', $player_profile['profile_id']);
		$xml_profile->addChild('game_id', $player_profile['game_id']);
		$xml_profile->addChild('user_id', $player_profile['user_id']);
		$xml_profile->addChild('player_name', htmlspecialchars($player_profile['player_name'], ENT_COMPAT, "UTF-8"));
		$xml_profile->addChild('created', $player_profile['created']);
		$xml_profile->addChild('cancellations', $player_profile['cancellations']);
		$xml_profile->addChild('is_panic', $player_profile['is_panic']);
		$xml_profile->addChild('game_ended', $player_profile['game_ended']);
		$xml_profile->addChild('scenario', $player_profile['scenario']);
		$cards = $xml_profile->addChild('played_cards');
		
		foreach ($player_profile['played_cards'] as $played_card) {
			
			$card = $cards->addChild('played_card');
			if (isset($played_card['relationship_id'])) {
				$card->addAttribute('id', $played_card['relationship_id']);
			}
			$card->addChild('card_id', $played_card['card_id']);
			$card->addChild('choice_id', $played_card['choice_id']);
			$card->addChild('result_id', $played_card['result_id']);
			$card->addChild('ordinal', $played_card['ordinal']);
		}
		
		if ($xml_profile->saveXML($xml_file)) {
			return true;
		}
		return false;
	}
	
	/**
	 * Parses the XML file containing information about cards of a single game.
    * @param string $xml_file File path of the cards XML file.
    * @return array Contains every card of the game.
   */
	private function readXMLCards($xml_file) {
		
		if ($this->debug) {
			$this->writeDebug('readXMLCards', '');
		}
		
		$cards = array();
		
		$xml_cards = simplexml_load_file($xml_file);
		
		if ($xml_cards) {
			$game_id = 0;
			if ($xml_cards['game_id']) {
				$game_id = doubleval(strval($xml_cards['game_id']));
			}
			// Get number of cards
			if (count($xml_cards->card) >= 1) {
				
				foreach ($xml_cards->card as $card) {
					
					$card_to_add = array();
					$card_to_add['card_id'] = doubleval(strval($card['id']));
					$card_to_add['card_external_id'] = $card_to_add['card_id'];
					if (isset($card->card_external_id)) {
						$card_to_add['card_external_id'] = doubleval(strval($card->card_external_id));
					}
					$card_to_add['game_id'] = $game_id;
					$card_to_add['enabled'] = intval(strval($card->enabled));
					$card_to_add['repetitive'] = intval(strval($card->repetitive));
					$card_to_add['created'] = strval($card->created);
					$card_to_add['modified'] = strval($card->modified);
					$card_to_add['creator_id'] = doubleval(strval($card->creator_id));
					$card_to_add['editor_id'] = doubleval(strval($card->editor_id));
					
					$card_to_add['tags'] = array();
					/*foreach ($card->tags->tag as $tag) {
						
						$new_tag = array();
						//$new_tag['tag_id'] = doubleval(strval($tag['id']));
						//$new_tag['tag_name'] = htmlspecialchars_decode(strval($tag));
						$new_tag = htmlspecialchars_decode(strval($tag));
						$card_to_add['tags'][] = $new_tag;
					}*/
					$tag_string = htmlspecialchars_decode(strval($card->tags));
					$tags = explode('#', $tag_string);
					$card_to_add['tags'][] = $new_tag;
					
					foreach ($tags as $tag) {
						$trimmed = trim($tag);
						if (!empty($trimmed)) {
							// find if theres space
							$pos = strrpos($trimmed, " ");
							if ($pos !== false) {
								$trimmed = substr($trimmed, 0, $pos);
							}
							$card_to_add['tags'][] = $trimmed;
						}
					}
					$card_to_add['stat_requirements'] = htmlspecialchars_decode(strval($card->stat_requirements));
					
					$card_to_add['comment'] = htmlspecialchars_decode(strval($card->comment));
					$card_to_add['attachment'] = strval($card->attachment);
					
					$card_to_add['layout'] = 'normal';
					if (isset($card->layout)) {
						$card_to_add['layout'] = strval($card->layout);
					}
					$card_to_add['hidden'] = 0;
					if (isset($card->hidden)) {
						$card_to_add['hidden'] = intval(strval($card->hidden));
					}
					
					$card_to_add['related_stats'] = htmlspecialchars_decode(strval($card->related_stats));
					$card_to_add['related_tags'] = htmlspecialchars_decode(strval($card->related_tags));
					
					$card_to_add['translations'] = array();
					
					foreach ($card->translations->translation as $trans) {
						
						$translation = array();
						$translation['translation_id'] = doubleval(strval($trans['id']));
						$translation['is_default'] = intval(strval($trans['is_default']));
						$translation['language_code'] = strval($trans['language_code']);
						$translation['card_name'] = htmlspecialchars_decode(strval($trans->card_name));
						$translation['description'] = htmlspecialchars_decode(strval($trans->description));
						$translation['expert_advice'] = htmlspecialchars_decode(strval($trans->expert_advice));
						$card_to_add['translations'][] = $translation;
					}
					
					$card_to_add['choices'] = array();
					
					foreach ($card->choices->choice as $choi) {
						
						$choice = array();
						$choice['choice_id'] = doubleval(strval($choi['id']));
						$choice['ordinal'] = intval(strval($choi['ordinal']));
						$choice['translations'] = array();
						
						foreach ($choi->choice_text as $trans) {
							
							$translation = array();
							$translation['card_id'] = doubleval(strval($card['id']));
							$translation['translation_id'] = doubleval(strval($trans['id']));
							$translation['is_default'] = intval(strval($trans['is_default']));
							$translation['language_code'] = strval($trans['language_code']);
							$translation['text'] = htmlspecialchars_decode(strval($trans));
							$choice['translations'][] = $translation;
						}
						unset($trans);
						$choice['results'] = array();
						
						foreach ($choi->results->result as $res) {
							
							$result = array();
							$result['result_id'] = doubleval(strval($res['id']));
							$result['ordinal'] = intval(strval($res['ordinal']));
							$result['weight'] = intval(strval($res->weight));
							$result['stat_changes'] = htmlspecialchars_decode(strval($res->stat_changes));
							$result['next_criteria'] = htmlspecialchars_decode(strval($res->next_card_criteria));
							$result['translations'] = array();
							
							foreach ($res->result_text as $trans) {
								
								$translation = array();
								$translation['card_id'] = doubleval(strval($card['id']));
								$translation['translation_id'] = doubleval(strval($trans['id']));
								$translation['is_default'] = intval(strval($trans['is_default']));
								$translation['language_code'] = strval($trans['language_code']);
								$translation['text'] = htmlspecialchars_decode(strval($trans));
								$result['translations'][] = $translation;
							}
							unset($trans);
							$choice['results'][] = $result;
						}
						$card_to_add['choices'][] = $choice;
					}
					$cards[] = $card_to_add;
				}
			}
		}
		
		return $cards;
	}
	
	/**
	 * Writes the XML file containing information about cards of a single game.
    * @param string $xml_file File path of the cards XML file.
    * @param array $cards An array containing information about cards.
    * @return bool Was the operation successful.
   */
	private function writeXMLCards($xml_file, $cards) {
		
		if ($this->debug) {
			$this->writeDebug('writeXMLCards', '');
		}
		
		$xml_base = <<<XML
<?xml version="1.0" encoding="UTF-8" ?>
<cards>
</cards>
XML;
		$xml_cards = simplexml_load_string($xml_base);
		$game_id = 0;
		if (isset($this->current_game)) {
			$game_id = $this->current_game;
		}
		$xml_cards->addAttribute('game_id', $game_id);
		
		foreach ($cards as $card) {
			
			$xml_card = $xml_cards->addChild('card');
			$xml_card->addAttribute('id', $card['card_id']);
			$xml_card->addChild('card_external_id', $card['card_external_id']);
			$xml_card->addChild('enabled', $card['enabled']);
			$xml_card->addChild('repetitive', $card['repetitive']);
			$xml_card->addChild('created', $card['created']);
			$xml_card->addChild('modified', $card['modified']);
			$xml_card->addChild('creator_id', $card['creator_id']);
			$xml_card->addChild('editor_id', $card['editor_id']);
			
			$card_tags = implode(' #', $card['tags']);
			if (!empty($card_tags)) {
				$card_tags = '#' . $card_tags;
			}
			$xml_card->addChild('tags', htmlspecialchars($card_tags, ENT_COMPAT, "UTF-8"));
			$xml_card->addChild('stat_requirements', htmlspecialchars($card['stat_requirements'], ENT_COMPAT, "UTF-8"));
			$xml_card->addChild('comment', htmlspecialchars($card['comment'], ENT_COMPAT, "UTF-8"));
			$xml_card->addChild('attachment', $card['attachment']);
			$xml_card->addChild('layout', $card['layout']);
			$xml_card->addChild('hidden', $card['hidden']);
			
			$xml_card->addChild('related_tags', htmlspecialchars($card['related_tags'], ENT_COMPAT, "UTF-8"));
			$xml_card->addChild('related_stats', htmlspecialchars($card['related_stats'], ENT_COMPAT, "UTF-8"));
			
			$card_translations = $xml_card->addChild('translations');
			
			foreach ($card['translations'] as $translation) {
				$trans = $card_translations->addChild('translation');
				$trans->addAttribute('id', $translation['translation_id']);
				$trans->addAttribute('is_default', $translation['is_default']);
				$trans->addAttribute('language_code', $translation['language_code']);
				$trans->addChild('card_name', htmlspecialchars($translation['card_name'], ENT_COMPAT, "UTF-8"));
				$trans->addChild('description', htmlspecialchars($translation['description'], ENT_COMPAT, "UTF-8"));
				$trans->addChild('expert_advice', htmlspecialchars($translation['expert_advice'], ENT_COMPAT, "UTF-8"));
			}
			unset($translation);
			/*$card_tags = $xml_card->addChild('tags');
			
			foreach ($card['tags'] as $tag) {	
				$xml_tag = $card_tags->addChild('tag', $tag['tag_name']);
				if (isset($tag['tag_id']) && !empty($tag['tag_id'])) {
					$xml_tag->addAttribute('id', $tag['tag_id']);
				}
			}*/
			$card_choices = $xml_card->addChild('choices');
			
			foreach ($card['choices'] as $choice) {
				$xml_choice = $card_choices->addChild('choice');
				$xml_choice->addAttribute('id', $choice['choice_id']);
				$xml_choice->addAttribute('ordinal', $choice['ordinal']);
				
				foreach ($choice['translations'] as $translation) {
					$choice_translation = $xml_choice->addChild('choice_text', htmlspecialchars($translation['text'], ENT_COMPAT, "UTF-8"));
					$choice_translation->addAttribute('id', $translation['translation_id']);
					$choice_translation->addAttribute('is_default', $translation['is_default']);
					$choice_translation->addAttribute('language_code', $translation['language_code']);
				}
				unset($translation);
				$choice_results = $xml_choice->addChild('results');
				
				foreach ($choice['results'] as $result) {
					$xml_result = $choice_results->addChild('result');
					$xml_result->addAttribute('id', $result['result_id']);
					$xml_result->addAttribute('ordinal', $result['ordinal']);
					$xml_result->addChild('weight', $result['weight']);
					$xml_result->addChild('stat_changes', htmlspecialchars($result['stat_changes'], ENT_COMPAT, "UTF-8"));
					$xml_result->addChild('next_card_criteria', htmlspecialchars($result['next_criteria'], ENT_COMPAT, "UTF-8"));
							
					foreach ($result['translations'] as $translation) {
						$result_translation = $xml_result->addChild('result_text', htmlspecialchars($translation['text'], ENT_COMPAT, "UTF-8"));
						$result_translation->addAttribute('id', $translation['translation_id']);
						$result_translation->addAttribute('is_default', $translation['is_default']);
						$result_translation->addAttribute('language_code', $translation['language_code']);
					}
				}
			}
		}
		
		if ($xml_cards->saveXML($xml_file)) {
			return true;
		}
		return false;
	}
	
	/**
	 * Parses the XML file containing latest ids used of GAL2.
    * @param string $xml_file File path of the latest ids XML file.
    * @return array Contains lastest ids used.
   */
	private function readXMLLatestIds($xml_file) {
		
		if ($this->debug) {
			$this->writeDebug('readXMLLatestIds', '');
		}
		
		$ids = array();
		
		$ids['user_id'] = 0;
		$ids['game_id'] = 0;
		$ids['card_id'] = 0;
		$ids['profile_id'] = 0;
		$ids['admin_relationship_id'] = 0;
		$ids['played_card_id'] = 0;
		$ids['played_card_id'] = 0;
		$ids['scenario_id'] = 0;
    	$ids['scenario_condition_id'] = 0;
   	$ids['criteria_id'] = 0;
    	$ids['criteria_condition_id'] = 0;
    	$ids['tag_id'] = 0; // TODO: might not needed
    	$ids['card_translation_id'] = 0;
    	$ids['choice_id'] = 0;
    	$ids['choice_translation_id'] = 0;
    	$ids['result_id'] = 0;
		$ids['result_translation_id'] = 0;
		$ids['game_texts_translation_id'] = 0;
		$xml_ids = simplexml_load_file($xml_file);

		if ($xml_ids) {
			
			if (doubleval(strval($xml_ids->user_id))) {
				$ids['user_id'] = doubleval(strval($xml_ids->user_id));
			}
			if (doubleval(strval($xml_ids->game_id))) {
				$ids['game_id'] = doubleval(strval($xml_ids->game_id));
			}
			if (doubleval(strval($xml_ids->card_id))) {
				$ids['card_id'] = doubleval(strval($xml_ids->card_id));
			}
			if (doubleval(strval($xml_ids->profile_id))) {
				$ids['profile_id'] = doubleval(strval($xml_ids->profile_id));
			}
			if (doubleval(strval($xml_ids->admin_relationship_id))) {
				$ids['admin_relationship_id'] = doubleval(strval($xml_ids->admin_relationship_id));
			}
			if (doubleval(strval($xml_ids->played_card_id))) {
				$ids['played_card_id'] = doubleval(strval($xml_ids->played_card_id));
			}
			if (doubleval(strval($xml_ids->scenario_id))) {
				$ids['scenario_id'] = doubleval(strval($xml_ids->scenario_id));
			}
			if (doubleval(strval($xml_ids->scenario_condition_id))) {
				$ids['scenario_condition_id'] = doubleval(strval($xml_ids->scenario_condition_id));
			}
			if (doubleval(strval($xml_ids->criteria_id))) {
				$ids['criteria_id'] = doubleval(strval($xml_ids->criteria_id));
			}
			if (doubleval(strval($xml_ids->criteria_condition_id))) {
				$ids['criteria_condition_id'] = doubleval(strval($xml_ids->criteria_condition_id));
			}
			if (doubleval(strval($xml_ids->tag_id))) {
				$ids['tag_id'] = doubleval(strval($xml_ids->tag_id));
			}
			if (doubleval(strval($xml_ids->card_translation_id))) {
				$ids['card_translation_id'] = doubleval(strval($xml_ids->card_translation_id));
			}
			if (doubleval(strval($xml_ids->choice_id))) {
				$ids['choice_id'] = doubleval(strval($xml_ids->choice_id));
			}
			if (doubleval(strval($xml_ids->choice_translation_id))) {
				$ids['choice_translation_id'] = doubleval(strval($xml_ids->choice_translation_id));
			}
			if (doubleval(strval($xml_ids->result_id))) {
				$ids['result_id'] = doubleval(strval($xml_ids->result_id));
			}
			if (doubleval(strval($xml_ids->result_translation_id))) {
				$ids['result_translation_id'] = doubleval(strval($xml_ids->result_translation_id));
			}
			if (doubleval(strval($xml_ids->game_texts_translation_id))) {
				$ids['game_texts_translation_id'] = doubleval(strval($xml_ids->game_texts_translation_id));
			}
		}
		return $ids;
	}
	
	/**
	 * Writes the XML file containing latest used ids of GAL2.
    * @param string $xml_file File path of the latest ids XML file.
    * @param array $ids An array containing latest ids used.
    * @return bool Was the operation successful.
   */
	private function writeXMLLatestIds($xml_file, $ids) {
		
		if ($this->debug) {
			$this->writeDebug('writeXMLLatestIds', '');
		}
		
		$xml_base = <<<XML
<?xml version="1.0" encoding="UTF-8" ?>
<latest_ids>
</latest_ids>
XML;
		$xml_ids = simplexml_load_string($xml_base);
		
		foreach ($ids as $key => $value) {
			$xml_id = $xml_ids->addChild($key, $value);
		}
		
		if ($xml_ids->saveXML($xml_file)) {
			return true;
		}
		return false;
	}
	
	/**
	 * Parses the XML file containing information about texts of a single game.
    * @param string $xml_file File path of the texts XML file.
    * @return array Contains texts of a single game.
   */
	private function readXMLTexts($xml_file) {
		
		if ($this->debug) {
			$this->writeDebug('readXMLTexts', '');
		}
		
		$texts = array();

		$xml_texts = simplexml_load_file($xml_file);

		$game_id = 0;
		if (isset($xml_texts['game_id'])) {
			$game_id = doubleval($xml_texts['game_id']);
		}
		if ($xml_texts) {
			
			if (count($xml_texts->translations->translation) >= 1) {
				
				foreach ($xml_texts->translations->translation as $text) {

					$translation = array();
					$translation['translation_id'] = doubleval(strval($text['id']));
					$translation['game_id'] = $game_id;
					$translation['language_code'] = strval($text['language_code']);
					$translation['is_default'] = intval(strval($text['is_default']));
					$translation['welcome'] = html_entity_decode(strval($text->welcome), ENT_COMPAT,'UTF-8');
					$translation['front_up'] = html_entity_decode(strval($text->front_up), ENT_COMPAT,'UTF-8');
					$translation['front_down'] = html_entity_decode(strval($text->front_down), ENT_COMPAT,'UTF-8');
					$translation['registration_up'] = html_entity_decode(strval($text->registration_up), ENT_COMPAT,'UTF-8');
					$translation['registration_down'] = html_entity_decode(strval($text->registration_down), ENT_COMPAT,'UTF-8');
					$translation['login_up'] = html_entity_decode(strval($text->login_up), ENT_COMPAT,'UTF-8');
					$translation['login_down'] = html_entity_decode(strval($text->login_down), ENT_COMPAT,'UTF-8');
					$translation['new_up'] = html_entity_decode(strval($text->new_up), ENT_COMPAT,'UTF-8');
					$translation['new_down'] = html_entity_decode(strval($text->new_down), ENT_COMPAT,'UTF-8');
					$translation['open_up'] = html_entity_decode(strval($text->open_up), ENT_COMPAT,'UTF-8');
					$translation['open_down'] = html_entity_decode(strval($text->open_down), ENT_COMPAT,'UTF-8');
					$translation['continue_up'] = html_entity_decode(strval($text->continue_up), ENT_COMPAT,'UTF-8');
					$translation['continue_down'] = html_entity_decode(strval($text->continue_down), ENT_COMPAT,'UTF-8');
					$translation['loading'] = html_entity_decode(strval($text->loading), ENT_COMPAT,'UTF-8');
					$translation['your_life'] = html_entity_decode(strval($text->your_life), ENT_COMPAT,'UTF-8');
					$translation['menu'] = html_entity_decode(strval($text->menu), ENT_COMPAT,'UTF-8');
					$translation['selection'] = html_entity_decode(strval($text->selection), ENT_COMPAT,'UTF-8');
					
					$texts[] = $translation;
				}
			}
		}
		return $texts;
	}
	
	/**
	 * Writes the XML file containing texts of a single game.
    * @param string $xml_file File path of the texts XML file.
    * @param array $texts An array containing texts of a single game.
    * @param double $game_id ID of the game of which texts are to be written.
    * @return bool Was the operation successful.
   */
	private function writeXMLTexts($xml_file, $texts, $game_id) {
		
		if ($this->debug) {
			$this->writeDebug('writeXMLTexts', '');
		}
		
		$xml_base = <<<XML
<?xml version="1.0" encoding="UTF-8" ?>
<game_texts>
</game_texts>
XML;
		$xml_texts = simplexml_load_string($xml_base);
		
		$xml_texts->addAttribute('game_id', $game_id);
		
		$translations = $xml_texts->addChild('translations');
		
		foreach ($texts as $text) {
			
			$translation = $translations->addChild('translation');
			$translation->addAttribute('id', $text['translation_id']);
			$translation->addAttribute('is_default', $text['is_default']);
			$translation->addAttribute('language_code', $text['language_code']);
			$translation->addChild('welcome', htmlspecialchars($text['welcome'], ENT_COMPAT,'UTF-8'));
			$translation->addChild('front_up', htmlspecialchars($text['front_up'], ENT_COMPAT,'UTF-8'));
			$translation->addChild('front_down', htmlspecialchars($text['front_down'], ENT_COMPAT,'UTF-8'));
			$translation->addChild('registration_up', htmlspecialchars($text['registration_up'], ENT_COMPAT,'UTF-8'));
			$translation->addChild('registration_down', htmlspecialchars($text['registration_down'], ENT_COMPAT,'UTF-8'));
			$translation->addChild('login_up', htmlspecialchars($text['login_up'], ENT_COMPAT,'UTF-8'));
			$translation->addChild('login_down', htmlspecialchars($text['login_down'], ENT_COMPAT,'UTF-8'));
			$translation->addChild('new_up', htmlspecialchars($text['new_up'], ENT_COMPAT,'UTF-8'));
			$translation->addChild('new_down', htmlspecialchars($text['new_down'], ENT_COMPAT,'UTF-8'));
			$translation->addChild('open_up', htmlspecialchars($text['open_up'], ENT_COMPAT,'UTF-8'));
			$translation->addChild('open_down', htmlspecialchars($text['open_down'], ENT_COMPAT,'UTF-8'));
			$translation->addChild('continue_up', htmlspecialchars($text['continue_up'], ENT_COMPAT,'UTF-8'));
			$translation->addChild('continue_down', htmlspecialchars($text['continue_down'], ENT_COMPAT,'UTF-8'));
			$translation->addChild('loading', htmlspecialchars($text['loading'], ENT_COMPAT,'UTF-8'));
			$translation->addChild('your_life', htmlspecialchars($text['your_life'], ENT_COMPAT,'UTF-8'));
			$translation->addChild('menu', htmlspecialchars($text['menu'], ENT_COMPAT,'UTF-8'));
			$translation->addChild('selection', htmlspecialchars($text['selection'], ENT_COMPAT,'UTF-8'));
		}
		if ($xml_texts->saveXML($xml_file)) {
			return true;
		}
		return false;
	}
	
	/**
	 * Handles authentication of the user.
	 * @param string $username Name of the user.
	 * @param string $password Password of the user.
    * @return int ID of the user if successful.
    * Error codes: -1 - user isn't found, -2 - wrong password
   */
	private function authenticateUser($username, $password) {
		
		if ($this->debug) {
			$this->writeDebug('authenticateUser', '');
		}
		
    	if (empty($this->users)) {
    		$this->initDB();
    	}
    	$password_hash = '';
    	$user_id = 0;
    	
    	// Get password hash and user id
    	foreach ($this->users as $user) {
    		if ($user['username'] == $username) {
    			$user_id = $user['user_id'];
    			$password_hash = $user['password'];
    			break;
    		}
    	}
		// If single password hash isn't found return error -1
    	if ($user_id < 1) {
    		$return_value = -1;
    	}
    	
    	if ($return_value >= 0) {
    		$config = Config::getInstance();
    		if (version_compare($config->php_version, '5.3.7') >= 0) {
    			
    			if (password_verify($password, $password_hash)) {
					// Valid user
    				return $user_id;
				}
				else {
					// Invalid
					$return_value = -2;
				}
			}
			else {
				
				$hasher = new PasswordHash($hash_cost_log2, $hash_portable);
				if ($hasher->CheckPassword($password, $password_hash)) {
					// Valid user
					return $user_id;
				}
				else {
					// Invalid
					$return_value = -2;
				}
				unset($hasher);
			}
    	}
		return $return_value;
	}
	
	/**
	 * Updates tags and stats that are related to card.
	 * @param double $card_id ID of the card to update.
    * @return void
   */
	private function updateCardRelatedTagsAndStats($card_id) {
		
		if ($this->debug) {
			$this->writeDebug('updateCardRelatedTagsAndStats', '');
		}
		if (empty($this->games) || empty($this->users)) {
			$this->initDB();
		}
		$game_id = $this->current_game;
		
		if (empty($this->cards)) {
			$game_id = 0;
			if ($this->current_game > 0) {
				$game_id = $this->current_game;
			}
			elseif (isset($_SESSION['admin']['game']['game_id'])) {
				$game_id = $_SESSION['admin']['game']['game_id'];
			}
			$this->getAllCards($game_id, $lang);
		}
		
		foreach ($this->cards as &$card) {
			
			if ($card['card_id'] == $card_id) {
				
				$card_tags = array();
				$card_stats = array();
				
				foreach ($card['tags'] as &$tag) {
					if ($tag[1] != '#') {
						$tag = '#' . $tag;
					}
					$card_tags[] = $tag;
				}
				foreach ($card['stat_requirements'] as $stat) {
					//$card_stats[] = $stat;
					$parsed_stat = GameEngine::parseStat($stat);
					if ($parsed_stat != false) {
						$parsed_stat['key'] = '@' . $parsed_stat['key'];
						$card_stats[] = $parsed_stat['key'];
					}
				}
				unset($stat);
				unset($tag);
		
				// Get stats and tags from results
				if (isset($card['choices'])) {
					foreach ($card['choices'] as $choice) {
						//$card_results = $this->getChoiceResults($choice['choice_id'], 'en-US');
						foreach ($choice['results'] as $result) {
							$stat_changes = explode("@", $result['stat_changes']);
							if (!empty($stat_changes)) {
								foreach ($stat_changes as $stat) {
									$parsed_stat = GameEngine::parseStat($stat);
									if ($parsed_stat != false) {
										$parsed_stat['key'] = '@' . $parsed_stat['key'];
										// If current stat isn't in array of stats, add it there
										if (!in_array($parsed_stat['key'], $card_stats)) {
											$card_stats[] = $parsed_stat['key'];
										}
									}
								}
							}
							$next_criteria = explode("#", $result['next_criteria']);
							if (!empty($next_criteria)) {
								foreach ($next_criteria as &$crit) {
									$crit = trim($crit);
									if (!empty($crit)) {
										// Check if tag is id
										$pos = strpos($crit, "ID");
										if ($pos !== false) {
											$id = substr($crit, $pos+2);
											if (is_numeric($id)) {
												// and continue if it is
												continue;
											}
										}
										$crit = '#' . $crit;
										// If current tag isn't in array of tags, add it there
										if (!in_array($crit, $card_tags)) {
											$card_tags[] = $crit;
										}
									}
								}
							}
						}
					}
				}
				$related_tags = '';
				$related_stats = '';
				$related_tags = implode(" ", $card_tags);
				$related_tags = trim($related_tags);
				$related_stats = implode(" ", $card_stats);
				$related_stats = trim($related_stats);
				
				$card['related_tags'] = $related_tags;
				$card['related_stats'] = $related_stats;
				
				break;
			}
		}
		
		// Update xml file
		if ($this->writeXMLCards($this->xml_folder . 'games/' . $game_id . '/cards_' . $game_id . '.xml', $this->cards)) {
			$_SESSION['xmldb']['cards'] = $this->cards;
			$this->latest_card = array();
			$_SESSION['xmldb']['latest_card'] = array();
		}
	}
	
}